@@ -98,10 +98,12 @@
 #include "vm_assert.h"
 #include "str.h"
 #include "strutil.h"
+#include "vm_version.h"
 #include "hgfsmounter_version.h"
 
 /* XXX embed_version.h does not currently support Mach-O binaries (OS X). */
 #if defined(linux) || defined(__FreeBSD__)
+#  include "vm_version.h"
 #  include "embed_version.h"
    VM_EMBED_VERSION(HGFSMOUNTER_VERSION_STRING);
 #endif
@@ -548,7 +550,7 @@ ParseDmask(const char *option,         // IN:  option string along with value
    ASSERT(option);
    ASSERT(mountInfo);
 
-   if (ParseMask(option, &mountInfo->fmask)) {
+   if (ParseMask(option, &mountInfo->dmask)) {
       LOG("Setting mount dmask to %o\n", mountInfo->dmask);
       return TRUE;
    }
@@ -592,7 +594,7 @@ ParseTtl(const char *option,        // IN:  option string along with value
    optString = StrUtil_GetNextToken(&idx, option, "=");
    ASSERT(optString);
 
-   if (StrUtil_GetNextIntToken(&ttl, &idx, option, "=") && ttl > 0) {
+   if (StrUtil_GetNextIntToken(&ttl, &idx, option, "=") && ttl >= 0) {
       LOG("Setting maximum attribute TTL to %u\n", ttl);
       mountInfo->ttl = ttl;
       success = TRUE;
