@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (c) 2012-2017,2019, 2023 VMware, Inc. All rights reserved.
+ * Copyright (C) 2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -83,11 +83,11 @@ typedef struct {
     * pretty print.
     */
    char *buffer;
-   uint32 buffLen;        /* available buffer size */
-   uint32 maxNumElems;    /* this limits the number of elements for list print. */
-   uint32 maxStrLen;      /* max number of bytes to print for each string */
+   int32 buffLen;        /* available buffer size */
+   int32 maxNumElems;    /* this limits the number of elements for list print. */
+   int32 maxStrLen;      /* max number of bytes to print for each string */
    FieldIdNameEntry *fieldIdList;   /* array for field ID to name mapping */
-   uint32 fieldIdListLen; /* fieldIdList size */
+   int32 fieldIdListLen; /* fieldIdList size */
 } ClientData;
 
 static const uint64 magic_cookie = 0x4d41474943ULL;   /* 'MAGIC' */
@@ -121,9 +121,7 @@ AddEntry_Int64(DataMap *that,      // IN/OUT
    }
    entry->type = DMFIELDTYPE_INT64;
    entry->value.number.val = value;
-   if (!HashMap_Put(that->map, &key, &entry)) {
-      return DMERR_INSUFFICIENT_MEM;
-   }
+   HashMap_Put(that->map, &key, &entry);
    return DMERR_SUCCESS;
 }
 
@@ -161,9 +159,7 @@ AddEntry_String(DataMap *that,      // IN/OUT
    entry->value.string.str = str;
    entry->value.string.length = strLen;
 
-   if (!HashMap_Put(that->map, &key, &entry)) {
-      return DMERR_INSUFFICIENT_MEM;
-   }
+   HashMap_Put(that->map, &key, &entry);
    return DMERR_SUCCESS;
 }
 
@@ -202,9 +198,7 @@ AddEntry_Int64List(DataMap *that,            // IN/OUT
    entry->value.numList.numbers = numbers;
    entry->value.numList.length = listLen;
 
-   if (!HashMap_Put(that->map, &key, &entry)) {
-      return DMERR_INSUFFICIENT_MEM;
-   }
+   HashMap_Put(that->map, &key, &entry);
    return DMERR_SUCCESS;
 }
 
@@ -248,9 +242,7 @@ AddEntry_StringList(DataMap *that,            // IN/OUT
    entry->value.strList.strings = strList;
    entry->value.strList.lengths = strLens;
 
-   if (!HashMap_Put(that->map, &key, &entry)) {
-      return DMERR_INSUFFICIENT_MEM;
-   }
+   HashMap_Put(that->map, &key, &entry);
    return DMERR_SUCCESS;
 }
 
@@ -518,27 +510,17 @@ DecodeString(char **buf,         // IN/OUT
 
    res = DecodeInt32(buf, left, strLen);
 
-   if (res != DMERR_SUCCESS) {
-      return res;
-   }
+   if (res == DMERR_SUCCESS) {
+      *str = (char *)malloc(*strLen);
+      if (*str == NULL) {
+         return DMERR_INSUFFICIENT_MEM;
+      }
 
-   if (*strLen <= 0) {
-      return DMERR_BAD_DATA;
+      memcpy(*str, *buf, *strLen);
+      *buf += *strLen;
+      *left -= *strLen;
    }
 
-   if (*left < *strLen) {
-      return DMERR_TRUNCATED_DATA;
-   }
-
-   *str = (char *)malloc(*strLen);
-   if (*str == NULL) {
-      return DMERR_INSUFFICIENT_MEM;
-   }
-
-   memcpy(*str, *buf, *strLen);
-   *buf += *strLen;
-   *left -= *strLen;
-
    return res;
 }
 
@@ -604,19 +586,12 @@ DecodeInt64List(char **buf,        // IN/OUT
    int32 listLen;
    int64 *numList = NULL;
    ErrorCode res;
-   int32 i;
 
    res = DecodeInt32(buf, left, &listLen);
-   if (res != DMERR_SUCCESS) {
-      return res;
-   }
 
-   if (listLen < 0 || listLen > *left / sizeof(int64)) {
-      /* listLen can be zero to support an empty list */
-      return DMERR_BAD_DATA;
-   }
+   if (res == DMERR_SUCCESS) {
+      int32 i;
 
-   if (listLen) {
       numList = (int64 *)malloc(sizeof(int64) * listLen);
       if (numList == NULL) {
          return DMERR_INSUFFICIENT_MEM;
@@ -628,15 +603,15 @@ DecodeInt64List(char **buf,        // IN/OUT
             break;
          }
       }
-   }
 
-   if (res == DMERR_SUCCESS) {
-      res = AddEntry_Int64List(that, fieldId, numList, listLen);
-   }
+      if (res == DMERR_SUCCESS) {
+         res = AddEntry_Int64List(that, fieldId, numList, listLen);
+      }
 
-   if (res != DMERR_SUCCESS) {
-      /* clean up memory */
-      free(numList);
+      if (res != DMERR_SUCCESS) {
+         /* clean up memory */
+         free(numList);
+      }
    }
 
    return res;
@@ -791,7 +766,7 @@ DataMap_Create(DataMap *that)    // IN/OUT
 
 static void
 ToBufferUpdate(ClientData *clientData,   // IN/OUT
-               uint32 len)
+               int32 len)
 {
    if (len >= clientData->buffLen) {
       clientData->result = DMERR_BUFFER_TOO_SMALL;
@@ -858,9 +833,9 @@ ToBufferString(ClientData *clientData,   // IN/OUT
 static void
 ToBufferStringN(ClientData *clientData,   // IN/OUT
                 const char *str,          // IN
-                uint32 len)                // IN
+                int32 len)                // IN
 {
-   uint32 copyLen = len;
+   int32 copyLen = len;
 
    if (clientData->result != DMERR_SUCCESS) {
       /* an error occurred already, so stop. */
@@ -927,7 +902,7 @@ static void
 ToBufferInt64(ClientData *clientData,   // IN/OUT
               int64 num)                // IN
 {
-   uint32 len;
+   int32 len;
 
    if (clientData->result != DMERR_SUCCESS) {
       /* an error occurred already, so stop. */
@@ -965,7 +940,7 @@ ToBufferIdType(ClientData *clientData,   // IN/OUT
                DMKeyType fieldId,        // IN
                const char *type)         // IN
 {
-   uint32 len;
+   int32 len;
 
    if (clientData->result != DMERR_SUCCESS) {
       /* an error occurred already, so stop. */
@@ -1074,7 +1049,7 @@ IsPrintable(const char *str,    // IN
 static void
 ToBufferHexString(ClientData *clientData,    // IN/OUT
                   const char *str,           // IN
-                  uint32 strLen)              // IN
+                  int32 strLen)              // IN
 {
    Bool printable;
    int32 len; /* the number of printable chars */
@@ -1273,8 +1248,7 @@ HashMapCalcEntrySizeCb(void *key,            // IN
 {
    DataMapEntry *entry = *((DataMapEntry **)data);
    ClientData *clientData = (ClientData *)userData;
-   uint32 oldLen = clientData->buffLen;
-   uint32 *buffLen = &(clientData->buffLen);
+   int32 *buffLen = &(clientData->buffLen);
 
    if (clientData->result != DMERR_SUCCESS) {
       /* a previous error has occurred, so stop. */
@@ -1315,11 +1289,6 @@ HashMapCalcEntrySizeCb(void *key,            // IN
             *buffLen += sizeof(int32);        /* list size */
 
             for (; *strPtr != NULL; strPtr++, lenPtr++) {
-               if (*buffLen < oldLen) {
-                  clientData->result = DMERR_INTEGER_OVERFLOW;
-                  return;
-               }
-
                *buffLen += sizeof(int32);   /* string length */
                *buffLen += *lenPtr;        /* string payload */
             }
@@ -1331,11 +1300,6 @@ HashMapCalcEntrySizeCb(void *key,            // IN
             return;
          }
    }
-
-   if (*buffLen < oldLen) {
-      clientData->result = DMERR_INTEGER_OVERFLOW;
-      return;
-   }
 }
 
 
@@ -1403,7 +1367,7 @@ HashMapSerializeEntryCb(void *key,            // IN
          ASSERT(0);    /*  we do not expect this to happen */
    }
 
-   /* Update left buffer size so we can do a confidence check at the end. */
+   /* Update left buffer size so we can do a sanity check at the end */
    clientData->buffLen -= (clientData->buffer - buffPtrOrig);
 }
 
@@ -1445,24 +1409,13 @@ DecodeStringList(char **buf,           // IN
       return res;
    }
 
-   if (listSize < 0 || listSize > *left / sizeof(int32)) {
-      /* listSize can be zero to support an empty list */
-      return DMERR_BAD_DATA;
-   }
-
    strList = (char **)calloc(listSize + 1, sizeof(char *));
-   if (strList == NULL) {
+   strLens = (int32 *)malloc(sizeof(int32) * listSize);
+
+   if (strList == NULL || strLens == NULL) {
+      FreeStringList(strList, strLens);
       return DMERR_INSUFFICIENT_MEM;
    }
-   if (listSize) {
-      strLens = (int32 *)malloc(sizeof(int32) * listSize);
-      if (strLens == NULL) {
-         FreeStringList(strList, strLens);
-         return DMERR_INSUFFICIENT_MEM;
-      }
-   } else {
-      strLens = NULL;
-   }
 
    for (i = 0; i < listSize; i++) {
       res = DecodeString(buf, left, &strList[i], &strLens[i]);
@@ -1519,14 +1472,10 @@ CopyStringListEntry(DMKeyType fieldId,                  // IN
    }
 
    newList = (char **)calloc(listSize + 1, sizeof(char *));
-   if (newList == NULL) {
-      return DMERR_INSUFFICIENT_MEM;
-   }
-
    newLens = (int32 *)malloc(sizeof(int32) * listSize);
 
-   if (newLens == NULL) {
-      free(newList);
+   if (newList == NULL || newLens == NULL) {
+      FreeStringList(newList, newLens);
       return DMERR_INSUFFICIENT_MEM;
    }
 
@@ -1723,13 +1672,13 @@ DataMap_Copy(const DataMap *src,  // IN
    ClientData clientData;
    ErrorCode res;
 
+   ASSERT(src->map != NULL);
+   ASSERT(src->cookie == magic_cookie);
+
    if (src == NULL || dst == NULL) {
       return  DMERR_INVALID_ARGS;
    }
 
-   ASSERT(src->map != NULL);
-   ASSERT(src->cookie == magic_cookie);
-
    /* init dst map */
    res = DataMap_Create(dst);
    if (res != DMERR_SUCCESS) {
@@ -1773,7 +1722,7 @@ DataMap_Copy(const DataMap *src,  // IN
 ErrorCode
 DataMap_Serialize(const DataMap *that,     // IN
                   char **buf,              // OUT
-                  uint32 *bufLen)          // OUT
+                  int32 *bufLen)           // OUT
 {
    ClientData clientData;
 
@@ -1790,12 +1739,7 @@ DataMap_Serialize(const DataMap *that,     // IN
       return clientData.result;
    }
 
-   /* 4 bytes is payload length */
-   *bufLen = clientData.buffLen + sizeof(uint32);
-   if (*bufLen < clientData.buffLen) {
-      return DMERR_INTEGER_OVERFLOW;
-   }
-
+   *bufLen = clientData.buffLen + sizeof(int32); /* 4 bytes is payload length */
    *buf = (char *)malloc(*bufLen);
 
    if (*buf == NULL) {
@@ -1812,7 +1756,7 @@ DataMap_Serialize(const DataMap *that,     // IN
 
    HashMap_Iterate(that->map, HashMapSerializeEntryCb, FALSE, &clientData);
 
-   /* confidence check, make sure the buffer size is just used up. */
+   /* sanity check, make sure the buffer size is just used up*/
    ASSERT(clientData.buffLen == 0);
 
    if (clientData.result != DMERR_SUCCESS) {
@@ -2192,7 +2136,7 @@ DataMap_SetStringList(DataMap *that,          // IN/OUT
 {
    DataMapEntry *entry;
 
-   if (that == NULL || strList == NULL || strLens == NULL) {
+   if (that == NULL || strList == NULL || strLens < 0) {
       return DMERR_INVALID_ARGS;
    }
 
@@ -2457,6 +2401,7 @@ DataMap_ToString(const DataMap *that,               // IN
 {
    ClientData clientData;
    char *buffPtr;
+   const char *truncStr = " DATA TRUNCATED!!!\n";
 
    /* This API is for debugging only, so we use hard coded buffer size */
    const int32 maxBuffSize = 10 * 1024;
@@ -2494,13 +2439,11 @@ DataMap_ToString(const DataMap *that,               // IN
 
    ToBufferString(&clientData, "--> End.\n");
 
-   /* confidence check, make sure the buffer is not overflown. */
+   /* sanity check, make sure the buffer is not overflown. */
    ASSERT(clientData.buffLen >= 0);
    ASSERT(buffPtr + maxBuffSize >= clientData.buffer);
 
    if (clientData.result == DMERR_BUFFER_TOO_SMALL) {
-      const char truncStr[] = " DATA TRUNCATED!!!\n";
-
       ASSERT(maxBuffSize > strlen(truncStr));
       Str_Strcpy(buffPtr + maxBuffSize - strlen(truncStr) - 1, truncStr,
 	         strlen(truncStr) + 1);
