@@ -66,6 +66,7 @@
 #include "hostType.h"
 #include "vm_atomic.h"
 #include "fileLock.h"
+#include "userlock.h"
 
 #include "unicodeOperations.h"
 
@@ -259,6 +260,7 @@ File_Unlink(ConstUnicode pathName)  // IN:
    return (FileDeletion(pathName, TRUE) == 0) ? 0 : -1;
 }
 
+
 /*
  *----------------------------------------------------------------------
  *
@@ -284,12 +286,38 @@ File_UnlinkNoFollow(ConstUnicode pathName)  // IN:
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * File_CreateDirectoryEx --
+ *
+ *      Creates the specified directory with the specified permissions.
+ *
+ * Results:
+ *      True if the directory is successfully created, false otherwise.
+ *
+ * Side effects:
+ *      Creates the directory on disk.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+File_CreateDirectoryEx(ConstUnicode pathName,  // IN:
+                       int mask)               // IN:
+{
+   int err = FileCreateDirectory(pathName, mask);
+
+   return err == 0;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
  * File_CreateDirectory --
  *
- *      Creates the specified directory.
+ *      Creates the specified directory with 0777 permissions.
  *
  * Results:
  *      True if the directory is successfully created, false otherwise.
@@ -303,9 +331,7 @@ File_UnlinkNoFollow(ConstUnicode pathName)  // IN:
 Bool
 File_CreateDirectory(ConstUnicode pathName)  // IN:
 {
-   int err = FileCreateDirectory(pathName, 0777);
-
-   return err == 0;
+   return File_CreateDirectoryEx(pathName, 0777);
 }
 
 
@@ -662,6 +688,55 @@ File_IsEmptyDirectory(ConstUnicode pathName)  // IN:
 }
 
 
+/*
+ *----------------------------------------------------------------------------
+ *
+ * File_IsOsfsVolumeEmpty --
+ *
+ *      Check if specified OSFS volume contains no files.
+ *      This method ignore hidden .sf files. *.sf files are VMFS
+ *      metadata files.
+ *
+ *      OSFS based volumes are considered empty even if they
+ *      contain vmfs metadata files. This emptiness can not be
+ *      checked by File_IsEmptyDirectory API (PR 1050328).
+ *
+ * Results:
+ *      Bool - TRUE -> is vmfs empty directory, FALSE -> not an vmfs
+ *      empty directory
+ *
+ * Side effects:
+ *      None
+ *
+ *----------------------------------------------------------------------------
+ */
+
+Bool
+File_IsOsfsVolumeEmpty(ConstUnicode pathName)  // IN:
+{
+   int i, numFiles;
+   Unicode *fileList = NULL;
+   static const char vmfsSystemFilesuffix[] = ".sf";
+   Bool onlyVmfsSystemFilesFound = TRUE;
+
+   numFiles = File_ListDirectory(pathName, &fileList);
+   if (numFiles == -1) {
+      return FALSE;
+   }
+
+   for (i = 0; i < numFiles; i++) {
+      if (!Unicode_EndsWith(fileList[i], vmfsSystemFilesuffix)) {
+         onlyVmfsSystemFilesFound = FALSE;
+         break;
+      }
+   }
+
+   Unicode_FreeList(fileList, numFiles);
+
+   return onlyVmfsSystemFilesFound;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1122,7 +1197,7 @@ File_CopyTree(ConstUnicode srcName,    // IN:
    if (!File_IsDirectory(srcName)) {
       err = Err_Errno();
       Msg_Append(MSGID(File.CopyTree.source.notDirectory)
-                 "The source path '%s' is not a directory.\n\n",
+                 "Source path '%s' is not a directory.",
                  UTF8(srcName));
       Err_SetErrno(err);
       return FALSE;
@@ -1131,7 +1206,7 @@ File_CopyTree(ConstUnicode srcName,    // IN:
    if (!File_IsDirectory(dstName)) {
       err = Err_Errno();
       Msg_Append(MSGID(File.CopyTree.dest.notDirectory)
-                 "The destination path '%s' is not a directory.\n\n",
+                 "Destination path '%s' is not a directory.",
                  UTF8(dstName));
       Err_SetErrno(err);
       return FALSE;
@@ -1371,7 +1446,8 @@ File_Move(ConstUnicode oldFile,  // IN:
 Bool
 File_MoveTree(ConstUnicode srcName,   // IN:
               ConstUnicode dstName,   // IN:
-              Bool overwriteExisting) // IN:
+              Bool overwriteExisting, // IN:
+              Bool *asMove)           // OUT:
 {
    Bool ret = FALSE;
    Bool createdDir = FALSE;
@@ -1379,24 +1455,35 @@ File_MoveTree(ConstUnicode srcName,   // IN:
    ASSERT(srcName);
    ASSERT(dstName);
 
+   if (asMove) {
+      *asMove = FALSE;
+   }
+
    if (!File_IsDirectory(srcName)) {
       Msg_Append(MSGID(File.MoveTree.source.notDirectory)
-                 "The source path '%s' is not a directory.\n\n",
+                 "Source path '%s' is not a directory.",
                  UTF8(srcName));
+
       return FALSE;
    }
 
    if (File_Rename(srcName, dstName) == 0) {
+      if (asMove) {
+         *asMove = TRUE;
+      }
+
       ret = TRUE;
    } else {
       struct stat statbuf;
+
       if (-1 == Posix_Stat(dstName, &statbuf)) {
          int err = Err_Errno();
 
          if (ENOENT == err) {
-            if (!File_CreateDirectoryHierarchy(dstName)) {
+            if (!File_CreateDirectoryHierarchy(dstName, NULL)) {
                Msg_Append(MSGID(File.MoveTree.dst.couldntCreate)
                           "Could not create '%s'.\n\n", UTF8(dstName));
+
                return FALSE;
             }
 
@@ -1405,6 +1492,7 @@ File_MoveTree(ConstUnicode srcName,   // IN:
             Msg_Append(MSGID(File.MoveTree.statFailed)
                        "%d:Failed to stat destination '%s'.\n\n",
                        err, UTF8(dstName));
+
             return FALSE;
          }
       } else {
@@ -1412,12 +1500,41 @@ File_MoveTree(ConstUnicode srcName,   // IN:
             Msg_Append(MSGID(File.MoveTree.dest.notDirectory)
                        "The destination path '%s' is not a directory.\n\n",
                        UTF8(dstName));
+
             return FALSE;
          }
       }
 
+#if !defined(__FreeBSD__) && !defined(sun)
+      /*
+       * File_GetFreeSpace is not defined for FreeBSD
+       */
+      if (createdDir) {
+         /*
+          * Check for free space on destination filesystem.
+          * We only check for free space if the destination directory
+          * did not exist. In this case, we will not be overwriting any existing
+          * paths, so we need as much space as srcName.
+          */
+         int64 srcSize;
+         int64 freeSpace;
+         srcSize = File_GetSizeEx(srcName);
+         freeSpace = File_GetFreeSpace(dstName, TRUE);
+         if (freeSpace < srcSize) {
+            Unicode spaceStr = Msg_FormatSizeInBytes(srcSize);
+            Msg_Append(MSGID(File.MoveTree.dst.insufficientSpace)
+                  "There is not enough space in the file system to "
+                  "move the directory tree. Free %s and try again.",
+                  spaceStr);
+            free(spaceStr);
+            return FALSE;
+         }
+      }
+#endif
+
       if (File_CopyTree(srcName, dstName, overwriteExisting, FALSE)) {
          ret = TRUE;
+
          if (!File_DeleteDirectoryTree(srcName)) {
             Msg_Append(MSGID(File.MoveTree.cleanupFailed)
                        "Forced to copy '%s' into '%s' but unable to remove "
@@ -1640,13 +1757,23 @@ File_GetSizeByPath(ConstUnicode pathName)  // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * File_CreateDirectoryHierarchy --
+ * File_CreateDirectoryHierarchyEx --
  *
  *      Create a directory including any parents that don't already exist.
+ *      All the created directories are tagged with the specified permission.
+ *      Returns the topmost directory which was created, to allow calling code
+ *      to remove it after in case later operations fail.
  *
  * Results:
  *      TRUE on success, FALSE on failure.
  *
+ *      If topmostCreated is not NULL, it returns the result of the hierarchy
+ *      creation. If no directory was created, *topmostCreated is set to NULL.
+ *      Otherwise *topmostCreated is set to the topmost directory which was
+ *      created. *topmostCreated is set even in case of failure.
+ *
+ *      The caller most Unicode_Free the resulting string.
+ *
  * Side effects:
  *      Only the obvious.
  *
@@ -1654,12 +1781,18 @@ File_GetSizeByPath(ConstUnicode pathName)  // IN:
  */
 
 Bool
-File_CreateDirectoryHierarchy(ConstUnicode pathName)  // IN:
+File_CreateDirectoryHierarchyEx(ConstUnicode pathName,   // IN:
+                                int mask,                // IN
+                                Unicode *topmostCreated) // OUT:
 {
    Unicode volume;
    UnicodeIndex index;
    UnicodeIndex length;
 
+   if (topmostCreated != NULL) {
+      *topmostCreated = NULL;
+   }
+
    if (pathName == NULL) {
       return TRUE;
    }
@@ -1694,22 +1827,67 @@ File_CreateDirectoryHierarchy(ConstUnicode pathName)  // IN:
 
       index = FileFirstSlashIndex(pathName, index + 1);
 
-      if (index == UNICODE_INDEX_NOT_FOUND) {
-         break;
+      temp = Unicode_Substr(pathName, 0, (index == UNICODE_INDEX_NOT_FOUND) ? -1 : index);
+
+      if (File_IsDirectory(temp)) {
+         failed = FALSE;
+      } else {
+         failed = !File_CreateDirectoryEx(temp, mask);
+         if (!failed && topmostCreated != NULL && *topmostCreated == NULL) {
+            *topmostCreated = temp;
+            temp = NULL;
+         }
       }
 
-      temp = Unicode_Substr(pathName, 0, index);
-
-      failed = !File_IsDirectory(temp) && !File_CreateDirectory(temp);
-
       Unicode_Free(temp);
 
       if (failed) {
          return FALSE;
       }
+
+      if (index == UNICODE_INDEX_NOT_FOUND) {
+         break;
+      }
+
    }
 
-   return File_IsDirectory(pathName) || File_CreateDirectory(pathName);
+   return TRUE;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * File_CreateDirectoryHierarchy --
+ *
+ *      Create a directory including any parents that don't already exist.
+ *      All the created directories are tagged with 0777 permissions.
+ *      Returns the topmost directory which was created, to allow calling code
+ *      to remove it after in case later operations fail.
+ *
+ * Results:
+ *      TRUE on success, FALSE on failure.
+ *
+ *      If topmostCreated is not NULL, it returns the result of the hierarchy
+ *      creation. If no directory was created, *topmostCreated is set to NULL.
+ *      Otherwise *topmostCreated is set to the topmost directory which was
+ *      created. *topmostCreated is set even in case of failure.
+ *
+ *      The caller most Unicode_Free the resulting string.
+ *
+ * Side effects:
+ *      Only the obvious.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+File_CreateDirectoryHierarchy(ConstUnicode pathName,   // IN:
+                              Unicode *topmostCreated) // OUT:
+{
+   return File_CreateDirectoryHierarchyEx(pathName,
+                                          0777,
+                                          topmostCreated);
 }
 
 
@@ -1870,9 +2048,11 @@ File_FindFileInSearchPath(const char *fileIn,      // IN:
    char *cur;
    char *tok;
    Bool found;
+   Bool full;
    char *saveptr = NULL;
    char *sp = NULL;
-   char *file = NULL;
+   Unicode dir = NULL;
+   Unicode file = NULL;
 
    ASSERT(fileIn);
    ASSERT(cwd);
@@ -1882,7 +2062,8 @@ File_FindFileInSearchPath(const char *fileIn,      // IN:
     * First check the usual places - the fullpath or the cwd.
     */
 
-   if (File_IsFullPath(fileIn)) {
+   full = File_IsFullPath(fileIn);
+   if (full) {
       cur = Util_SafeStrdup(fileIn);
    } else {
       cur = Str_SafeAsprintf(NULL, "%s%s%s", cwd, DIRSEPS, fileIn);
@@ -1898,13 +2079,24 @@ File_FindFileInSearchPath(const char *fileIn,      // IN:
    free(cur);
    cur = NULL;
 
+   if (full) {
+      goto done;
+   }
+
+   File_GetPathName(fileIn, &dir, &file);
+
+   /*
+    * Search path applies only if filename is simple basename.
+    */
+   if (Unicode_LengthInCodePoints(dir) != 0) {
+      goto done;
+   }
+
    /*
     * Didn't find it in the usual places so strip it to its bare minimum and
     * start searching.
     */
 
-   File_GetPathName(fileIn, NULL, &file);
-
    sp = Util_SafeStrdup(searchPath);
    tok = strtok_r(sp, FILE_SEARCHPATHTOKEN, &saveptr);
 
@@ -1955,7 +2147,8 @@ done:
    }
 
    free(sp);
-   free(file);
+   Unicode_Free(dir);
+   Unicode_Free(file);
 
    return found;
 }
@@ -2006,9 +2199,6 @@ File_ExpandAndCheckDir(const char *dirName)  // IN:
  *
  *      Return a random number in the range of 0 and 2^32-1.
  *
- *      This isn't thread safe but it's more than good enough for the
- *      purposes required of it.
- *
  * Results:
  *      Random number is returned.
  *
@@ -2021,38 +2211,35 @@ File_ExpandAndCheckDir(const char *dirName)  // IN:
 uint32
 FileSimpleRandom(void)
 {
-   static Atomic_Ptr atomic; /* Implicitly initialized to NULL. --mbellon */
-   rqContext *context;
+   static Atomic_Ptr lckStorage;
+   static rqContext *context = NULL;
+   uint32 result;
+   MXUserExclLock *lck = MXUser_CreateSingletonExclLock(&lckStorage,
+                                                        "fileSimpleRandomLock",
+                                                        RANK_LEAF);
 
-   context = Atomic_ReadPtr(&atomic);
+   ASSERT_NOT_IMPLEMENTED(lck != NULL);
+
+   MXUser_AcquireExclLock(lck);
 
    if (UNLIKELY(context == NULL)) {
-      rqContext *newContext;
       uint32 value;
 
-      /*
-       * Threads will hash up this RNG - this isn't officially thread safe
-       * which is just fine - but ensure that different processes have
-       * different answer streams.
-       */
-
 #if defined(_WIN32)
       value = GetCurrentProcessId();
 #else
       value = getpid();
 #endif
 
-      newContext = Random_QuickSeed(value);
-
-      if (Atomic_ReadIfEqualWritePtr(&atomic, NULL, (void *) newContext)) {
-         free(newContext);
-      }
-
-      context = Atomic_ReadPtr(&atomic);
+      context = Random_QuickSeed(value);
       ASSERT(context);
    }
 
-   return Random_Quick(context);
+   result = Random_Quick(context);
+
+   MXUser_ReleaseExclLock(lck);
+
+   return result;
 }
 
 
@@ -2134,6 +2321,10 @@ FileRotateByRename(const char *fileName,  // IN: full path to file
    int i;
    int result;
 
+   if (newFileName != NULL) {
+      *newFileName = NULL;
+   }
+
    for (i = n; i >= 0; i--) {
       src = (i == 0) ? (char *) fileName :
                        Str_SafeAsprintf(NULL, "%s-%d%s", baseName, i - 1, ext);
@@ -2158,8 +2349,8 @@ FileRotateByRename(const char *fileName,  // IN: full path to file
          }
       }
 
-      if ((src == fileName) && (newFileName != NULL)) {
-         *newFileName = result == -1 ? NULL : strdup(dst);
+      if ((src == fileName) && (newFileName != NULL) && (result == 0)) {
+         *newFileName = Util_SafeStrdup(dst);
       }
 
       ASSERT(dst != fileName);
@@ -2232,6 +2423,10 @@ FileRotateByRenumber(const char *filePath,       // IN: full path to file
    uint32 *fileNumbers = NULL;
    int result;
 
+   if (newFilePath != NULL) {
+      *newFilePath = NULL;
+   }
+
    fullPathNoExt = File_FullPath(filePathNoExt);
    if (fullPathNoExt == NULL) {
       Log(LGPFX" %s: failed to get full path for '%s'.\n", __FUNCTION__,
@@ -2295,7 +2490,6 @@ FileRotateByRenumber(const char *filePath,       // IN: full path to file
 
    if (newFilePath != NULL) {
       if (result == -1) {
-         *newFilePath = NULL;
          free(tmp);
       } else {
          *newFilePath = tmp;
