@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -144,57 +144,8 @@ static Bool CompareKeys(struct HashMap *map, const void *key, const void *compar
 static Bool NeedsResize(struct HashMap *map);
 static void Resize(struct HashMap *map);
 INLINE void EnsureSanity(HashMap *map);
+static INLINE Bool CheckSanity(HashMap *map);
 
-/*
- * ----------------------------------------------------------------------------
- *
- * CheckSanity --
- *
- *   Same code as EnsureSanity except return Bool instead of ASSERTing
- *
- * Results:
- *    TRUE is sane.
- *
- * Side Effects:
- *    None.
- *
- * ----------------------------------------------------------------------------
- */
-
-static INLINE Bool
-CheckSanity(HashMap *map)
-{
-#ifdef VMX86_DEBUG
-   uint32 i, cnt = 0;
-
-   ASSERT(map);
-   for (i = 0; i < map->numEntries; i++) {
-      HashMapEntryHeader *header = NULL;
-      void *key, *data;
-
-      GetEntry(map, i, &header, &key, &data);
-      ASSERT(header);
-      ASSERT(header->state == HashMapState_FILLED ||
-             header->state == HashMapState_EMPTY ||
-             header->state == HashMapState_DELETED);
-      if (header->state == HashMapState_FILLED) {
-         cnt++;
-         if (header->hash != ComputeHash(map, key)) {
-            return FALSE;
-         }
-      }
-   }
-
-   if (cnt != map->count) {
-      return FALSE;
-   }
-
-   if (!map->numEntries) {
-      return FALSE;
-   }
-#endif
-   return TRUE;
-}
 
 /*
  * ----------------------------------------------------------------------------
@@ -347,12 +298,11 @@ InitMap(struct HashMap *map,     // IN
  *
  * HashMap_Put --
  *
- *    Put the value at data against the key.  This will replace any existing
+ *    The the value at data against the key.  This will replace any existing
  *    data that is in the table without warning.
  *
  * Results:
- *    TRUE if the put operation is successful
- *    FALSE otherwise
+ *    None.
  *
  * Side Effects:
  *    The value in data is copied to the table and can be referenced in the
@@ -361,7 +311,7 @@ InitMap(struct HashMap *map,     // IN
  * ----------------------------------------------------------------------------
  */
 
-Bool
+void
 HashMap_Put(struct HashMap *map,    // IN
             const void *key,        // IN
             const void *data)       // IN
@@ -386,7 +336,7 @@ HashMap_Put(struct HashMap *map,    // IN
             /*
              * The resize must have failed.
              */
-            return FALSE;
+            return;
          }
       }
 
@@ -403,7 +353,6 @@ HashMap_Put(struct HashMap *map,    // IN
    memcpy(tableData, data, map->dataSize);
 
    EnsureSanity(map);
-   return TRUE;
 }
 
 
@@ -538,7 +487,6 @@ HashMap_Count(struct HashMap *map) // IN
    return map->count;
 }
 
-
 #ifdef VMX86_SERVER
 /*
  * ----------------------------------------------------------------------------
@@ -555,8 +503,7 @@ HashMap_Count(struct HashMap *map) // IN
  *
  * ----------------------------------------------------------------------------
  */
-
-uint64
+uint64 
 HashMap_Store(struct HashMap *map,       // IN
               void           *h,         // IN
               uint64    startByteOffset) // IN
@@ -575,7 +522,7 @@ HashMap_Store(struct HashMap *map,       // IN
    hashMapOnDisk.entrySize = map->entrySize;
    hashMapOnDisk.keyOffset = map->keyOffset;
    hashMapOnDisk.dataOffset = map->dataOffset;
-
+   
    /*
     * write map
     */
@@ -583,9 +530,12 @@ HashMap_Store(struct HashMap *map,       // IN
    vec.iov_len = sizeof hashMapOnDisk;
    numBytes = vec.iov_len;
 
-   aioErr = AIOMgr_Queue(*handle, &vec, 1, OP_WRITE, startByteOffset, numBytes,
-                         AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL);
-   ASSERT(!AIOMgr_IsAsync(aioErr));
+   aioErr = AIOMgr_Queue(*handle, &vec, 1, OP_WRITE,
+                         startByteOffset, numBytes, AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL );
+   if (AIOMgr_IsAsync(aioErr)) {
+      aioErr = AIOMgr_Sync(*handle);
+   }
+
    if (!AIOMgr_IsSuccess(aioErr)) {
       return -1;
    }
@@ -597,9 +547,11 @@ HashMap_Store(struct HashMap *map,       // IN
    vec.iov_len = map->numEntries * map->entrySize;
 
    aioErr = AIOMgr_Queue(*handle, &vec, 1, OP_WRITE,
-                         startByteOffset + numBytes, vec.iov_len,
-                         AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL);
-   ASSERT(!AIOMgr_IsAsync(aioErr));
+                        startByteOffset + numBytes, vec.iov_len, AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL );
+   if (AIOMgr_IsAsync(aioErr)) {
+      aioErr = AIOMgr_Sync(*handle);
+   }
+
    if (!AIOMgr_IsSuccess(aioErr)) {
       return -1;
    }
@@ -645,10 +597,11 @@ HashMap_Retrieve(void      *h,         // IN
    vec.iov_len = sizeof hashMapOnDisk;
    numBytes = vec.iov_len;
    if (handle) {
-      aioErr = AIOMgr_Queue(*handle, &vec, 1, OP_READ, startByteOffset,
-                            numBytes, AIOMGR_INVALID_IO_REQUEST_ID, NULL,
-                            NULL);
-      ASSERT(!AIOMgr_IsAsync(aioErr));
+      aioErr = AIOMgr_Queue(*handle, &vec, 1, OP_READ,
+                        startByteOffset, numBytes, AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL );
+      if (AIOMgr_IsAsync(aioErr)) {
+         aioErr = AIOMgr_Sync(*handle);
+      }
    }
 
    if (!AIOMgr_IsSuccess(aioErr)) {
@@ -682,9 +635,11 @@ HashMap_Retrieve(void      *h,         // IN
    vec.iov_len = map->numEntries * map->entrySize;
    ASSERT(handle);
    aioErr = AIOMgr_Queue(*handle, &vec, 1, OP_READ,
-                         startByteOffset + numBytes, vec.iov_len,
-                         AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL);
-   ASSERT(!AIOMgr_IsAsync(aioErr));
+                     startByteOffset + numBytes, vec.iov_len, AIOMGR_INVALID_IO_REQUEST_ID, NULL, NULL );
+   if (AIOMgr_IsAsync(aioErr)) {
+      aioErr = AIOMgr_Sync(*handle);
+   }
+
    if (!AIOMgr_IsSuccess(aioErr)) {
       free(map->entries);
       free(map);
@@ -697,6 +652,7 @@ HashMap_Retrieve(void      *h,         // IN
       free(map);
       return NULL;
    }
+
    return map;
 }
 
@@ -717,7 +673,7 @@ HashMap_Retrieve(void      *h,         // IN
  *
  * ----------------------------------------------------------------------------
  */
-uint64
+uint64 
 HashMap_Store(struct HashMap *map,       // IN
               void           *h,         // IN
               uint64    startByteOffset) // IN
@@ -1183,8 +1139,61 @@ HashMap_Iterate(HashMap *map,             // IN
 void
 EnsureSanity(HashMap *map)
 {
+#ifdef VMX86_DEBUG
    ASSERT(CheckSanity(map) == TRUE);
+#endif
 }
 
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CheckSanity --
+ *
+ *   Same code as EnsureSanity except return Bool instead of ASSERTing
+ *
+ * Results:
+ *    TRUE is sane.
+ *
+ * Side Effects:
+ *    None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+static Bool
+CheckSanity(HashMap *map)
+{
+#ifdef VMX86_DEBUG
+   uint32 i, cnt = 0;
+
+   ASSERT(map);
+   for (i = 0; i < map->numEntries; i++) {
+      HashMapEntryHeader *header = NULL;
+      void *key, *data;
+
+      GetEntry(map, i, &header, &key, &data);
+      ASSERT(header);
+      ASSERT(header->state == HashMapState_FILLED ||
+             header->state == HashMapState_EMPTY ||
+             header->state == HashMapState_DELETED);
+      if (header->state == HashMapState_FILLED) {
+         cnt++;
+         if (header->hash != ComputeHash(map, key)) {
+            return FALSE;
+         }
+      }
+   }
+
+   if (cnt != map->count) {
+      return FALSE;
+   }
+
+   if (!map->numEntries) {
+      return FALSE;
+   }
+#endif
+   return TRUE;
+}
+
 
