@@ -60,19 +60,19 @@ static Bool HgfsChannelGuestBdInit(HgfsServerSessionCallbacks *serverCBTable,
 static void HgfsChannelGuestBdExit(HgfsGuestConn *data);
 static Bool HgfsChannelGuestBdSend(void *data,
                                    HgfsPacket *packet,
-                                   char *buffer,
-                                   size_t bufferLen,
                                    HgfsSendFlags flags);
 static Bool HgfsChannelGuestBdReceive(HgfsGuestConn *data,
                                       char const *packetIn,
                                       size_t packetInSize,
                                       char *packetOut,
                                       size_t *packetOutSize);
+static uint32 HgfsChannelGuestBdInvalidateInactiveSessions(HgfsGuestConn *data);
 
 HgfsGuestChannelCBTable gGuestBackdoorOps = {
    HgfsChannelGuestBdInit,
    HgfsChannelGuestBdExit,
    HgfsChannelGuestBdReceive,
+   HgfsChannelGuestBdInvalidateInactiveSessions,
 };
 
 /* Private functions. */
@@ -342,6 +342,10 @@ static Bool
 HgfsChannelGuestConnConnect(HgfsGuestConn *connData)  // IN: our connection data
 {
    Bool result;
+   static HgfsServerChannelData HgfsBdCapData = {
+      0,
+      HGFS_LARGE_PACKET_MAX
+   };
 
    connData->channelCbTable.getWriteVa = NULL;
    connData->channelCbTable.getReadVa = NULL;
@@ -349,7 +353,7 @@ HgfsChannelGuestConnConnect(HgfsGuestConn *connData)  // IN: our connection data
    connData->channelCbTable.send = HgfsChannelGuestBdSend;
    result = connData->serverCbTable->connect(connData,
                                              &connData->channelCbTable,
-                                             0,
+                                             &HgfsBdCapData,
                                              &connData->serverSession);
    if (result) {
       HgfsChannelGuestConnGet(connData);
@@ -479,9 +483,11 @@ HgfsChannelGuestReceiveInternal(HgfsGuestConn *connData,  // IN: connection
    packet.iov[0].len = packetInSize;
    packet.iovCount = 1;
    packet.metaPacket = (void *)packetIn;
+   packet.metaPacketDataSize = packetInSize;
    packet.metaPacketSize = packetInSize;
    packet.replyPacket = packetOut;
    packet.replyPacketSize = *packetOutSize;
+   packet.state |= HGFS_STATE_CLIENT_REQUEST;
 
    /* The server will perform a synchronous processing of requests. */
    connData->serverCbTable->receive(&packet, connData->serverSession);
@@ -567,6 +573,45 @@ exit:
 }
 
 
+/*
+ *----------------------------------------------------------------------------
+ *
+ * HgfsChannelGuestBdInvalidateInactiveSessions --
+ *
+ *    Sends a request to invalidate all the inactive HGFS server sessions.
+ *
+ * Results:
+ *    Number of active sessions remaining inside the HGFS server.
+ *
+ * Side effects:
+ *    None
+ *
+ *----------------------------------------------------------------------------
+ */
+
+uint32
+HgfsChannelGuestBdInvalidateInactiveSessions(HgfsGuestConn *connData)  // IN: connection
+{
+   ASSERT(NULL != connData);
+
+   if (NULL == connData) {
+      return 0;
+   }
+
+   if (connData->state == HGFS_GST_CONN_UNINITIALIZED) {
+      /* The connection was closed as we are exiting, so bail. */
+      return 0;
+   }
+
+   /* The server will perform a synchronous processing of requests. */
+   if (connData->serverSession) {
+      return connData->serverCbTable->invalidateInactiveSessions(connData->serverSession);
+   }
+
+   return 0;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -586,28 +631,20 @@ exit:
 static Bool
 HgfsChannelGuestBdSend(void *conn,              // IN: our connection data
                        HgfsPacket *packet,      // IN/OUT: Hgfs Packet
-                       char *buffer,            // IN: buffer to be sent
-                       size_t bufferLen,        // IN: buffer length
                        HgfsSendFlags flags)     // IN: Flags to say how to process
 {
    HgfsGuestConn *connData = conn;
-   unsigned char *packetOut = &connData->packetOut[0];
 
    ASSERT(NULL != connData);
    ASSERT(NULL != packet);
-   ASSERT(NULL != buffer);
-   ASSERT(bufferLen <= HGFS_LARGE_PACKET_MAX &&
-          bufferLen <= packet->replyPacketSize);
+   ASSERT(NULL != packet->replyPacket);
+   ASSERT(packet->replyPacketDataSize <= connData->packetOutLen);
+   ASSERT(packet->replyPacketSize == connData->packetOutLen);
 
-   if (connData->clientPacketOut != NULL) {
-      /* Client passed us an out buffer so use it. */
-      packetOut = connData->clientPacketOut;
+   if (packet->replyPacketDataSize > connData->packetOutLen) {
+      packet->replyPacketDataSize = connData->packetOutLen;
    }
-   ASSERT(bufferLen <= connData->packetOutLen);
-   if (bufferLen > connData->packetOutLen) {
-      bufferLen = connData->packetOutLen;
-   }
-   connData->packetOutLen = (uint32)bufferLen;
+   connData->packetOutLen = (uint32)packet->replyPacketDataSize;
 
    if (!(flags & HGFS_SEND_NO_COMPLETE)) {
       connData->serverCbTable->sendComplete(packet,
