@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2003-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2003 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -65,27 +65,6 @@
 #define ASOCKERR_BIND              10
 #define ASOCKERR_BINDADDRINUSE     11
 #define ASOCKERR_LISTEN            12
-#define ASOCKERR_CONNECTSSL        13
-
-
-/*
- * Websocket close status codes --
- *
- * enum has numbers in names because RFC6455 refers to the numbers frequently.
- */
-enum {
-   WEB_SOCKET_CLOSE_STATUS_1000_NORMAL = 1000,
-   WEB_SOCKET_CLOSE_STATUS_1001_GOING_AWAY = 1001,
-   WEB_SOCKET_CLOSE_STATUS_1002_PROTOCOL_ERROR = 1002,
-   WEB_SOCKET_CLOSE_STATUS_1003_INVALID_DATA = 1003,
-   WEB_SOCKET_CLOSE_STATUS_1005_EMPTY = 1005,
-   WEB_SOCKET_CLOSE_STATUS_1006_ABNORMAL = 1006,
-   WEB_SOCKET_CLOSE_STATUS_1007_INCONSISTENT_DATA = 1007,
-   WEB_SOCKET_CLOSE_STATUS_1008_POLICY_VIOLATION = 1008,
-   WEB_SOCKET_CLOSE_STATUS_1009_MESSAGE_TOO_BIG = 1009,
-   WEB_SOCKET_CLOSE_STATUS_1010_UNSUPPORTED_EXTENSIONS = 1010,
-   WEB_SOCKET_CLOSE_STATUS_1015_TLS_HANDSHAKE_ERROR = 1015,
-};
 
 /*
  * Flags passed into AsyncSocket_Connect*().
@@ -110,6 +89,11 @@ typedef enum {
 struct SSLSockStruct;
 struct _SSLVerifyParam;
 
+/*
+ * sockaddr type declaration (so we don't have to include winsock2.h, etc)
+ */
+struct sockaddr_in;
+
 /*
  * AsyncSocket type is opaque
  */
@@ -164,7 +148,7 @@ int AsyncSocket_GetGenericErrno(AsyncSocket *s);
 int AsyncSocket_GetID(AsyncSocket *asock);
 
 /*
- * Return the fd corresponding to the socket.
+ * Return the fd corresponding to the connection
  */
 int AsyncSocket_GetFd(AsyncSocket *asock);
 
@@ -179,15 +163,14 @@ int AsyncSocket_GetLocalVMCIAddress(AsyncSocket *asock,
 int AsyncSocket_GetRemoteVMCIAddress(AsyncSocket *asock,
                                      uint32 *cid, uint32 *port);
 
-int AsyncSocket_GetINETIPStr(AsyncSocket *asock, int socketFamily,
-                             char **ipRetStr);
-unsigned int AsyncSocket_GetPort(AsyncSocket *asock);
-
 /*
  * Recv callback fires once previously requested data has been received
  */
 typedef void (*AsyncSocketRecvFn) (void *buf, int len, AsyncSocket *asock,
                                    void *clientData);
+typedef void (*AsyncSocketRecvUDPFn) (void *buf, int len, AsyncSocket *asock,
+                                      void *clientData, struct sockaddr_in *sin,
+                                      int sin_len);
 
 /*
  * Send callback fires once previously queued data has been sent
@@ -209,53 +192,69 @@ typedef void (*AsyncSocketSslAcceptFn) (Bool status, AsyncSocket *asock,
 /*
  * Listen on port and fire callback with new asock
  */
-AsyncSocket *AsyncSocket_Listen(const char *addrStr,
-                                unsigned int port,
+AsyncSocket *AsyncSocket_Listen(unsigned short port,
                                 AsyncSocketConnectFn connectFn,
                                 void *clientData,
                                 AsyncSocketPollParams *pollParams,
-                                int *outError);
-AsyncSocket *AsyncSocket_ListenLoopback(unsigned int port,
-                                        AsyncSocketConnectFn connectFn,
-                                        void *clientData,
-                                        AsyncSocketPollParams *pollParams,
-                                        int *outError);
+                                int *error);
+AsyncSocket *AsyncSocket_ListenIP(unsigned int ip,
+                                  unsigned short port,
+                                  AsyncSocketConnectFn connectFn,
+                                  void *clientData,
+                                  AsyncSocketPollParams *pollParams,
+                                  int *error);
+AsyncSocket *AsyncSocket_ListenIPStr(const char *ipStr,
+                                     unsigned short port,
+                                     AsyncSocketConnectFn connectFn,
+                                     void *clientData,
+                                     AsyncSocketPollParams *pollParams,
+                                     int *error);
 AsyncSocket *AsyncSocket_ListenVMCI(unsigned int cid,
                                     unsigned int port,
                                     AsyncSocketConnectFn connectFn,
                                     void *clientData,
                                     AsyncSocketPollParams *pollParams,
-                                    int *outError);
-#ifndef VMX86_TOOLS
-AsyncSocket *AsyncSocket_ListenWebSocket(const char *addrStr,
+                                    int *error);
+AsyncSocket *AsyncSocket_ListenWebSocket(const char *ipStr,
                                          unsigned int port,
                                          Bool useSSL,
-                                         const char *protocols[],
                                          AsyncSocketConnectFn connectFn,
                                          void *clientData,
                                          AsyncSocketPollParams *pollParams,
                                          int *outError);
-#ifndef _WIN32
-AsyncSocket *AsyncSocket_ListenWebSocketUDS(const char *pipeName,
-                                            Bool useSSL,
-                                            const char *protocols[],
-                                            AsyncSocketConnectFn connectFn,
-                                            void *clientData,
-                                            AsyncSocketPollParams *pollParams,
-                                            int *outError);
-#endif
-#endif
+AsyncSocket *AsyncSocket_ListenWebSocketIP(const int32 ipAddr,
+                                           unsigned int port,
+                                           Bool useSSL,
+                                           AsyncSocketConnectFn connectFn,
+                                           void *clientData,
+                                           AsyncSocketPollParams *pollParams,
+                                           int *outError);
 
 /*
  * Connect to address:port and fire callback with new asock
  */
-AsyncSocket *AsyncSocket_Connect(const char *hostname,
-                                 unsigned int port,
+AsyncSocket *AsyncSocket_Connect(int socketFamily,
+                                 const char *hostname,
+                                 unsigned short port,
                                  AsyncSocketConnectFn connectFn,
                                  void *clientData,
                                  AsyncSocketConnectFlags flags,
                                  AsyncSocketPollParams *pollParams,
                                  int *error);
+AsyncSocket *AsyncSocket_ConnectIP(unsigned int ip,
+                                   unsigned short port,
+                                   AsyncSocketConnectFn connectFn,
+                                   void *clientData,
+                                   AsyncSocketConnectFlags flags,
+                                   AsyncSocketPollParams *pollParams,
+                                   int *error);
+AsyncSocket *AsyncSocket_ConnectIPStr(const char *ipStr,
+                                      unsigned short port,
+                                      AsyncSocketConnectFn connectFn,
+                                      void *clientData,
+                                      AsyncSocketConnectFlags flags,
+                                      AsyncSocketPollParams *pollParams,
+                                      int *error);
 AsyncSocket *AsyncSocket_ConnectVMCI(unsigned int cid, unsigned int port,
                                      AsyncSocketConnectFn connectFn,
                                      void *clientData,
@@ -278,29 +277,22 @@ AsyncSocket_ConnectNamedPipe(char *pipeName,
                              AsyncSocketPollParams *pollParams,
                              int *outError);
 #endif
+AsyncSocket *AsyncSocket_ConnectWebSocket(int socketFamily,
+                                          const char *url,
+                                          Bool permitUnverifiedSSL,
+                                          const char *cookies,
+                                          AsyncSocketConnectFn connectFn,
+                                          void *clientData,
+                                          AsyncSocketConnectFlags flags,
+                                          AsyncSocketPollParams *pollParams,
+                                          int *error);
 
-#ifndef VMX86_TOOLS
-AsyncSocket *
-AsyncSocket_ConnectWebSocket(const char *url,
-                             struct _SSLVerifyParam *sslVerifyParam,
-                             const char *cookies,
-                             const char *protocols[],
-                             AsyncSocketConnectFn connectFn,
-                             void *clientData,
-                             AsyncSocketConnectFlags flags,
-                             AsyncSocketPollParams *pollParams,
-                             int *error);
-#endif
-
-#ifndef USE_SSL_DIRECT
 /*
  * Initiate SSL connection on existing asock, with optional cert verification
  */
 Bool AsyncSocket_ConnectSSL(AsyncSocket *asock,
-                            struct _SSLVerifyParam *verifyParam,
-                            void *sslContext);
+                            struct _SSLVerifyParam *verifyParam);
 Bool AsyncSocket_AcceptSSL(AsyncSocket *asock);
-#endif
 
 void AsyncSocket_StartSslAccept(AsyncSocket *asock,
                                 void *sslCtx,
@@ -418,7 +410,6 @@ Bool AsyncSocket_SetBufferSizes(AsyncSocket *asock,  // IN
  */
 int AsyncSocket_Close(AsyncSocket *asock);
 
-#ifndef VMX86_TOOLS
 /*
  * Retrieve the URI Supplied for a websocket connection
  */
@@ -428,24 +419,6 @@ char *AsyncSocket_GetWebSocketURI(AsyncSocket *asock);
  * Retrieve the Cookie Supplied for a websocket connection
  */
 char *AsyncSocket_GetWebSocketCookie(AsyncSocket *asock);
-#endif
-
-/*
- * Retrieve the close status, if received, for a websocket connection
- */
-uint16 AsyncSocket_GetWebSocketCloseStatus(const AsyncSocket *asock);
-
-/*
- * Set low-latency mode for sends:
- */
-void AsyncSocket_SetSendLowLatencyMode(AsyncSocket *asock, Bool enable);
-
-/*
- * Get negotiated websocket protocol
- */
-const char *AsyncSocket_GetWebSocketProtocol(AsyncSocket *asock);
-
-const char * stristr(const char *s, const char *find);
 
 /*
  * Some logging macros for convenience
