@@ -59,13 +59,11 @@
  *   FreeBSD: really UTF-8
  *   ESX: UTF-8 by policy decree
  *   Mac: really UTF-8
- *   Netware: don't know (either no iconv, or really UTF-8)
  */
 
 #if defined(__FreeBSD__) || \
     defined(VMX86_SERVER) || \
     defined(__APPLE__) || \
-    defined(N_PLAT_NLM) || \
     defined __ANDROID__
 #define CURRENT_IS_UTF8
 #endif
@@ -378,6 +376,9 @@ Bool CodeSet_UTF32ToUTF8(const char *utf32,
 
 int CodeSet_LengthInCodePoints(const char *utf8);
 
+int CodeSet_CodePointOffsetToByteOffset(const char *utf8,
+                                        int codePointOffset);
+
 int CodeSet_GetUtf8(const char *string,
                     const char *end,
                     uint32 *uchar);
@@ -460,7 +461,7 @@ CodeSet_Utf16ToUtf8(const utf16_t *strW)  // IN:
  *      put a NUL terminator.
  *
  * Results:
- *      
+ *
  *      Returns the offset of the byte immediately following the last
  *      complete UTF-8 code point in buf that is entirely within the
  *      range [0, offset-1]. Note that if the final UTF-8 code point
@@ -545,7 +546,7 @@ CodeSet_Utf8FindCodePointBoundary(const char *buf,   // IN
  *      put a NUL terminator.
  *
  * Results:
- *      
+ *
  *      Returns the offset of the byte immediately following the last
  *      complete UTF-16 code point in buf that is entirely within the
  *      range [0, offset-1]. Note that if the final UTF-16 code point
