@@ -117,6 +117,9 @@ struct FS_PartitionListResult;
 
 int File_GetVMFSAttributes(ConstUnicode pathName,
                            struct FS_PartitionListResult **fsAttrs);
+int File_GetVMFSFSType(ConstUnicode pathName,
+                       int fd,
+                       uint16 *fsTypeNum);
 int File_GetVMFSVersion(ConstUnicode pathName,
                         uint32 *versionNum);
 int File_GetVMFSBlockSize(ConstUnicode pathName,
@@ -180,6 +183,8 @@ Bool File_DeleteDirectoryTree(ConstUnicode pathName);
 int File_ListDirectory(ConstUnicode pathName,
                        Unicode **ids);
 
+Bool File_IsOsfsVolumeEmpty(ConstUnicode pathName);
+
 /*
  * Simple file-system walk.
  */
@@ -199,6 +204,8 @@ Bool File_IsSymLink(ConstUnicode pathName);
 
 Bool File_IsCharDevice(ConstUnicode pathName);
 
+Bool File_GetParent(Unicode *canPath);
+
 Bool File_IsRemote(ConstUnicode pathName);
 
 Bool File_IsEmptyDirectory(ConstUnicode pathName);
@@ -224,12 +231,18 @@ int File_MakeTempEx2(ConstUnicode dir,
                      void *createFuncData,
                      Unicode *presult);
 
+Unicode File_MakeSafeTempDir(ConstUnicode prefix);
+
 int64 File_GetModTime(ConstUnicode pathName);
 
 char *File_GetModTimeString(ConstUnicode pathName);
 
 char *File_GetUniqueFileSystemID(const char *pathName);
 
+#ifdef _WIN32
+Unicode File_GetVolumeGUID(ConstUnicode pathName);
+#endif
+
 Bool File_GetTimes(ConstUnicode pathName,
                    VmTimeType *createTime,
                    VmTimeType *accessTime,
@@ -287,7 +300,8 @@ Bool File_CopyFromNameToName(ConstUnicode srcName,
 
 Bool File_MoveTree(ConstUnicode srcName,
                    ConstUnicode dstName,
-                   Bool overwriteExisting);
+                   Bool overwriteExisting,
+                   Bool *asMove);
 
 Bool File_CopyTree(ConstUnicode srcName,
                    ConstUnicode dstName,
