@@ -130,11 +130,6 @@ Bool Gos_InSetArray(uint32 gos, const uint32 *set);
 #define ALLWIN32	      ALLWIN9X, ALLWINNT32
 #define ALLWIN64              ALLWINNT64
 #define ALLWIN                ALLWIN32, ALLWIN64
-#define ALLWIN_EXCEPT9XAND2000                                       \
-                              ALLWINXP, ALLWINNET, ALLWINLONGHORN,   \
-                              ALLWINVISTA, ALLWIN2008R2, ALLWINSEVEN,\
-                              ALLWINEIGHTSERVER, ALLWINEIGHTCLIENT,  \
-                              ALLHYPER_V, BS(WINNT)
 
 #define ALLSOLARIS            BS(SOLARIS_6_AND_7), BS(SOLARIS8),     \
                               BS(SOLARIS9), BS(SOLARIS10),           \
@@ -145,20 +140,23 @@ Bool Gos_InSetArray(uint32 gos, const uint32 *set);
                               BS(RHEL), BS(UBUNTU)
 #define ALL26XLINUX64         BS(OTHER26XLINUX_64), BS(DEBIAN45_64), \
                               BS(RHEL_64)
+#define ALL3XLINUX32          BS(OTHER3XLINUX)
+#define ALL3XLINUX64          BS(OTHER3XLINUX_64)
 
 #define ALLVMKERNEL           BS(VMKERNEL), BS(VMKERNEL5)
 
-#define ALLLINUX32            BS(OTHER24XLINUX), ALL26XLINUX32,      \
+#define ALLLINUX32            BS(OTHER24XLINUX), ALL26XLINUX32, ALL3XLINUX32, \
                               BS(OTHERLINUX), BS(VMKERNEL)
-#define ALLLINUX64            BS(OTHER24XLINUX_64), ALL26XLINUX64,   \
-                              BS(OTHERLINUX_64)
+#define ALLLINUX64            BS(OTHER24XLINUX_64), ALL26XLINUX64, \
+                              ALL3XLINUX64, BS(OTHERLINUX_64)
 #define ALLLINUX              ALLLINUX32, ALLLINUX64
 #define ALLDARWIN32           BS(DARWIN9), BS(DARWIN10), BS(DARWIN11)
-#define ALLDARWIN64           BS(DARWIN9_64), BS(DARWIN10_64),       \
-                              BS(DARWIN11_64), BS(DARWIN12_64)
+#define ALLDARWIN64           BS(DARWIN9_64), BS(DARWIN10_64), \
+                              BS(DARWIN11_64), BS(DARWIN12_64), \
+                              BS(DARWIN13_64)
 #define ALLDARWIN             ALLDARWIN32, ALLDARWIN64
-#define ALL64                 ALLWIN64, ALLLINUX64, BS(SOLARIS10_64),\
-                              BS(FREEBSD_64), ALLDARWIN64,           \
+#define ALL64                 ALLWIN64, ALLLINUX64, BS(SOLARIS10_64), \
+                              BS(FREEBSD_64), ALLDARWIN64,            \
                               BS(OTHER_64), ALLVMKERNEL
 
 #define ALLECOMSTATION        BS(ECOMSTATION), BS(ECOMSTATION2)
@@ -190,6 +188,7 @@ Bool Gos_InSetArray(uint32 gos, const uint32 *set);
 #define STR_OS_DEBIAN_5            "debian5"
 #define STR_OS_DEBIAN_6            "debian6"
 #define STR_OS_DEBIAN_7            "debian7"
+#define STR_OS_DEBIAN_7_1          "debian7"
 #define STR_OS_FEDORA              "Fedora"
 #define STR_OS_GENTOO              "Gentoo"
 #define STR_OS_IMMUNIX             "Immunix"
@@ -391,7 +390,7 @@ Bool Gos_InSetArray(uint32 gos, const uint32 *set);
 #define STR_OS_WIN_EIGHT               "windows8"
 #define STR_OS_WIN_EIGHT_X64           "windows8-64"
 #define STR_OS_WIN_EIGHT_GENERIC       "Windows 8"
-#define STR_OS_WIN_EIGHTSERVER_GENERIC "Windows 8 Server"
+#define STR_OS_WIN_EIGHTSERVER_GENERIC "Windows Server 2012"
 
 /*
  * XXX - These need to be updated when MS announces official Win 8 names.
