@@ -260,7 +260,7 @@ typedef uint32 HgfsAccessMode;
 #define HGFS_MODE_WRITE_DATA          (1 << 4)  // Ability to writge file data
 #define HGFS_MODE_APPEND_DATA         (1 << 5)  // Appending data to the end of file
 #define HGFS_MODE_DELETE              (1 << 6)  // Ability to delete the file
-#define HGFS_MODE_TRAVERSE_DIRECTORY  (1 << 7)  // Ability to access files in a directory 
+#define HGFS_MODE_TRAVERSE_DIRECTORY  (1 << 7)  // Ability to access files in a directory
 #define HGFS_MODE_LIST_DIRECTORY      (1 << 8)  // Ability to list file names
 #define HGFS_MODE_ADD_SUBDIRECTORY    (1 << 9)  // Ability to create a new subdirectory
 #define HGFS_MODE_ADD_FILE            (1 << 10) // Ability to create a new file
@@ -450,7 +450,7 @@ typedef uint64 HgfsAttrValid;
 #define HGFS_ATTR_VALID_GROUPID           (1 << 13)
 #define HGFS_ATTR_VALID_FILEID            (1 << 14)
 #define HGFS_ATTR_VALID_VOLID             (1 << 15)
-/* 
+/*
  * Add our file and volume identifiers.
  * NOTE: On Windows hosts, the file identifier is not guaranteed to be valid
  *       particularly with FAT. A defrag operation could cause it to change.
@@ -1654,13 +1654,17 @@ struct HgfsReplySymlinkCreateV3 {
 HgfsReplySymlinkCreateV3;
 
 /* HGFS protocol version 4 definitions. */
+#define HGFS_HEADER_VERSION_1                 1
+#define HGFS_HEADER_VERSION                   HGFS_HEADER_VERSION_1
 
 /*
- * HGFS_PACKET_FLAG_NOTIFICATION set in flags field means that
- * this is a notification or a callback request (not a reply to client request).
+ * Flags to indicate the type of packet following the header and
+ * the overall state of the operation.
  */
 
-#define HGFS_PACKET_FLAG_NOTIFICATION         (1 << 0)
+#define HGFS_PACKET_FLAG_REQUEST              (1 << 0)       // Request packet
+#define HGFS_PACKET_FLAG_REPLY                (1 << 1)       // Reply packet
+#define HGFS_PACKET_FLAG_INFO_EXTERROR        (1 << 2)       // Info has ext error
 
 typedef
 #include "vmware_pack_begin.h"
@@ -1674,7 +1678,7 @@ struct HgfsHeader {
    uint32 requestId;    /* Request ID. */
    uint32 op;           /* Operation. */
    uint32 status;       /* Return value. */
-   uint32 flags;        /* Flags. */
+   uint32 flags;        /* Flags. See above. */
    uint32 information;  /* Generic field, used e.g. for native error code. */
    uint64 sessionId;    /* Session ID. */
    uint64 reserved;     /* Reserved for future use. */
@@ -1782,18 +1786,32 @@ struct HgfsIdentity {
 #include "vmware_pack_end.h"
 HgfsIdentity;
 
+#define HGFS_INVALID_SESSION_ID     (~((uint64)0))
+
+/*
+ * The HGFS session flags. These determine the state and validity of the session
+ * information.
+ * It is envisaged that flags will be set for notifying the clients of file system
+ * feature support that transcend multiple request types i.e., HGFS opcodes.
+ */
+typedef uint32 HgfsSessionFlags;
+
+#define HGFS_SESSION_MAXPACKETSIZE_VALID    (1 << 0)
+#define HGFS_SESSION_CHANGENOTIFY_ENABLED   (1 << 1)
+#define HGFS_SESSION_OPLOCK_ENABLED         (1 << 2)
+
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsRequestCreateSessionV4 {
    uint32 numCapabilities;            /* Number of capabilities to follow. */
    uint32 maxPacketSize;              /* Maximum packet size supported. */
-   uint64 reserved;                   /* Reserved for future use. */
+   HgfsSessionFlags flags;            /* Session capability flags. */
+   uint32 reserved;                   /* Reserved for future use. */
    HgfsCapability capabilities[1];    /* Array of HgfsCapabilities. */
 }
 #include "vmware_pack_end.h"
 HgfsRequestCreateSessionV4;
 
-#define HGFS_INVALID_SESSION_ID     (~((uint64)0))
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplyCreateSessionV4 {
@@ -1801,7 +1819,8 @@ struct HgfsReplyCreateSessionV4 {
    uint32 numCapabilities;            /* Number of capabilities to follow. */
    uint32 maxPacketSize;              /* Maximum packet size supported. */
    uint32 identityOffset;             /* Offset to HgfsIdentity or 0 if no identity. */
-   uint64 reserved;                   /* Reserved for future use. */
+   HgfsSessionFlags flags;            /* Flags. */
+   uint32 reserved;                   /* Reserved for future use. */
    HgfsCapability capabilities[1];    /* Array of HgfsCapabilities. */
 }
 #include "vmware_pack_end.h"
@@ -2150,7 +2169,9 @@ HgfsRequestOplockBreakV4;
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplyOplockBreakV4 {
-   uint64 reserved;                 /* Reserved for future use. */
+   HgfsHandle fid;                    /* File handle. */
+   HgfsLockType serverLock;           /* Lock type. */
+   uint64 reserved;                   /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
 HgfsReplyOplockBreakV4;
@@ -2159,7 +2180,7 @@ HgfsReplyOplockBreakV4;
  *  Flusing of a whole volume is not supported.
  *  Flusing of reqular files is supported on all hosts.
  *  Flusing of directories is supproted on POSIX hosts and is
- *  NOOP on Windows hosts. 
+ *  NOOP on Windows hosts.
  */
 typedef
 #include "vmware_pack_begin.h"
