@@ -41,59 +41,6 @@
 
 #include "vmci_defs.h"
 
-/****************************************
- * Vsock, Tcp specific data structures  *
- ****************************************/
-
-/* Some fudged values for TCP over sockets. */
-#define HGFS_HOST_PORT 2000
-
-/* Socket packet magic. */
-#define HGFS_SOCKET_VERSION1   1
-
-/*
- * Socket status codes.
- */
-
-typedef enum {
-   HGFS_SOCKET_STATUS_SUCCESS,                  /* Socket header is good. */
-   HGFS_SOCKET_STATUS_SIZE_MISMATCH,            /* Size and version are incompatible. */
-   HGFS_SOCKET_STATUS_VERSION_NOT_SUPPORTED,    /* Version not handled by remote. */
-   HGFS_SOCKET_STATUS_INVALID_PACKETLEN,        /* Message len exceeds maximum. */
-} HgfsSocketStatus;
-
-/*
- * Socket flags.
- */
-
-typedef uint32 HgfsSocketFlags;
-
-/* Used by backdoor proxy socket client to Hgfs server (out of VMX process). */
-#define HGFS_SOCKET_SYNC         (1 << 0)
-
-/* Socket packet header. */
-typedef
-#include "vmware_pack_begin.h"
-struct HgfsSocketHeader {
-   uint32 version;            /* Header version. */
-   uint32 size;               /* Header size, should match for the specified version. */
-   HgfsSocketStatus status;   /* Status: always success when sending (ignored) valid on replies. */
-   uint32 packetLen;          /* The length of the packet to follow. */
-   HgfsSocketFlags flags;     /* The flags to indicate how to deal with the packet. */
-}
-#include "vmware_pack_end.h"
-HgfsSocketHeader;
-
-#define HgfsSocketHeaderInit(hdr, _version, _size, _status, _pktLen, _flags) \
-   do {                                                                      \
-      (hdr)->version    = (_version);                                        \
-      (hdr)->size       = (_size);                                           \
-      (hdr)->status     = (_status);                                         \
-      (hdr)->packetLen  = (_pktLen);                                         \
-      (hdr)->flags      = (_flags);                                          \
-   } while (0)
-
-
 /************************************************
  *    VMCI specific data structures, macros     *
  ************************************************/
@@ -110,16 +57,17 @@ typedef enum {
    HGFS_ASYNC_IOREQ_SHMEM,
    HGFS_ASYNC_IOREQ_GET_PAGES,
    HGFS_ASYNC_IOREP,
-} HgfsAsyncReplyFlags;
+} HgfsTransportReplyType;
 
 typedef enum {
    HGFS_TH_REP_GET_PAGES,
    HGFS_TH_REQUEST,
    HGFS_TH_TERMINATE_SESSION,
-} HgfsTransportPacketType;
+} HgfsTransportRequestType;
 
-#define HGFS_VMCI_TRANSPORT_ERROR   (VMCI_ERROR_CLIENT_MIN - 1)
-#define HGFS_VMCI_VERSION_MISMATCH  (VMCI_ERROR_CLIENT_MIN - 2)
+#define HGFS_VMCI_TRANSPORT_ERROR      (VMCI_ERROR_CLIENT_MIN - 1)
+#define HGFS_VMCI_VERSION_MISMATCH     (VMCI_ERROR_CLIENT_MIN - 2)
+#define HGFS_VMCI_TYPE_NOT_SUPPORTED   (VMCI_ERROR_CLIENT_MIN - 3)
 
 /*
  * Used By : Guest and Host
@@ -157,14 +105,28 @@ HgfsAsyncIov;
  * Every VMCI request will have this transport Header sent over
  * in the datagram by the Guest OS.
  *
+ * Node fields are set to be compatible for backwards compatibility
+ * for version 1 and common for newer versions.
+ *
  * Used By : Guest and Host
  * Lives in : Sent by Guest inside VMCI datagram
  */
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsVmciTransportHeader {
+struct HgfsVmciHeaderNode {
    uint32 version;                          /* Version number */
-   HgfsTransportPacketType pktType;         /* Type of packet */
+   union {
+      HgfsTransportRequestType pktType;     /* Type of packet for client to server */
+      HgfsTransportReplyType replyType;     /* Type of packet for server to client */
+   };
+}
+#include "vmware_pack_end.h"
+HgfsVmciHeaderNode;
+
+typedef
+#include "vmware_pack_begin.h"
+struct HgfsVmciTransportHeader {
+   HgfsVmciHeaderNode node;                 /* Node: version, type etc. */
    uint32 iovCount;                         /* Number of iovs */
    union {
       HgfsIov iov[1];                       /* (PA, len) */
@@ -210,8 +172,7 @@ HgfsVmciAsyncShmem;
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsVmciAsyncReply {
-   uint32 version;
-   HgfsAsyncReplyFlags pktType;
+   HgfsVmciHeaderNode node;                 /* Node: version, type etc. */
    union {
      HgfsVmciAsyncResponse response;
      HgfsVmciAsyncShmem shmem;
