@@ -73,7 +73,6 @@
  *	INCLUDE_ALLOW_DISTRIBUTE
  *	INCLUDE_ALLOW_VMK_MODULE
  *      INCLUDE_ALLOW_VMKDRIVERS
- *      INCLUDE_ALLOW_VMIROM
  *      INCLUDE_ALLOW_MKS
  *
  * Then include this file.
@@ -155,12 +154,7 @@
 #undef INCLUDE_ALLOW_VMK_MODULE
 #undef INCLUDE_ALLOW_VMKDRIVERS
 
-#if defined VMIROM && ! defined INCLUDE_ALLOW_VMIROM
-#error "The surrounding include file is not allowed in vmirom."
-#endif
-#undef INCLUDE_ALLOW_VMIROM
-
-#if defined INCLUDE_ALLOW_MKS && !defined LOCALMKS && !defined REMOTEMKS
+#if defined INCLUDE_ALLOW_MKS && !(defined COREMKS)
 #error "The surrounding include file is not allowed outside of the MKS."
 #endif
 #undef INCLUDE_ALLOW_MKS
