@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2003 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2013 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -57,6 +57,7 @@
    LOGLEVEL_VAR(serial), \
    LOGLEVEL_VAR(parallel), \
    LOGLEVEL_VAR(chipset), \
+   LOGLEVEL_VAR(smram), \
    LOGLEVEL_VAR(smc), \
    LOGLEVEL_VAR(ich7m), \
    LOGLEVEL_VAR(hpet), \
@@ -99,6 +100,7 @@
    LOGLEVEL_VAR(vcpuNUMA), \
    LOGLEVEL_VAR(heci), \
    LOGLEVEL_VAR(vmiopluginlib), \
+   LOGLEVEL_VAR(vsock), \
    \
    /* user/disk */ \
    LOGLEVEL_VAR(aioMgr), \
@@ -120,6 +122,7 @@
    LOGLEVEL_VAR(gui), \
    LOGLEVEL_VAR(guiWin32), \
    LOGLEVEL_VAR(mks), \
+   LOGLEVEL_VAR(mksInput), \
    LOGLEVEL_VAR(mksSWB), \
    LOGLEVEL_VAR(mksClient), \
    LOGLEVEL_VAR(mksServer), \
@@ -128,12 +131,16 @@
    LOGLEVEL_VAR(mksMouse), \
    LOGLEVEL_VAR(mksHostCursor), \
    LOGLEVEL_VAR(mksCursorPosition), \
-   LOGLEVEL_VAR(mksHostOps), \
+   LOGLEVEL_VAR(mksRenderOps), \
+   LOGLEVEL_VAR(mksGLBasic), \
    LOGLEVEL_VAR(mksGLManager), \
    LOGLEVEL_VAR(mksGLFBO), \
    LOGLEVEL_VAR(mksGLShader), \
    LOGLEVEL_VAR(mksGLState), \
    LOGLEVEL_VAR(mksGLWindow), \
+   LOGLEVEL_VAR(mksGLContextMux), \
+   LOGLEVEL_VAR(mksGLDraw), \
+   LOGLEVEL_VAR(mksGLQuery), \
    LOGLEVEL_VAR(mksWinBSOD), \
    LOGLEVEL_VAR(vdpPlugin), \
    \
@@ -250,6 +257,7 @@
    LOGLEVEL_VAR(inputdevtap), \
    LOGLEVEL_VAR(objlib), \
    LOGLEVEL_VAR(vsanobj), \
+   LOGLEVEL_VAR(vvolbe), \
    LOGLEVEL_VAR(svgadevtap), \
    LOGLEVEL_VAR(masReceipt), /* lib/masReceipt */ \
    LOGLEVEL_VAR(serviceImpl), /* lib/serviceImpl */ \
@@ -270,6 +278,8 @@
    LOGLEVEL_VAR(vva),  /* apps/rde/vva */ \
    LOGLEVEL_VAR(ftConfig), /*lib/ftConfig */ \
    LOGLEVEL_VAR(vmname),  /* lib/vmname */ \
+   LOGLEVEL_VAR(gpumgmt), \
+   LOGLEVEL_VAR(unityMsg),  /* mks/remote/vdpUnityVmdb */ \
    /* end of list */
 
 LOGLEVEL_EXTENSION_DECLARE(LOGLEVEL_USER);
