@@ -69,16 +69,11 @@
 #define RANK_hgfsSearchArrayLock     (RANK_libLockBase + 0x4060)
 #define RANK_hgfsNodeArrayLock       (RANK_libLockBase + 0x4070)
 
-/*
- * SLPv2 global lock
- */
-#define RANK_slpv2GlobalLock         (RANK_libLockBase + 0x4305)
-
 /*
  * vigor (must be < VMDB range and < disklib, see bug 741290)
  */
-#define RANK_vigorClientLock         (RANK_libLockBase + 0x4400)
-#define RANK_vigorOfflineClientLock  (RANK_libLockBase + 0x4410)
+#define RANK_vigorOnlineLock         (RANK_libLockBase + 0x4400)
+#define RANK_vigorOfflineLock        (RANK_libLockBase + 0x4410)
 
 /*
  * NFC lib lock
@@ -86,9 +81,9 @@
 #define RANK_nfcLibLock              (RANK_libLockBase + 0x4505)
 
 /*
- * policy ops pending list lock
+ * Policy lib lock
  */
-#define RANK_popPendingListLock      (RANK_libLockBase + 0x4605)
+#define RANK_policyLibLock           (RANK_libLockBase + 0x4605)
 
 /*
  * disklib and I/O related locks
@@ -128,6 +123,7 @@
 #define RANK_usbEnumGlobalLock       (RANK_libLockBase + 0x6506)
 #define RANK_usbArbLibAsockLock      (RANK_libLockBase + 0x6507)
 #define RANK_usbEnumBackendLock      (RANK_libLockBase + 0x6508)
+#define RANK_sensorQueueLock         (RANK_libLockBase + 0x6509)
 
 /*
  * misc locks
@@ -138,15 +134,15 @@
  * At least:
  *    impersonate < pollDefault
  *    keyLocator < preference (for checking AESNI)
- *    keyLocator < ssl (bug 743010)
+ *    keyLocator < sslState (bug 743010)
  *    configDb < keyLocator (for unlocking dictionaries)
  *    battery/button < preference
  *    workerLib < something for sure under VThread_Create
  *    licenseCheck < preference
+ *    sslState < getSafeTmpDir
  */
 
 #define RANK_vigorTransportListLock  (RANK_libLockBase + 0x7010)
-#define RANK_getSafeTmpDirLock       (RANK_libLockBase + 0x7020)
 #define RANK_batteryLock             (RANK_libLockBase + 0x7030)
 #define RANK_buttonLock              (RANK_libLockBase + 0x7040)
 #define RANK_impersonateLock         (RANK_libLockBase + 0x7045)
@@ -155,7 +151,9 @@
 #define RANK_configDbLock            (RANK_libLockBase + 0x7070)
 #define RANK_keyLocatorLock          (RANK_libLockBase + 0x7080)
 #define RANK_sslStateLock            (RANK_libLockBase + 0x7085)
+#define RANK_getSafeTmpDirLock       (RANK_libLockBase + 0x7086)
 #define RANK_licenseCheckLock        (RANK_libLockBase + 0x7090)
 #define RANK_preferenceLock          (RANK_libLockBase + 0x7100)
 
+
 #endif /* _LIBMUTEXRANK_H */
