@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2004 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2013 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -79,7 +79,6 @@
 #define INCLUDE_ALLOW_VMK_MODULE
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 
 // XXX not necessary except some places include vm_assert.h improperly
@@ -114,6 +113,8 @@ extern "C" {
            Panic(_##name##Fmt "\n", __FILE__, __LINE__)
 #define _ASSERT_PANIC_BUG(bug, name) \
            Panic(_##name##Fmt " bugNr=%d\n", __FILE__, __LINE__, bug)
+#define _ASSERT_PANIC_NORETURN(name) \
+           Panic(_##name##Fmt "\n", __FILE__, __LINE__)
 #endif /* VMKERNEL */
 
 #ifdef VMX86_DEVEL
@@ -179,11 +180,11 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
     */
 
    #ifdef __GNUC__
-      #define ASSERT_IFNOT(cond, panic)                                              \
-                 ({if (UNLIKELY(!(cond))) { panic; if (0) { if (cond) { ; } } } (void)0;})
+      #define ASSERT_IFNOT(cond, panic)                                       \
+         ({if (UNLIKELY(!(cond))) { panic; if (0) { if (cond) {;}}} (void)0;})
    #else
-      #define ASSERT_IFNOT(cond, panic)                               \
-                 (UNLIKELY(!(cond)) ? (panic) : (void)0)
+      #define ASSERT_IFNOT(cond, panic)                                       \
+         (UNLIKELY(!(cond)) ? (panic) : (void)0)
    #endif
 #endif
 
@@ -195,7 +196,7 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
  * ASSERT() is special cased because of interaction with Windows DDK.
  */
 
-#if defined VMX86_DEBUG || defined ASSERT_ALWAYS_AVAILABLE
+#if defined VMX86_DEBUG
 #undef ASSERT
 #define ASSERT(cond) \
            ASSERT_IFNOT(cond, _ASSERT_PANIC(AssertAssert))
@@ -207,21 +208,23 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define PANIC()        _ASSERT_PANIC(AssertPanic)
 #define PANIC_BUG(bug) _ASSERT_PANIC_BUG(bug, AssertPanic)
 
-#ifdef VMKERNEL
-#define ASSERT_OR_IN_PANIC(cond) ASSERT((cond) || Panic_IsSystemInPanic())
-#endif
-
 #define ASSERT_NOT_IMPLEMENTED(cond) \
            ASSERT_IFNOT(cond, NOT_IMPLEMENTED())
 #define ASSERT_NOT_IMPLEMENTED_BUG(bug, cond) \
            ASSERT_IFNOT(cond, NOT_IMPLEMENTED_BUG(bug))
 
+#if defined VMM
+#define NOT_IMPLEMENTED()        _ASSERT_PANIC_NORETURN(AssertNotImplemented)
+#define NOT_IMPLEMENTED_BUG(bug) \
+          _ASSERT_PANIC_BUG_NORETURN(bug, AssertNotImplemented)
+#else
 #if defined VMKERNEL && defined VMX86_DEBUG
 #define NOT_IMPLEMENTED()        _ASSERT_PANIC_NORETURN(AssertNotImplemented)
 #else
 #define NOT_IMPLEMENTED()        _ASSERT_PANIC(AssertNotImplemented)
 #endif
 #define NOT_IMPLEMENTED_BUG(bug) _ASSERT_PANIC_BUG(bug, AssertNotImplemented)
+#endif
 
 #if defined VMKERNEL && defined VMX86_DEBUG
 #define NOT_REACHED()            _ASSERT_PANIC_NORETURN(AssertNotReached)
@@ -233,6 +236,37 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define ASSERT_MEM_ALLOC(cond) \
            ASSERT_IFNOT(cond, _ASSERT_PANIC(AssertMemAlloc))
 
+#ifdef FROBOS
+
+#undef ASSERT
+#undef NOT_IMPLEMENTED
+#undef NOT_REACHED
+#undef ASSERT_MEM_ALLOC
+
+EXTERN void Test_Panic(const char *filename, int lineno, const char *fmt, ...)
+            PRINTF_DECL(3, 4);
+
+#define ASSERT(_cond)                                           \
+do {                                                            \
+   if (UNLIKELY(!(_cond))) {                                    \
+      Test_Panic(__FILE__, __LINE__, "ASSERT(%s)", #_cond);     \
+   }                                                            \
+} while (0)
+
+#define NOT_IMPLEMENTED() \
+do {                                                            \
+   Test_Panic(__FILE__, __LINE__, "NOT_IMPLEMENTED");           \
+} while (0)
+
+#define ASSERT_MEM_ALLOC(_cond) ASSERT(_cond)
+
+#define NOT_REACHED()                                       \
+do {                                                        \
+   Test_Panic(__FILE__, __LINE__, "NOT_REACHED executed");   \
+} while (0)
+
+#endif	/* FROBOS */
+
 #ifdef VMX86_DEVEL
    #define ASSERT_DEVEL(cond) ASSERT(cond)
 #else
@@ -256,10 +290,10 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #define LOG_ONCE(_s) DO_ONCE(Log _s)
 
 #ifdef VMX86_DEVEL
-   #define DEPRECATED(_fix) DO_ONCE(                                        \
-                               Warning("%s:%d: %s is DEPRECATED; %s\n",     \
-                                          __FILE__, __LINE__, __FUNCTION__, \
-                                          _fix))
+   #define DEPRECATED(_fix) DO_ONCE(                                     \
+                               Warning("%s:%d: %s is DEPRECATED. %s\n",  \
+                                       __FILE__, __LINE__, __FUNCTION__, \
+                                       _fix))
 #else
    #define DEPRECATED(_fix) do {} while (0)
 #endif
@@ -269,7 +303,7 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
  * Redefine macros that are only in debug versions
  */
 
-#if !defined VMX86_DEBUG && !defined ASSERT_ALWAYS_AVAILABLE // {
+#if !defined VMX86_DEBUG // {
 
 #undef  ASSERT
 #define ASSERT(cond) ((void) 0)
@@ -277,9 +311,6 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
 #undef  ASSERT_BUG_DEBUGONLY
 #define ASSERT_BUG_DEBUGONLY(bug, cond) ((void) 0)
 
-#undef  ASSERT_LENGTH
-#define ASSERT_LENGTH(real, expected) ((void) 0)
-
 /*
  * Expand NOT_REACHED() as appropriate for each situation.
  *
@@ -331,12 +362,17 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
  * The implementation uses both enum and typedef because the typedef alone is
  * insufficient; gcc allows arrays to be declared with non-constant expressions
  * (even in typedefs, where it makes no sense).
+ *
+ * NOTE: if GCC ever changes so that it ignores unused types altogether, this
+ * assert might not fire!  We explicitly mark it as unused because GCC 4.8+
+ * uses -Wunused-local-typedefs as part of -Wall, which means the typedef will
+ * generate a warning.
  */
 
 #define ASSERT_ON_COMPILE(e) \
    do { \
       enum { AssertOnCompileMisused = ((e) ? 1 : -1) }; \
-      typedef char AssertOnCompileFailed[AssertOnCompileMisused]; \
+      UNUSED_TYPE(typedef char AssertOnCompileFailed[AssertOnCompileMisused]); \
    } while (0)
 
 
@@ -357,8 +393,8 @@ EXTERN void WarningThrottled(uint32 *count, const char *fmt, ...)
  */
 
 #define MY_ASSERTS(name, assertions) \
-   static INLINE void name(void) { \
-      assertions \
+   static INLINE void name(void) {   \
+      assertions                     \
    }
 
 #ifdef __cplusplus
