@@ -37,8 +37,6 @@
 #ifndef _ATOMIC_H_
 #define _ATOMIC_H_
 
-//#define FAKE_ATOMIC /* defined if true atomic not needed */
-
 #define INCLUDE_ALLOW_USERLEVEL
 
 #define INCLUDE_ALLOW_MODULE
@@ -48,24 +46,20 @@
 #define INCLUDE_ALLOW_VMKERNEL
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 
 #include "vm_basic_types.h"
 
 
-/* Basic atomic type: 32 bits */
+/* Basic atomic types: 32 and 64 bits */
 typedef struct Atomic_uint32 {
    volatile uint32 value;
-} Atomic_uint32;
+} Atomic_uint32 ALIGNED(4);
 
-
-/* Basic atomic type: 64 bits */
 typedef struct  Atomic_uint64 {
    volatile uint64 value;
 } Atomic_uint64 ALIGNED(8);
 
-
 /*
  * Prototypes for msft atomics.  These are defined & inlined by the
  * compiler so no function definition is needed.  The prototypes are
@@ -90,9 +84,10 @@ long  _InterlockedCompareExchange(long volatile*, long, long);
 long  _InterlockedExchangeAdd(long volatile*, long);
 long  _InterlockedDecrement(long volatile*);
 long  _InterlockedIncrement(long volatile*);
+void  _ReadWriteBarrier(void);
 #pragma intrinsic(_InterlockedExchange, _InterlockedCompareExchange)
 #pragma intrinsic(_InterlockedExchangeAdd, _InterlockedDecrement)
-#pragma intrinsic(_InterlockedIncrement)
+#pragma intrinsic(_InterlockedIncrement, _ReadWriteBarrier)
 
 #if defined(VM_X86_64)
 long     _InterlockedAnd(long volatile*, long);
@@ -121,7 +116,7 @@ __int64  _InterlockedCompareExchange64(__int64 volatile*, __int64, __int64);
 #endif
 #endif /* _MSC_VER */
 
-#if defined(__arm__) && !defined(FAKE_ATOMIC)
+#if defined(__arm__)
 /*
  * LDREX without STREX or CLREX may cause problems in environments where the
  * context switch may not clear the reference monitor - according ARM manual
@@ -206,13 +201,38 @@ Atomic_VolatileToAtomic64(volatile uint64 *var)
 #define ATOMIC_USE_FENCE
 #endif
 
+/* 
+ * Starting with vSphere 2014, we no longer support ESX on AMD Rev F. 
+ * Thus, we can eliminate all dynamic checks for whether to enable 
+ * the Errata 147 work-around when compiling many of our binaries. 
+ * However, we use an opt-in approach here rather than assuming all 
+ * parts of our builds are safe. For example, the "fdm" binary from 
+ * a new build may time travel back to hosts running older versions 
+ * of ESX on Rev F, so "fdm" continues to require the ability to 
+ * dynamically enable the errata work-around. With vSphere 2017,
+ * this will no longer be required as the oldest version of ESX that 
+ * VC 2017 will support is ESX 2014 (which won't run on Rev F).
+ */
+#if (!defined(VMX86_SERVER) ||                                          \
+      (!defined(VMX86_VMX) && !defined(VMKERNEL) &&                     \
+       !defined(VMM)       && !defined(VMCORE)))
+#define MAY_NEED_AMD_REVF_WORKAROUND 1
+#else
+#define MAY_NEED_AMD_REVF_WORKAROUND 0
+#endif
+
+#if MAY_NEED_AMD_REVF_WORKAROUND
 #if defined(VMATOMIC_IMPORT_DLLDATA)
 VMX86_EXTERN_DATA Bool AtomicUseFence;
 #else
 EXTERN Bool AtomicUseFence;
 #endif
-
 EXTERN Bool atomicFenceInitialized;
+#else   /* MAY_NEED_AMD_REVF_WORKAROUND */
+#define AtomicUseFence         FALSE
+#define atomicFenceInitialized TRUE
+#endif  /* MAY_NEED_AMD_REVF_WORKAROUND */
+
 
 void AtomicInitFence(void);
 
@@ -220,22 +240,20 @@ static INLINE void
 Atomic_Init(void)
 {
 #ifdef ATOMIC_USE_FENCE
-   if (!atomicFenceInitialized) {
+   if (MAY_NEED_AMD_REVF_WORKAROUND && !atomicFenceInitialized) {
       AtomicInitFence();
    }
 #endif
 }
 
 static INLINE void
-Atomic_SetFence(Bool fenceAfterLock) /* IN: TRUE to enable lfence */
-                                     /*     FALSE to disable. */
+Atomic_SetFence(Bool fenceAfterLock)
 {
+   (void)fenceAfterLock;     /* Work around unused parameter. */
+#if MAY_NEED_AMD_REVF_WORKAROUND
    AtomicUseFence = fenceAfterLock;
-#if defined(__VMKERNEL__)
-   extern void Atomic_SetFenceVMKAPI(Bool fenceAfterLock);
-   Atomic_SetFenceVMKAPI(fenceAfterLock);
-#endif
    atomicFenceInitialized = TRUE;
+#endif
 }
 
 
@@ -243,21 +261,21 @@ Atomic_SetFence(Bool fenceAfterLock) /* IN: TRUE to enable lfence */
 static INLINE void
 AtomicEpilogue(void)
 {
-#ifdef ATOMIC_USE_FENCE
+#if MAY_NEED_AMD_REVF_WORKAROUND && defined(ATOMIC_USE_FENCE)
 #ifdef VMM
       /* The monitor conditionally patches out the lfence when not needed.*/
       /* Construct a MonitorPatchTextEntry in the .patchtext section. */
-   asm volatile ("1:\n\t"
-                 "lfence\n\t"
-                 "2:\n\t"
-                 ".pushsection .patchtext\n\t"
-                 ".quad 1b\n\t"
-                 ".quad 2b\n\t"
-                 ".popsection\n\t" ::: "memory");
+      asm volatile ("1:\n\t"
+                    "lfence\n\t"
+                    "2:\n\t"
+                    ".pushsection .patchtext\n\t"
+                    ".quad 1b\n\t"
+                    ".quad 2b\n\t"
+                    ".popsection\n\t" ::: "memory");
 #else
-   if (UNLIKELY(AtomicUseFence)) {
-      asm volatile ("lfence" ::: "memory");
-   }
+      if (UNLIKELY(AtomicUseFence)) {
+         asm volatile ("lfence" ::: "memory");
+      }
 #endif
 #endif
 }
@@ -318,36 +336,49 @@ AtomicEpilogue(void)
 static INLINE uint32
 Atomic_Read(Atomic_uint32 const *var) // IN
 {
-   return var->value;
+   uint32 value;
+
+#if defined(VMM)
+   ASSERT(((uintptr_t)var % 4) == 0);
+#endif
+
+#if defined(__GNUC__)
+   /*
+    * Use inline assembler to force using a single load instruction to
+    * ensure that the compiler doesn't split a transfer operation into multiple
+    * instructions.
+    */
+
+#if defined(VM_ARM_V7)
+   __asm__ __volatile__(
+      "ldr %0, [%1]"
+      : "=r" (value)
+      : "r" (var->value)
+   );
+#else
+   __asm__ __volatile__(
+      "mov %1, %0"
+      : "=r" (value)
+      : "m" (var->value)
+   );
+#endif
+#elif defined(_MSC_VER)
+   /*
+    * Microsoft docs guarantee simple reads and writes to properly
+    * aligned 32-bit variables use only a single instruction.
+    * http://msdn.microsoft.com/en-us/library/ms684122%28VS.85%29.aspx
+    */
+
+   value = var->value;
+#else
+#error No compiler defined for Atomic_Read
+#endif
+
+   return value;
 }
 #define Atomic_Read32 Atomic_Read
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * Atomic_Write --
- *
- *      Write
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static INLINE void
-Atomic_Write(Atomic_uint32 *var, // IN
-             uint32 val)         // IN
-{
-   var->value = val;
-}
-#define Atomic_Write32 Atomic_Write
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -368,11 +399,7 @@ static INLINE uint32
 Atomic_ReadWrite(Atomic_uint32 *var, // IN
                  uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   uint32 retval = var->value;
-   var->value = val;
-   return retval;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 retVal;
    register volatile uint32 res;
@@ -424,6 +451,83 @@ Atomic_ReadWrite(Atomic_uint32 *var, // IN
 #define Atomic_ReadWrite32 Atomic_ReadWrite
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Atomic_Write --
+ *
+ *      Write
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void
+Atomic_Write(Atomic_uint32 *var, // IN
+             uint32 val)         // IN
+{
+#if defined(VMM)
+   ASSERT(((uintptr_t)var % 4) == 0);
+#endif
+
+#if defined(__GNUC__)
+#if defined(VM_ARM_V7)
+   /*
+    * A3.4.1 ARM DDI 0406C:
+    *
+    * When a processor writes using any instruction other than a
+    * Store-Exclusive:
+    *
+    * - if the write is to a physical address that is not covered by its local
+    *   monitor the write does not affect the state of the local monitor
+    * - if the write is to a physical address that is covered by its local
+    *   monitor it is IMPLEMENTATION DEFINED whether the write affects the
+    *   state of the local monitor.
+    *
+    * A3.4.5 ARM DDI 0406C:
+    *
+    * If two STREX instructions are executed without an intervening LDREX the
+    * second STREX returns a status value of 1. This means that:
+    *
+    * — ARM recommends that, in a given thread of execution, every STREX has a
+    *   preceding LDREX associated with it
+    * — it is not necessary for every LDREX to have a subsequent STREX.
+    */
+
+   Atomic_ReadWrite(var, val);
+#else
+   /*
+    * Use inline assembler to force using a single store instruction to
+    * ensure that the compiler doesn't split a transfer operation into multiple
+    * instructions.
+    */
+
+   __asm__ __volatile__(
+      "mov %1, %0"
+      : "=m" (var->value)
+      : "r" (val)
+   );
+#endif
+#elif defined(_MSC_VER)
+   /*
+    * Microsoft docs guarantee simple reads and writes to properly
+    * aligned 32-bit variables use only a single instruction.
+    * http://msdn.microsoft.com/en-us/library/ms684122%28VS.85%29.aspx
+    */
+
+   var->value = val;
+#else
+#error No compiler defined for Atomic_Write
+#endif
+}
+#define Atomic_Write32 Atomic_Write
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -445,14 +549,7 @@ Atomic_ReadIfEqualWrite(Atomic_uint32 *var, // IN
                         uint32 oldVal,      // IN
                         uint32 newVal)      // IN
 {
-#ifdef FAKE_ATOMIC
-   uint32 readVal = var->value;
-
-   if (oldVal == readVal) {
-     var->value = newVal;
-   }
-   return oldVal;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register uint32 retVal;
    register uint32 res;
@@ -513,6 +610,52 @@ Atomic_ReadIfEqualWrite(Atomic_uint32 *var, // IN
 #define Atomic_ReadIfEqualWrite32 Atomic_ReadIfEqualWrite
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Atomic_ReadIfEqualWriteLockAcquire --
+ *
+ *      Compare exchange: Read variable, if equal to oldVal, write newVal
+ *      using hint to elide the hardware lock.
+ *
+ * Results:
+ *      The value of the atomic variable before the write.
+ *
+ * Side effects:
+ *      The atomic variable may be modified.
+ *      May return in transactional execution mode on TSX enabled CPUs
+ *      IFF the atomic variable is modified.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE uint32
+Atomic_ReadIfEqualWriteLockAcquire(Atomic_uint32 *var, // IN
+				   uint32 oldVal,      // IN
+				   uint32 newVal)      // IN
+{
+#if defined(__GNUC__) && defined(__x86_64__)
+   uint32 val;
+   __asm__ __volatile__(
+      "repnz; lock; cmpxchgl %2, %1"
+      : "=a" (val),
+	"+m" (var->value)
+      : "r" (newVal),
+	"0" (oldVal)
+      : "cc"
+   );
+   AtomicEpilogue();
+   if (val != oldVal) {
+      /* Leave transactional execution mode if TSX enabled. */
+      __asm__ __volatile__("pause");
+   }
+   return val;
+#else
+   return Atomic_ReadIfEqualWrite(var, oldVal, newVal);
+#endif
+}
+
+
 #if defined(__x86_64__) || defined(VM_ARM_V7)
 /*
  *-----------------------------------------------------------------------------
@@ -604,9 +747,7 @@ static INLINE void
 Atomic_And(Atomic_uint32 *var, // IN
            uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   var->value &= val;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 res;
    register volatile uint32 tmp;
@@ -670,9 +811,7 @@ static INLINE void
 Atomic_Or(Atomic_uint32 *var, // IN
           uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   var->value |= val;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 res;
    register volatile uint32 tmp;
@@ -736,9 +875,7 @@ static INLINE void
 Atomic_Xor(Atomic_uint32 *var, // IN
            uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   var->value ^= val;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 res;
    register volatile uint32 tmp;
@@ -841,9 +978,7 @@ static INLINE void
 Atomic_Add(Atomic_uint32 *var, // IN
            uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   var->value += val;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 res;
    register volatile uint32 tmp;
@@ -946,9 +1081,7 @@ static INLINE void
 Atomic_Sub(Atomic_uint32 *var, // IN
            uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   var->value -= val;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 res;
    register volatile uint32 tmp;
@@ -1051,7 +1184,7 @@ static INLINE void
 Atomic_Inc(Atomic_uint32 *var) // IN
 {
 #ifdef __GNUC__
-#if defined(VM_ARM_V7) || defined(FAKE_ATOMIC)
+#if defined(VM_ARM_V7)
    Atomic_Add(var, 1);
 #else // VM_ARM_V7
    /* Checked against the Intel manual and GCC --walken */
@@ -1097,7 +1230,7 @@ static INLINE void
 Atomic_Dec(Atomic_uint32 *var) // IN
 {
 #ifdef __GNUC__
-#if defined(VM_ARM_V7) || defined(FAKE_ATOMIC)
+#if defined(VM_ARM_V7)
    Atomic_Sub(var, 1);
 #else // VM_ARM_V7
    /* Checked against the Intel manual and GCC --walken */
@@ -1279,24 +1412,21 @@ static INLINE uint32
 Atomic_FetchAndAddUnfenced(Atomic_uint32 *var, // IN
                            uint32 val)         // IN
 {
-#ifdef FAKE_ATOMIC
-   uint32 res = var->value;
-   var->value = res + val;
-   return res;
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    register volatile uint32 res;
    register volatile uint32 retVal;
+   register volatile uint32 tmp;
 
    dmb();
 
    __asm__ __volatile__(
       "1: ldrex %[retVal], [%[var]] \n\t"
-      "add %[val], %[val], %[retVal] \n\t"
-      "strex %[res], %[val], [%[var]] \n\t"
+      "add %[tmp], %[val], %[retVal] \n\t"
+      "strex %[res], %[tmp], [%[var]] \n\t"
       "teq %[res], #0 \n\t"
       "bne 1b"
-      : [res] "=&r" (res), [retVal] "=&r" (retVal)
+      : [tmp] "=&r" (tmp), [res] "=&r" (res), [retVal] "=&r" (retVal)
       : [var] "r" (&var->value), [val] "r" (val)
       : "cc"
    );
@@ -1335,6 +1465,49 @@ Atomic_FetchAndAddUnfenced(Atomic_uint32 *var, // IN
 #define Atomic_ReadAdd32 Atomic_FetchAndAdd
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Atomic_FetchAndAddLockReleaseUnfenced --
+ *
+ *      Atomic read (returned), add a value, write
+ *      using hint to elide the hardware lock.
+ *
+ *      If you have to implement FetchAndAdd() on an architecture other than
+ *      x86 or x86-64, you might want to consider doing something similar to
+ *      Atomic_FetchAndOr().
+ *
+ *      The "Unfenced" version of Atomic_FetchAndInc never executes
+ *      "lfence" after the interlocked operation.
+ *
+ * Results:
+ *      The value of the variable before the operation.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE uint32
+Atomic_FetchAndAddLockReleaseUnfenced(Atomic_uint32 *var, // IN
+				      uint32 val)         // IN
+{
+#if defined(__GNUC__) && defined(__x86_64__)
+   __asm__ __volatile__(
+      "repz; lock; xaddl %0, %1"
+      : "=r" (val),
+	"+m" (var->value)
+      : "0" (val)
+      : "cc"
+   );
+   return val;
+#else
+   return Atomic_FetchAndAddUnfenced(var, val);
+#endif
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1362,7 +1535,7 @@ static INLINE uint32
 Atomic_FetchAndAdd(Atomic_uint32 *var, // IN
                    uint32 val)         // IN
 {
-#if defined(__GNUC__) && !defined(VM_ARM_V7) && !defined(FAKE_ATOMIC)
+#if defined(__GNUC__) && !defined(VM_ARM_V7)
    val = Atomic_FetchAndAddUnfenced(var, val);
    AtomicEpilogue();
    return val;
@@ -1372,6 +1545,44 @@ Atomic_FetchAndAdd(Atomic_uint32 *var, // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Atomic_FetchAndAddLockRelease --
+ *
+ *      Atomic read (returned), add a value, write
+ *      using hint to elide the hardware lock.
+ *
+ *      If you have to implement FetchAndAdd() on an architecture other than
+ *      x86 or x86-64, you might want to consider doing something similar to
+ *      Atomic_FetchAndOr().
+ *
+ *      Unlike "Unfenced" version, this one may execute the "lfence" after
+ *      interlocked operation.
+ *
+ * Results:
+ *      The value of the variable before the operation.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE uint32
+Atomic_FetchAndAddLockRelease(Atomic_uint32 *var, // IN
+			      uint32 val)         // IN
+{
+#if defined(__GNUC__) && !defined(VM_ARM_V7)
+   val = Atomic_FetchAndAddLockReleaseUnfenced(var, val);
+   AtomicEpilogue();
+   return val;
+#else
+   return Atomic_FetchAndAddLockReleaseUnfenced(var, val);
+#endif
+}
+
+
 #if defined(__x86_64__)
 /*
  *-----------------------------------------------------------------------------
@@ -1410,6 +1621,30 @@ Atomic_ReadAdd64(Atomic_uint64 *var, // IN
 #error No compiler defined for Atomic_ReadAdd64
 #endif
 }
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Atomic_ReadSub64 --
+ *
+ *      Atomic read (returned), sub a value, write.
+ *
+ * Results:
+ *      The value of the variable before the operation.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE uint64
+Atomic_ReadSub64(Atomic_uint64 *var, // IN
+                 uint64 val)         // IN
+{
+   // Do an sub by an add and a overflow
+   return Atomic_ReadAdd64(var, -val);
+}
 #endif
 
 
@@ -1585,14 +1820,7 @@ Atomic_CMPXCHG64(Atomic_uint64 *var,   // IN/OUT
                  uint64 const *oldVal, // IN
                  uint64 const *newVal) // IN
 {
-#ifdef FAKE_ATOMIC
-   uint64 readVal = var->value;
-
-   if (*oldVal == readVal) {
-     var->value = *newVal;
-   }
-   return (*oldVal == readVal);
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #if defined(VM_ARM_V7)
    return (Atomic_ReadIfEqualWrite64(var, *oldVal, *newVal) == *oldVal);
 
@@ -1755,14 +1983,7 @@ Atomic_CMPXCHG32(Atomic_uint32 *var,   // IN/OUT
                  uint32 oldVal, // IN
                  uint32 newVal) // IN
 {
-#ifdef FAKE_ATOMIC
-   uint32 readVal = var->value;
-
-   if (oldVal == readVal) {
-     var->value = newVal;
-   }
-   return (oldVal == readVal);
-#elif defined(__GNUC__)
+#if defined(__GNUC__)
 #ifdef VM_ARM_V7
    return (Atomic_ReadIfEqualWrite(var, oldVal, newVal) == oldVal);
 #else // VM_ARM_V7
@@ -1807,9 +2028,7 @@ Atomic_CMPXCHG32(Atomic_uint32 *var,   // IN/OUT
 static INLINE uint64
 Atomic_Read64(Atomic_uint64 const *var) // IN
 {
-#ifdef FAKE_ATOMIC
-   return var->value;
-#elif defined(__GNUC__) && defined(__x86_64__)
+#if defined(__GNUC__) && defined(__x86_64__)
    uint64 value;
 
 #ifdef VMM
@@ -2397,7 +2616,7 @@ Atomic_TestBit64(Atomic_uint64 *var, // IN
       "bt %2, %1; setc %0"
       : "=rm"(out)
       : "m" (var->value),
-        "ri" (bit)
+        "rJ" (bit)
       : "cc"
    );
    return out;
@@ -2421,6 +2640,9 @@ Atomic_TestBit64(Atomic_uint64 *var, // IN
  * (void *)val, we have (void *)(uintptr_t)val.
  * The specific problem case is the Windows ddk compiler
  * (as used by the SVGA driver).  -- edward
+ *
+ * NOTE: See the comment in vm_assert.h for why we need UNUSED_TYPE in
+ * AtomicAssertOnCompile(), and why we need to be very careful doing so.
  */
 
 #define MAKE_ATOMIC_TYPE(name, size, in, out, cast)                           \
@@ -2434,7 +2656,7 @@ Atomic_TestBit64(Atomic_uint64 *var, // IN
                                       && 8 * sizeof (out) == size             \
                                       && 8 * sizeof (cast) == size            \
                                          ? 1 : -1 };                          \
-      typedef char AssertOnCompileFailed[AssertOnCompileMisused];             \
+      UNUSED_TYPE(typedef char AssertOnCompileFailed[AssertOnCompileMisused]);\
    }                                                                          \
                                                                               \
                                                                               \
@@ -2602,6 +2824,16 @@ MAKE_ATOMIC_TYPE(Ptr, 32, void const *, void *, uintptr_t)
 MAKE_ATOMIC_TYPE(Int, 32, int, int, int)
 
 
+/* Prevent the compiler from re-ordering memory references. */
+#ifdef __GNUC__
+#define ATOMIC_COMPILER_BARRIER()   __asm__ __volatile__ ("": : :"memory")
+#elif defined(_MSC_VER)
+#define ATOMIC_COMPILER_BARRIER()   _ReadWriteBarrier()
+#else
+#error No compiler defined for ATOMIC_COMPILER_BARRIER
+#endif
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -2626,7 +2858,13 @@ static INLINE void
 Atomic_MFence(void)
 {
    Atomic_uint32 fence;
+   ATOMIC_COMPILER_BARRIER();
    Atomic_Xor(&fence, 0x1);
+   ATOMIC_COMPILER_BARRIER();
 }
 
+#ifdef ATOMIC_COMPILER_BARRIER
+#undef ATOMIC_COMPILER_BARRIER
+#endif
+
 #endif // ifndef _ATOMIC_H_
