@@ -47,7 +47,6 @@
 #define INCLUDE_ALLOW_VMKERNEL
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 
 #ifdef VM_X86_64
@@ -58,6 +57,24 @@
 #error "x86-64 not supported"
 #endif
 
+/*
+ * XTEST
+ *     Return TRUE if processor is in transaction region.
+ *
+ */
+#if defined(__GNUC__) && (defined(VMM) || defined(VMKERNEL) || defined(FROBOS))
+static INLINE Bool
+xtest(void)
+{
+   uint8 al;
+   __asm__ __volatile__(".byte 0x0f, 0x01, 0xd6    # xtest \n"
+                        "setnz %%al\n"
+                        : "=a"(al) : : "cc");
+   return al;
+}
+
+#endif /* __GNUC__ */
+
 
 /*
  * FXSAVE/FXRSTOR
@@ -82,22 +99,23 @@
  */
 #if defined(__GNUC__)
 static INLINE void 
-FXSAVE_ES1(uint8 *save)
+FXSAVE_ES1(void *save)
 {
-   __asm__ __volatile__ ("fxsave %0\n" : "=m" (*save) : : "memory");
+   __asm__ __volatile__ ("fxsave %0\n" : "=m" (*(uint8 *)save) : : "memory");
 }
 
 static INLINE void 
-FXRSTOR_ES1(const uint8 *load)
+FXRSTOR_ES1(const void *load)
 {
-   __asm__ __volatile__ ("fxrstor %0\n" : : "m" (*load) : "memory");
+   __asm__ __volatile__ ("fxrstor %0\n"
+                         : : "m" (*(const uint8 *)load) : "memory");
 }
 
 static INLINE void 
-FXRSTOR_AMD_ES0(const uint8 *load)
+FXRSTOR_AMD_ES0(const void *load)
 {
    uint64 dummy = 0;
-      
+
    __asm__ __volatile__ 
        ("fnstsw  %%ax    \n"     // Grab x87 ES bit
         "bt      $7,%%ax \n"     // Test ES bit
@@ -109,7 +127,7 @@ FXRSTOR_AMD_ES0(const uint8 *load)
                                  // x87 exception pointers.
         "fxrstor %1      \n"
         :  
-        : "m" (dummy), "m" (*load)
+        : "m" (dummy), "m" (*(const uint8 *)load)
         : "ax", "memory");
 }
 #endif /* __GNUC__ */
@@ -124,46 +142,58 @@ FXRSTOR_AMD_ES0(const uint8 *load)
 #if defined(__GNUC__) && (defined(VMM) || defined(VMKERNEL) || defined(FROBOS))
 
 static INLINE void 
-XSAVE_ES1(uint8 *save, uint64 mask)
+XSAVE_ES1(void *save, uint64 mask)
 {
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
    __asm__ __volatile__ (
         ".byte 0x0f, 0xae, 0x21 \n"
         :
-        : "c" (save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "c" ((uint8 *)save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #else
    __asm__ __volatile__ (
         "xsave %0 \n"
-        : "=m" (*save)
+        : "=m" (*(uint8 *)save)
         : "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #endif
 }
 
 static INLINE void 
-XRSTOR_ES1(const uint8 *load, uint64 mask)
+XSAVEOPT_ES1(void *save, uint64 mask)
+{
+   __asm__ __volatile__ (
+        ".byte 0x0f, 0xae, 0x31 \n"
+        :
+        : "c" ((uint8 *)save), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "memory");
+}
+
+static INLINE void 
+XRSTOR_ES1(const void *load, uint64 mask)
 {
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
    __asm__ __volatile__ (
         ".byte 0x0f, 0xae, 0x29 \n"
         :
-        : "c" (load), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "c" ((const uint8 *)load),
+          "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #else
    __asm__ __volatile__ (
         "xrstor %0 \n"
         :
-        : "m" (*load), "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (*(const uint8 *)load),
+          "a" ((uint32)mask), "d" ((uint32)(mask >> 32))
         : "memory");
 #endif
 }
 
 static INLINE void 
-XRSTOR_AMD_ES0(const uint8 *load, uint64 mask)
+XRSTOR_AMD_ES0(const void *load, uint64 mask)
 {
    uint64 dummy = 0;
-      
+
    __asm__ __volatile__ 
        ("fnstsw  %%ax    \n"     // Grab x87 ES bit
         "bt      $7,%%ax \n"     // Test ES bit
@@ -177,12 +207,13 @@ XRSTOR_AMD_ES0(const uint8 *load, uint64 mask)
 #if __GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ == 1
         ".byte 0x0f, 0xae, 0x29 \n"
         :
-        : "m" (dummy),
-          "c" (load), "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (dummy), "c" ((const uint8 *)load),
+          "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
 #else
         "xrstor %1 \n"
         :
-        : "m" (dummy), "m" (*load), "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
+        : "m" (dummy), "m" (*(const uint8 *)load),
+          "b" ((uint32)mask), "d" ((uint32)(mask >> 32))
 #endif
         : "eax", "memory");
 }
@@ -502,7 +533,7 @@ Muls64x32s64(int64 multiplicand, uint32 multiplier, uint32 shift)
            "shrdl %%edx, %%eax\n\t"   // result = hi(p2):hi(p1):lo(p1) >> shift
            "shrdl %1, %%edx\n"
         "3:\n\t"
-           : "=A" (result), "=&r" (tmp1), "=&r" (tmp2)
+           : "=A" (result), "=&r" (tmp1), "=&rm" (tmp2)
            : "0" (multiplicand), "rm" (multiplier), "c" (shift)
            : "cc");
    return result;
