@@ -71,23 +71,36 @@ uint64 __shiftright128(uint64 lowPart, uint64 highPart, uint8 shift);
 
 #endif // _MSC_VER
 
+#if defined(__GNUC__)
 /*
  * GET_CURRENT_RIP
  *
- * Return an approximation of the current instruction pointer. For example for a
- * function call
- * foo.c
- * L123: Foo(GET_CURRENT_RIP())
+ * Returns the current instruction pointer. In the example below:
  *
- * The return value from GET_CURRENT_RIP will point a debugger to L123.
+ *   foo.c
+ *   L123: Foo(GET_CURRENT_RIP())
+ *
+ * the return value from GET_CURRENT_RIP will point a debugger to L123.
  */
-#if defined(__GNUC__)
-#define GET_CURRENT_RIP() ({                                                   \
-      void *__rip;                                                             \
-      asm("lea 0(%%rip), %0;\n\t"                                              \
-         : "=r" (__rip));                                                      \
-      __rip;                                                                   \
+#define GET_CURRENT_RIP() ({                                          \
+      void *__rip;                                                    \
+      asm("lea 0(%%rip), %0;\n\t"                                     \
+         : "=r" (__rip));                                             \
+      __rip;                                                          \
 })
+
+/*
+ * GET_CURRENT_LOCATION
+ *
+ * Updates the arguments with the values of the %rip, %rbp, and %rsp
+ * registers at the current code location where the macro is invoked.
+ */
+#define GET_CURRENT_LOCATION(rip, rbp, rsp)                           \
+   asm("lea 0(%%rip), %0\n"                                           \
+       "mov %%rbp, %1\n"                                              \
+       "mov %%rsp, %2\n"                                              \
+       : "=r" (rip), "=r" (rbp), "=r" (rsp))
+
 #endif
 
 /*
@@ -287,6 +300,23 @@ XRSTOR_AMD_ES0(const void *load, uint64 mask)
 
 #endif /* __GNUC__ */
 
+/*
+ * XTEST
+ *     Return TRUE if processor is in transaction region.
+ *
+ */
+#if defined(__GNUC__) && (defined(VMM) || defined(VMKERNEL) || defined(FROBOS))
+static INLINE Bool
+xtest(void)
+{
+   uint8 al;
+   __asm__ __volatile__(".byte 0x0f, 0x01, 0xd6    # xtest \n"
+                        "setnz %%al\n"
+                        : "=a"(al) : : "cc"); 
+   return al;
+}
+
+#endif /* __GNUC__ */
 
 /*
  *-----------------------------------------------------------------------------
@@ -322,7 +352,7 @@ Mul64x6464(uint64 multiplicand,
    uint64 result, dummy;
 
    __asm__("mulq    %3      \n\t"
-           "shrdq   %1, %0  \n\t"
+           "shrdq   %b4, %1, %0  \n\t"
            : "=a" (result),
              "=d" (dummy)
            : "0"  (multiplier),
@@ -380,7 +410,7 @@ Muls64x64s64(int64 multiplicand, int64 multiplier, uint32 shift)
    int64 result, dummy;
 
    __asm__("imulq   %3      \n\t"
-       "shrdq   %1, %0  \n\t"
+       "shrdq   %b4, %1, %0  \n\t"
        : "=a" (result),
          "=d" (dummy)
        : "0"  (multiplier),
