@@ -76,10 +76,13 @@
 #define INCLUDE_ALLOW_VMK_MODULE
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 #include "vm_basic_types.h" // For INLINE.
 
+#if defined VMKERNEL
+#include "vm_assert.h"      // For ASSERT_ON_COMPILE.
+#endif
+
 /* Checks for FreeBSD, filtering out VMKERNEL. */
 #define __IS_FREEBSD__ (!defined(VMKERNEL) && defined(__FreeBSD__))
 #define __IS_FREEBSD_VER__(ver) (__IS_FREEBSD__ && __FreeBSD_version >= (ver))
@@ -213,8 +216,10 @@ Max(int a, int b)
  * Wide versions of string constants.
  */
 
+#ifndef WSTR
 #define WSTR_(X)     L ## X
 #define WSTR(X)      WSTR_(X)
+#endif
 
 
 /*
@@ -276,6 +281,14 @@ Max(int a, int b)
 #define PAGES_2_MBYTES(_npages) ((_npages) >> (20 - PAGE_SHIFT))
 #endif
 
+#ifndef GBYTES_2_PAGES
+#define GBYTES_2_PAGES(_nbytes) ((_nbytes) << (30 - PAGE_SHIFT))
+#endif
+
+#ifndef PAGES_2_GBYTES
+#define PAGES_2_GBYTES(_npages) ((_npages) >> (30 - PAGE_SHIFT))
+#endif
+
 #ifndef BYTES_2_MBYTES
 #define BYTES_2_MBYTES(_nbytes) ((_nbytes) >> 20)
 #endif
@@ -284,9 +297,17 @@ Max(int a, int b)
 #define MBYTES_2_BYTES(_nbytes) ((uint64)(_nbytes) << 20)
 #endif
 
+#ifndef BYTES_2_GBYTES
+#define BYTES_2_GBYTES(_nbytes) ((_nbytes) >> 30)
+#endif
+
+#ifndef GBYTES_2_BYTES
+#define GBYTES_2_BYTES(_nbytes) ((uint64)(_nbytes) << 30)
+#endif
+
 #ifndef VM_PAE_LARGE_PAGE_SHIFT
 #define VM_PAE_LARGE_PAGE_SHIFT 21
-#endif 
+#endif
 
 #ifndef VM_PAE_LARGE_PAGE_SIZE
 #define VM_PAE_LARGE_PAGE_SIZE (1 << VM_PAE_LARGE_PAGE_SHIFT)
@@ -352,45 +373,36 @@ void *_ReturnAddress(void);
 #define GetReturnAddress() _ReturnAddress()
 #elif __GNUC__
 #define GetReturnAddress() __builtin_return_address(0)
+
+#ifdef VMKERNEL
+/*
+ * Using __builtin_frame_address(N) and__builtin_return_address(N)
+ * with N >= 1 causes crashes for code compiled without frame pointers.
+ * Use Stackwalk_ReturnAddress(N) instead. Or, to collect a backtrace 
+ * fragment, use STACKWALK_STORE_BACKTRACE().
+ */
+#define __builtin_frame_address(N) ({           \
+         ASSERT_ON_COMPILE(N == 0);             \
+         __builtin_frame_address(N);            \
+      })
+#define __builtin_return_address(N) ({          \
+         ASSERT_ON_COMPILE(N == 0);             \
+         __builtin_return_address(N);           \
+      })
+#endif
 #endif
 
 
 #ifdef __GNUC__
 #ifndef sun
 
-static INLINE_SINGLE_CALLER uintptr_t
-GetFrameAddr(void)
-{
-   uintptr_t bp;
-#if  !(__GNUC__ == 4 && (__GNUC_MINOR__ == 0 || __GNUC_MINOR__ == 1))
-   bp = (uintptr_t)__builtin_frame_address(0);
-#else
-   /*
-    * We use this assembly hack due to a bug discovered in gcc 4.1.1.
-    * The bug was fixed in 4.2.0; assume it originated with 4.0.
-    * PR147638, PR554369.
-    */
-     __asm__ __volatile__(
-#  if defined(VM_X86_64)
-                          "movq %%rbp, %0\n"
-#  else
-                          "movl %%ebp, %0\n"
-#  endif
-                          : "=g" (bp));
-#endif
-   return bp;
-}
-
-
 /*
- * Returns the frame pointer of the calling function.
- * Equivalent to __builtin_frame_address(1).
+ * A bug in __builtin_frame_address was discovered in gcc 4.1.1, and
+ * fixed in 4.2.0; assume it originated in 4.0. PR 147638 and 554369.
  */
-static INLINE_SINGLE_CALLER uintptr_t
-GetCallerFrameAddr(void)
-{
-   return *(uintptr_t*)GetFrameAddr();
-}
+#if  !(__GNUC__ == 4 && (__GNUC_MINOR__ == 0 || __GNUC_MINOR__ == 1))
+#define GetFrameAddr() __builtin_frame_address(0)
+#endif
 
 #endif // sun
 #endif // __GNUC__
@@ -558,6 +570,9 @@ typedef int pid_t;
 #if __GLIBC_PREREQ(2, 5) && !defined GLIBC_VERSION_25
 #define GLIBC_VERSION_25
 #endif
+#if __GLIBC_PREREQ(2, 12) && !defined GLIBC_VERSION_212
+#define GLIBC_VERSION_212
+#endif
 #endif
 
 /*
