@@ -78,7 +78,6 @@
 #define INCLUDE_ALLOW_VMK_MODULE
 #define INCLUDE_ALLOW_DISTRIBUTE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMIROM
 #include "includeCheck.h"
 
 /* STRICT ANSI means the Xserver build and X defines Bool differently. */
@@ -455,6 +454,9 @@ typedef uintptr_t VPN;
 typedef uint64    PA;
 typedef uint32    PPN;
 
+typedef uint64    TPA;
+typedef uint32    TPPN;
+
 typedef uint64    PhysMemOff;
 typedef uint64    PhysMemSize;
 
@@ -547,7 +549,13 @@ typedef uint64 LPN64;
 typedef uint64 PA64;
 typedef uint64 PPN64;
 typedef uint64 MA64;
+
+
+#if defined(VMKERNEL) || defined(VMKBOOT)
+typedef uint8 * MPN64;
+#else
 typedef uint64 MPN64;
+#endif
 
 /*
  * VA typedefs for user world apps.
@@ -574,14 +582,30 @@ typedef void * UserVA;
 #define INVALID_MPN       ((MPN)-1)
 #define MEMREF_MPN        ((MPN)-2)
 #define RELEASED_MPN      ((MPN)-3)
-#define MAX_MPN           ((MPN)0x7fffffff)  /* 43 bits of address space. */
+
+/* 0xfffffffc to account for special MPNs defined above. */
+#define MAX_MPN           ((MPN)0xfffffffc)    /* 44 bits of address space. */
+#define MAX_MPN64         ((MPN64)0xfffffffc)  /* 44 bits of address space. */
 
 #define INVALID_LPN       ((LPN)-1)
 #define INVALID_VPN       ((VPN)-1)
 #define INVALID_LPN64     ((LPN64)-1)
 #define INVALID_PAGENUM   ((PageNum)-1)
 
-#define INVALID_MPN64     ((MPN64)-1)
+#if defined(VMKERNEL) || defined(VMKBOOT)
+#define INVALID_MPN64     ((MPN64)(uintptr_t)INVALID_MPN)
+#else
+/*
+ * We are zero extending the 32 bit MPN in a 64 bit MPN container
+ * This should change when we actually get a host supporting 64 bit
+ * memory/address space.
+ */
+
+#define INVALID_MPN64     ((MPN64)INVALID_MPN)
+#define RESERVED_MPN64    ((MPN64)RESERVED_MPN)
+#define MEMREF_MPN64      ((MPN64)MEMREF_MPN)
+#define RELEASED_MPN64    ((MPN64)RELEASED_MPN)
+#endif
 
 /*
  * Format modifier for printing VA, LA, and VPN.
@@ -811,6 +835,11 @@ typedef void * UserVA;
 # endif
 #endif
 
+#ifndef UNUSED_TYPE
+// XXX _Pragma would better but doesn't always work right now.
+#  define UNUSED_TYPE(_parm) UNUSED_PARAM(_parm)
+#endif
+
 #ifndef UNUSED_VARIABLE
 // XXX is there a better way?
 #  define UNUSED_VARIABLE(_var) (void)_var
@@ -893,6 +922,7 @@ typedef void * UserVA;
 #         define _SSIZE_T
 #         define __ssize_t_defined
 #         define _SSIZE_T_DECLARED
+#         define _SSIZE_T_DEFINED_
 #         ifdef VM_X86_64
              typedef int64 ssize_t;
 #         else
@@ -902,6 +932,7 @@ typedef void * UserVA;
 #         define _SSIZE_T
 #         define __ssize_t_defined
 #         define _SSIZE_T_DECLARED
+#         define _SSIZE_T_DEFINED_
              typedef int32 ssize_t;
 #      endif
 #   endif
