@@ -59,6 +59,7 @@
  * Brief names are used when the VMware prefix is not wanted.
  */
 #define PRODUCT_SCALABLE_SERVER_BRIEF_NAME "ESX"
+#define PRODUCT_ESXI_BRIEF_NAME "ESXi"
 #define PRODUCT_WORKSTATION_BRIEF_NAME "Workstation"
 #define PRODUCT_WORKSTATION_ENTERPRISE_BRIEF_NAME \
          PRODUCT_WORKSTATION_BRIEF_NAME " " "ACE Edition"
@@ -448,13 +449,9 @@
 
 #   if defined(__APPLE__)
 #      if defined VMX86_DESKTOP
-/*
- * We will remove this definition soon. Fusion's library directory should not
- * be hardcoded: it prevents Fusion from being relocated. Use
- * Location_GetLibrary() instead.
- */
+/* On Mac OS, use Location_Get() instead of DEFAULT_LIBDIRECTORY. */
 #         define DEFAULT_LIBDIRECTORY \
-             "/Applications/" PRODUCT_SHORT_NAME ".app/Contents/Library"
+             "/dev/null/Non-existing DEFAULT_LIBDIRECTORY"
 #      else
 #         define DEFAULT_LIBDIRECTORY VMWARE_HOST_DIRECTORY
 #      endif
