@@ -1153,6 +1153,14 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_HOSTED12_UPDATE2_V_MNR  2
 #define   TOOLS_VERSION_HOSTED12_UPDATE2_V_BASE 2
 
+/* Workstation 9.0.2 (hosted12-pd-rel) */
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED12_UPDATE3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED12_UPDATE3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED12_UPDATE3_V_MJR  9
+#define   TOOLS_VERSION_HOSTED12_UPDATE3_V_MNR  2
+#define   TOOLS_VERSION_HOSTED12_UPDATE3_V_BASE 3
+
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_OP_BETA1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_BETA1_V)
 #endif /* RC_INVOKED */
@@ -1160,11 +1168,53 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_OP_BETA1_V_MNR  3
 #define   TOOLS_VERSION_OP_BETA1_V_BASE 1
 
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_OP_BETA2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_BETA2_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_OP_BETA2_V_MJR  9
+#define   TOOLS_VERSION_OP_BETA2_V_MNR  3
+#define   TOOLS_VERSION_OP_BETA2_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_OP_RC1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_RC1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_OP_RC1_V_MJR  9
+#define   TOOLS_VERSION_OP_RC1_V_MNR  3
+#define   TOOLS_VERSION_OP_RC1_V_BASE 3
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_OP_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_OP_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_OP_RELEASE_V_MJR  9
+#define   TOOLS_VERSION_OP_RELEASE_V_MNR  4
+#define   TOOLS_VERSION_OP_RELEASE_V_BASE 0
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V_MNR  5
+#define   TOOLS_VERSION_HOSTED13_TECHPREVIEW1_V_BASE 1
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED13_RC TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED13_RC_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_RC_V_MNR  5
+#define   TOOLS_VERSION_HOSTED13_RC_V_BASE 2
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_HOSTED13_RELEASE TOOLS_VERSION_TO_UINT(TOOLS_VERSION_HOSTED13_RELEASE_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_HOSTED13_RELEASE_V_MJR  9
+#define   TOOLS_VERSION_HOSTED13_RELEASE_V_MNR  6
+#define   TOOLS_VERSION_HOSTED13_RELEASE_V_BASE 0
+
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_NEXT TOOLS_VERSION_TO_UINT(TOOLS_VERSION_NEXT_V)
 #endif /* RC_INVOKED */
 #define   TOOLS_VERSION_NEXT_V_MJR  9
-#define   TOOLS_VERSION_NEXT_V_MNR  5
+#define   TOOLS_VERSION_NEXT_V_MNR  7
 #define   TOOLS_VERSION_NEXT_V_BASE 0
 
 #define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_NEXT
