@@ -90,10 +90,10 @@ typedef enum VMCIIntrType {
 
 
 /*
- * A single VMCI device has an upper limit of 1024MB on the amount of
+ * A single VMCI device has an upper limit of 128 MiB on the amount of
  * memory that can be used for queue pairs.
  */
-#define VMCI_MAX_GUEST_QP_MEMORY (1024 * 1024 * 1024)
+#define VMCI_MAX_GUEST_QP_MEMORY (128 * 1024 * 1024)
 
 /*
  * Queues with pre-mapped data pages must be small, so that we don't pin
@@ -117,36 +117,34 @@ typedef enum VMCIIntrType {
 typedef uint32 VMCI_Resource;
 
 /* VMCI reserved hypervisor datagram resource IDs. */
-#define VMCI_RESOURCES_QUERY      0
-#define VMCI_GET_CONTEXT_ID       1
-#define VMCI_SET_NOTIFY_BITMAP    2
-#define VMCI_DOORBELL_LINK        3
-#define VMCI_DOORBELL_UNLINK      4
-#define VMCI_DOORBELL_NOTIFY      5
+#define VMCI_RESOURCES_QUERY        0
+#define VMCI_GET_CONTEXT_ID         1
+#define VMCI_SET_NOTIFY_BITMAP      2
+#define VMCI_DOORBELL_LINK          3
+#define VMCI_DOORBELL_UNLINK        4
+#define VMCI_DOORBELL_NOTIFY        5
 /*
  * VMCI_DATAGRAM_REQUEST_MAP and VMCI_DATAGRAM_REMOVE_MAP are
  * obsoleted by the removal of VM to VM communication.
  */
-#define VMCI_DATAGRAM_REQUEST_MAP 6
-#define VMCI_DATAGRAM_REMOVE_MAP  7
-#define VMCI_EVENT_SUBSCRIBE      8
-#define VMCI_EVENT_UNSUBSCRIBE    9
-#define VMCI_QUEUEPAIR_ALLOC      10
-#define VMCI_QUEUEPAIR_DETACH     11
+#define VMCI_DATAGRAM_REQUEST_MAP   6
+#define VMCI_DATAGRAM_REMOVE_MAP    7
+#define VMCI_EVENT_SUBSCRIBE        8
+#define VMCI_EVENT_UNSUBSCRIBE      9
+#define VMCI_QUEUEPAIR_ALLOC        10
+#define VMCI_QUEUEPAIR_DETACH       11
 /*
  * VMCI_VSOCK_VMX_LOOKUP was assigned to 12 for Fusion 3.0/3.1,
  * WS 7.0/7.1 and ESX 4.1
  */
-#define VMCI_HGFS_TRANSPORT       13
-#define VMCI_UNITY_PBRPC_REGISTER 14
+#define VMCI_HGFS_TRANSPORT         13
+#define VMCI_UNITY_PBRPC_REGISTER   14
 /*
- * The next two resources are for RPC calls from guest Tools, to replace the
- * backdoor calls we used previously.  Privileged is for admin/root RPCs,
- * unprivileged is for RPCs from any user.
+ * This resource is used for VMCI socket control packets sent to the
+ * hypervisor (CID 0) because RID 1 is already reserved.
  */
-#define VMCI_RPC_PRIVILEGED       15
-#define VMCI_RPC_UNPRIVILEGED     16
-#define VMCI_RESOURCE_MAX         17
+#define VSOCK_PACKET_HYPERVISOR_RID 15
+#define VMCI_RESOURCE_MAX           16
 /*
  * The core VMCI device functionality only requires the resource IDs of
  * VMCI_QUEUEPAIR_DETACH and below.
@@ -358,8 +356,8 @@ typedef uint32 VMCI_Event;
 #define VMCI_EVENT_CTX_ID_UPDATE  0  // Only applicable to guest endpoints
 #define VMCI_EVENT_CTX_REMOVED    1  // Applicable to guest and host
 #define VMCI_EVENT_QP_RESUMED     2  // Only applicable to guest endpoints
-#define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest and host
-#define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest and host
+#define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest, host and VMX
+#define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest, host and VMX
 #define VMCI_EVENT_MEM_ACCESS_ON  5  // Applicable to VMX and vmk.  On vmk,
                                      // this event has the Context payload type.
 #define VMCI_EVENT_MEM_ACCESS_OFF 6  // Applicable to VMX and vmk.  Same as
@@ -374,14 +372,17 @@ typedef uint32 VMCI_Event;
  * endpoints.
  */
 
-#define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_MEM_ACCESS_ON || \
+#define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_QP_PEER_ATTACH || \
+                                      _event == VMCI_EVENT_QP_PEER_DETACH || \
+                                      _event == VMCI_EVENT_MEM_ACCESS_ON || \
                                       _event == VMCI_EVENT_MEM_ACCESS_OFF)
 
 #if defined(VMX86_SERVER)
 #define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX)
 #else // VMX86_SERVER
 #define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX && \
-                                  !VMCI_EVENT_VALID_VMX(_event))
+                                  _event != VMCI_EVENT_MEM_ACCESS_ON && \
+                                  _event != VMCI_EVENT_MEM_ACCESS_OFF)
 #endif // VMX86_SERVER
 
 /* Reserved guest datagram resource ids. */
