@@ -131,11 +131,23 @@ typedef gboolean (*SignalSourceCb)(const siginfo_t *, gpointer);
 GSource *
 VMTools_NewSignalSource(int signum);
 
+gchar *
+VMTools_GetLibdir(void);
+
 #endif
 
 GSource *
 VMTools_CreateTimer(gint timeout);
 
+void
+VMTools_SetGuestSDKMode(void);
+
+void
+VMTools_StopLogging(void);
+
+void
+VMTools_RestartLogging(void);
+
 GArray *
 VMTools_WrapArray(gconstpointer data,
                   guint elemSize,
