@@ -29,6 +29,7 @@
 #include "includeCheck.h"
 
 #include "vm_basic_types.h"
+#include "vm_atomic.h"
 #include "unicodeTypes.h"
 
 #ifdef _WIN32
@@ -66,7 +67,6 @@ Unicode W32Util_GetCommonAppDataFilePath(ConstUnicode fileName);
 Unicode W32Util_GetVmwareCommonAppDataFilePath(ConstUnicode fileName);
 
 Unicode W32Util_GetMyDocumentPath(void);
-Unicode W32Util_GetMyVideoPath(BOOL myDocumentsOnFail);
 
 Unicode W32Util_GetDefaultVMPath(ConstUnicode pref);
 Unicode W32Util_GetInstalledFilePath(ConstUnicode fileName);
@@ -147,11 +147,13 @@ Bool W32Util_RegisterService(Bool bRegister,
 Bool W32Util_DebugService(ConstUnicode dbgFile);
 
 Bool W32Util_RegisterEventLog(ConstUnicode serviceName,
+                              Bool registryVolatile,
                               DWORD typesSupported,
                               ConstUnicode eventMsgFile,
                               ConstUnicode categoryMsgFile,
                               DWORD categoryCount,
                               ConstUnicode paramMsgFile);
+
 Bool W32Util_UnregisterEventLog(ConstUnicode serviceName);
 
 typedef enum SetSDPrivsAccounts {
@@ -227,7 +229,11 @@ Bool W32Util_DismountVolumes(uint16 drive,
                              uint64 offset,
                              uint64 size,
                              void **handle);
-void W32Util_CloseDismountHandle(void *handle);
+Bool W32Util_CloseDismountHandle(void *handle);
+
+Bool W32Util_EnableSafePathSearching(void);
+
+CRITICAL_SECTION *W32Util_GetSingletonCriticalSection(Atomic_Ptr *csMemory);
 
 #endif // _WIN32
 #endif // WIN32UTIL_H_
