@@ -35,6 +35,8 @@ void (*MXUserMX_LockRec)(struct MX_MutexRec *lock) = NULL;
 void (*MXUserMX_UnlockRec)(struct MX_MutexRec *lock) = NULL;
 Bool (*MXUserMX_TryLockRec)(struct MX_MutexRec *lock) = NULL;
 Bool (*MXUserMX_IsLockedByCurThreadRec)(const struct MX_MutexRec *lock) = NULL;
+static void (*MXUserMX_SetInPanic)(void) = NULL;
+static Bool (*MXUserMX_InPanic)(void) = NULL;
 
 
 /*
@@ -122,36 +124,11 @@ MXUserSyndrome(void)
    syndrome = Atomic_Read(&syndromeMem);
 
    if (syndrome == 0) {
-      uint32 retries = 25;
-
-      /*
-       * Do not assume that the source of bits from the host OS are sane.
-       * Perhaps its random bits service is not working or always returns
-       * zero or something is misconfigured. Only perform a small number
-       * of retries attempting to appropriate the required bits.
-       */
-
-      do {
-         /* Only changes syndrome on success. No need to check for errors */
-         Random_Crypto(sizeof syndrome, &syndrome);
-
-         if (syndrome != 0) {
-            break;
-         }
-      } while (retries--);
-
-      /*
-       * If the source was unable to provide the appropriate bits, switch
-       * to plan B.
-       */
-
-      if (syndrome == 0) {
 #if defined(_WIN32)
-         syndrome = GetTickCount();
+      syndrome = GetTickCount();
 #else
-         syndrome = time(NULL) & 0xFFFFFFFF;
+      syndrome = time(NULL) & 0xFFFFFFFF;
 #endif
-      }
 
       /*
        * Protect against a total failure.
@@ -296,11 +273,8 @@ MXUserDumpAndPanic(MXUserHeader *header,  // IN:
  *  MXUser_SetInPanic --
  *	Notify the locking system that a panic is occurring.
  *
- *      This is the "out of the monitor" - userland - implementation. The "in
- *      the monitor" implementation lives in mutex.c.
- *
  *  Results:
- *     Set the internal "in a panic" global variable.
+ *     Set the "in a panic" state both in userland, and monitor, if applicable.
  *
  *  Side effects:
  *     None
@@ -312,6 +286,9 @@ void
 MXUser_SetInPanic(void)
 {
    mxInPanic = TRUE;
+   if (MXUserMX_SetInPanic != NULL) {
+      MXUserMX_SetInPanic();
+   }
 }
 
 
@@ -321,9 +298,6 @@ MXUser_SetInPanic(void)
  *  MXUser_InPanic --
  *	Is the caller in the midst of a panic?
  *
- *      This is the "out of the monitor" - userland - implementation. The "in
- *      the monitor" implementation lives in mutex.c.
- *
  *  Results:
  *     TRUE   Yes
  *     FALSE  No
@@ -337,7 +311,7 @@ MXUser_SetInPanic(void)
 Bool
 MXUser_InPanic(void)
 {
-   return mxInPanic;
+   return mxInPanic || (MXUserMX_InPanic != NULL && MXUserMX_InPanic());
 }
 
 
@@ -365,7 +339,9 @@ MXUserInstallMxHooks(void (*theLockListFunc)(void),
                      void (*theLockFunc)(struct MX_MutexRec *lock),
                      void (*theUnlockFunc)(struct MX_MutexRec *lock),
                      Bool (*theTryLockFunc)(struct MX_MutexRec *lock),
-                     Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock))
+                     Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock),
+                     void (*theSetInPanicFunc)(void),
+                     Bool (*theInPanicFunc)(void))
 {
    /*
     * This function can be called more than once but the second and later
@@ -378,20 +354,27 @@ MXUserInstallMxHooks(void (*theLockListFunc)(void),
        (MXUserMX_LockRec == NULL) &&
        (MXUserMX_UnlockRec == NULL) &&
        (MXUserMX_TryLockRec == NULL) &&
-       (MXUserMX_IsLockedByCurThreadRec == NULL)) {
+       (MXUserMX_IsLockedByCurThreadRec == NULL) &&
+       (MXUserMX_SetInPanic == NULL) &&
+       (MXUserMX_InPanic == NULL)
+       ) {
       MXUserMxLockLister = theLockListFunc;
       MXUserMxCheckRank = theRankFunc;
       MXUserMX_LockRec = theLockFunc;
       MXUserMX_UnlockRec = theUnlockFunc;
       MXUserMX_TryLockRec = theTryLockFunc;
       MXUserMX_IsLockedByCurThreadRec = theIsLockedFunc;
+      MXUserMX_SetInPanic = theSetInPanicFunc;
+      MXUserMX_InPanic = theInPanicFunc;
    } else {
       ASSERT((MXUserMxLockLister == theLockListFunc) &&
              (MXUserMxCheckRank == theRankFunc) &&
              (MXUserMX_LockRec == theLockFunc) &&
              (MXUserMX_UnlockRec == theUnlockFunc) &&
              (MXUserMX_TryLockRec == theTryLockFunc) &&
-             (MXUserMX_IsLockedByCurThreadRec == theIsLockedFunc)
+             (MXUserMX_IsLockedByCurThreadRec == theIsLockedFunc) &&
+             (MXUserMX_SetInPanic == theSetInPanicFunc) &&
+             (MXUserMX_InPanic == theInPanicFunc)
             );
    }
 }
