@@ -19,13 +19,26 @@
 #ifndef _ULINTSHARED_H_
 #define _ULINTSHARED_H_
 
+
+#if defined(MXUSER_DEBUG)
 extern void MXUserListLocks(void);
+#else
+static INLINE void
+MXUserListLocks(void)
+{
+   return;
+}
+#endif
+
+MX_Rank MXUserCurrentRank(void);
 
 extern void MXUserInstallMxHooks(void (*theLockListFunc)(void),
                                  MX_Rank (*theRankFunc)(void),
                                  void (*theLockFunc)(struct MX_MutexRec *lock),
                                  void (*theUnlockFunc)(struct MX_MutexRec *lock),
                                  Bool (*theTryLockFunc)(struct MX_MutexRec *lock),
-                                 Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock));
+                                 Bool (*theIsLockedFunc)(const struct MX_MutexRec *lock),
+                                 void (*theSetInPanicFunc)(void),
+                                 Bool (*theInPanicFunc)(void));
 
 #endif
