@@ -453,7 +453,7 @@ Hostinfo_GetSystemBitness(void)
 }
 
 
-#if !defined __APPLE__
+#if !defined __APPLE__ && !defined USERWORLD
 /*
  *-----------------------------------------------------------------------------
  *
@@ -579,6 +579,8 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
          Str_Strcpy(distroShort, STR_OS_DEBIAN_6, distroShortSize);
       } else if (strstr(distroLower, "7.0")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_7, distroShortSize);
+      } else if (strstr(distroLower, "7.1")) {
+         Str_Strcpy(distroShort, STR_OS_DEBIAN_7_1, distroShortSize);
       }
    } else if (StrUtil_StartsWith(distroLower, "enterprise linux") ||
               StrUtil_StartsWith(distroLower, "oracle")) {
@@ -742,6 +744,7 @@ out:
 #endif
 
 
+#ifndef USERWORLD
 /*
  *----------------------------------------------------------------------
  *
@@ -761,17 +764,27 @@ out:
 static char *
 HostinfoGetCmdOutput(const char *cmd)  // IN:
 {
+   Bool isSuperUser = FALSE;
    DynBuf db;
    FILE *stream;
    char *out = NULL;
 
+   /*
+    * Attempt to lower privs, because we use popen and an attacker
+    * may control $PATH.
+    */
+   if (vmx86_linux && Id_IsSuperUser()) {
+      Id_EndSuperUser(getuid());
+      isSuperUser = TRUE;
+   }
+
    DynBuf_Init(&db);
 
    stream = Posix_Popen(cmd, "r");
    if (stream == NULL) {
       Warning("Unable to get output of command \"%s\"\n", cmd);
 
-      return NULL;
+      goto exit;
    }
 
    for (;;) {
@@ -805,13 +818,19 @@ HostinfoGetCmdOutput(const char *cmd)  // IN:
    if (DynBuf_Get(&db)) {
       out = (char *) DynBuf_AllocGet(&db);
    }
- closeIt:
-   DynBuf_Destroy(&db);
 
+ closeIt:
    pclose(stream);
 
+ exit:
+   DynBuf_Destroy(&db);
+
+   if (isSuperUser) {
+      Id_BeginSuperUser();
+   }
    return out;
 }
+#endif
 
 
 /*
@@ -860,7 +879,15 @@ HostinfoOSData(void)
    Str_Sprintf(osNameFull, sizeof osNameFull, "%s %s", buf.sysname,
                buf.release);
 
-#if defined __APPLE__
+#if defined USERWORLD
+   if (buf.release[0] <= '4') {
+      Str_Strcpy(osName, "vmkernel", sizeof osName);
+   } else {
+      Str_Strcpy(osName, "vmkernel5", sizeof osName);
+   }
+   Str_Snprintf(osNameFull, sizeof osNameFull, "VMware ESXi %c.x",
+                buf.release[0]);
+#elif defined __APPLE__
    {
       /*
        * The easiest way is to invoke "system_profiler" and hope that the
@@ -928,7 +955,7 @@ HostinfoOSData(void)
        * Try to get OS detailed information from the lsb_release command.
        */
 
-      lsbOutput = HostinfoGetCmdOutput("lsb_release -sd 2>/dev/null");
+      lsbOutput = HostinfoGetCmdOutput("/usr/bin/lsb_release -sd 2>/dev/null");
       if (!lsbOutput) {
          int i;
 
@@ -1035,6 +1062,7 @@ HostinfoOSData(void)
    }
 #endif
 
+#ifndef USERWORLD
    if (Hostinfo_GetSystemBitness() == 64) {
       if (strlen(osName) + sizeof STR_OS_64BIT_SUFFIX > sizeof osName) {
          Warning("%s: Error: buffer too small\n", __FUNCTION__);
@@ -1043,6 +1071,7 @@ HostinfoOSData(void)
       }
       Str_Strcat(osName, STR_OS_64BIT_SUFFIX, sizeof osName);
    }
+#endif
 
    /*
     * Before returning, truncate the \n character at the end of the full name.
