@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -63,7 +63,7 @@ static int GetpwInternal_r(struct passwd *pw, char *buf, size_t size,
  */
 
 struct passwd *
-Posix_Getpwnam(const char *name)  // IN:
+Posix_Getpwnam(ConstUnicode name)  // IN:
 {
    struct passwd *pw;
    char *tmpname;
@@ -616,7 +616,7 @@ EmulateGetgrnam_r(const char *name,       // IN:
  */
 
 int
-Posix_Getpwnam_r(const char *name,     // IN:
+Posix_Getpwnam_r(ConstUnicode name,    // IN:
                  struct passwd *pw,    // IN:
                  char *buf,            // IN:
                  size_t size,          // IN:
@@ -858,10 +858,10 @@ exit:
  */
 
 int
-Posix_GetGroupList(const char *user,  // IN:
-                   gid_t group,       // IN:
-                   gid_t *groups,     // OUT:
-                   int *ngroups)      // IN/OUT:
+Posix_GetGroupList(ConstUnicode user,  // IN:
+                   gid_t group,        // IN:
+                   gid_t *groups,      // OUT:
+                   int *ngroups)       // IN/OUT:
 {
    char *tmpuser;
    int ret;
@@ -917,7 +917,7 @@ Posix_GetGroupList(const char *user,  // IN:
  */
 
 struct group *
-Posix_Getgrnam(const char *name)  // IN:
+Posix_Getgrnam(ConstUnicode name)  // IN:
 {
    struct group *gr;
    char *tmpname;
@@ -940,7 +940,7 @@ Posix_Getgrnam(const char *name)  // IN:
    free(sgr.gr_passwd);
    sgr.gr_passwd = NULL;
    if (sgr.gr_mem != NULL) {
-      Util_FreeStringList(sgr.gr_mem, -1);
+      Unicode_FreeList(sgr.gr_mem, -1);
       sgr.gr_mem = NULL;
    }
 
@@ -993,7 +993,7 @@ Posix_Getgrnam(const char *name)  // IN:
  */
 
 int
-Posix_Getgrnam_r(const char *name,     // IN:
+Posix_Getgrnam_r(ConstUnicode name,    // IN:
                  struct group *gr,     // IN:
                  char *buf,            // IN:
                  size_t size,          // IN:
@@ -1101,7 +1101,7 @@ Posix_Getgrnam_r(const char *name,     // IN:
    free(grpasswd);
    free(grname);
    if (grmem) {
-      Util_FreeStringList(grmem, -1);
+      Unicode_FreeList(grmem, -1);
    }
 
    return ret;
