@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2003 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -309,31 +309,6 @@ Poll_CallbackRemoveOneByCB(PollClassSet classSet,
 }
 
 
-/*
- *----------------------------------------------------------------------------
- *
- * Poll_NotifyChange --
- *
- *      Wake up a sleeping Poll_LoopTimeout() when there is a change
- *      it should notice, and no normal event can be expected to wake
- *      it up in a timely manner.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-Poll_NotifyChange(PollClassSet classSet)  // IN
-{
-   pollImpl->NotifyChange(classSet);
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1067,7 +1042,7 @@ PollAddRemoveCBThread(void *clientData)  // IN: unused
       /* Add and remove real time and main loop callbacks */
       cbLock = MXUser_CreateRecLock("pollUnitTestLock",
                                     RANK_pollUnitTestLock);
-      VERIFY(cbLock);
+      ASSERT_NOT_IMPLEMENTED(cbLock);
       rtDeleted = FALSE;
       Poll_Callback(POLL_CS_MAIN,
                     periodicFlag[oddIter],
@@ -1119,7 +1094,7 @@ PollAddRemoveCBThread(void *clientData)  // IN: unused
       /* Add and remove device callbacks */
       cbLock = MXUser_CreateRecLock("pollUnitTestLock",
                                     RANK_pollUnitTestLock);
-      VERIFY(cbLock);
+      ASSERT_NOT_IMPLEMENTED(cbLock);
       drDeleted = FALSE;
       Poll_Callback(POLL_CS_MAIN,
                     POLL_FLAG_SOCKET | POLL_FLAG_READ | periodicFlag[oddIter],
@@ -1942,7 +1917,7 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
          useLocking = TRUE;
          cbLock = MXUser_CreateRecLock("pollUnitTestLock",
                                        RANK_pollUnitTestLock);
-         VERIFY(cbLock);
+         ASSERT_NOT_IMPLEMENTED(cbLock);
 
    #ifdef _WIN32
          /* Discard sockets used in connect test and re-create them. */
@@ -2063,7 +2038,7 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
          /* The previous test may have destroyed the lock. */
          cbLock = MXUser_CreateRecLock("pollUnitTestLock",
                                        RANK_pollUnitTestLock);
-         VERIFY(cbLock);
+         ASSERT_NOT_IMPLEMENTED(cbLock);
       }
       deviceEv0Count = 0;
       deviceEv1Count = 0;
@@ -2079,7 +2054,7 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
          break;
       }
       events[0] = CreateEvent(NULL, FALSE, TRUE, NULL);
-      VERIFY(events[0]);
+      ASSERT_NOT_IMPLEMENTED(events[0]);
       Poll_Callback(POLL_CS_MAIN,
                     POLL_FLAG_READ | POLL_FLAG_PERIODIC,
                     PollUnitTest_DeviceEvent
@@ -2088,7 +2063,7 @@ PollUnitTest_StateMachine(void *clientData) // IN: Unused
                     (PollDevHandle)events[0],
                     cbLock);
       events[1] = CreateEvent(NULL, FALSE, TRUE, NULL);
-      VERIFY(events[1]);
+      ASSERT_NOT_IMPLEMENTED(events[1]);
       Poll_Callback(POLL_CS_MAIN,
                     POLL_FLAG_READ,
                     PollUnitTest_DeviceEvent
