@@ -33,9 +33,12 @@
 #include <unistd.h>
 #if !defined(__FreeBSD__) && !defined(sun) && !defined(__APPLE__)
 #include <asm/param.h>
+#endif
+#if !defined(sun) && !defined(__APPLE__)
 #include <locale.h>
 #include <sys/stat.h>
 #endif
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -48,13 +51,23 @@
 #include <time.h>
 #include <grp.h>
 #include <sys/syscall.h>
-#if defined(linux) || defined(HAVE_SYS_USER_H)
+#if defined(linux) || defined(__FreeBSD__) || defined(HAVE_SYS_USER_H)
 // sys/param.h is required on FreeBSD before sys/user.h
 #   include <sys/param.h>
 // Pull in PAGE_SIZE/PAGE_SHIFT defines ahead of vm_basic_defs.h
 #   include <sys/user.h>
 #endif
-
+#if defined (__FreeBSD__)
+#include <kvm.h>
+#include <limits.h>
+#include <paths.h>
+#include <sys/sysctl.h>
+#endif
+#if defined(__APPLE__)
+#include <sys/sysctl.h>
+#include <sys/user.h>
+#include "posix.h"
+#endif
 #include "vmware.h"
 #include "procMgr.h"
 #include "vm_assert.h"
@@ -65,8 +78,10 @@
 #undef offsetof
 #include "file.h"
 #include "dynbuf.h"
+#include "dynarray.h"
 #include "su.h"
 #include "str.h"
+#include "strutil.h"
 #include "fileIO.h"
 #include "codeset.h"
 #include "unicode.h"
@@ -110,11 +125,19 @@ static Bool ProcMgrKill(pid_t pid,
                         int sig,
                         int timeout);
 
-#ifdef sun
-#define  SOLARIS_BASH_PATH "/usr/bin/bash"
+#if defined(__APPLE__)
+static int ProcMgrGetCommandLineArgs(long pid,
+                                     DynBuf *argsBuf,
+                                     char **procCmdName);
 #endif
 
-#if defined(linux) && !defined(GLIBC_VERSION_23)
+#ifdef sun
+#define  BASH_PATH "/usr/bin/bash"
+#else
+#define  BASH_PATH "/bin/bash"
+#endif
+
+#if defined(linux) && !defined(GLIBC_VERSION_23) && !defined(__UCLIBC__)
 /*
  * Implements the system calls (they are not wrapped by glibc til 2.3.2).
  *
@@ -171,13 +194,12 @@ setresgid(gid_t ruid,
  *----------------------------------------------------------------------
  */
 
-#if !defined(sun)
+#if defined(linux)
 int
 ProcMgr_ReadProcFile(int fd,                       // IN
                      char **contents)              // OUT
 {
    int size = 0;
-#if !defined(__FreeBSD__) && !defined(__APPLE__)
    char tmp[512];
    int numRead;
 
@@ -223,11 +245,9 @@ ProcMgr_ReadProcFile(int fd,                       // IN
       DynBuf_Destroy(&dbuf);
    }
 done:
-#endif
    return size;
 }
 
-#endif   // !sun
 
 /*
  *----------------------------------------------------------------------
@@ -240,34 +260,30 @@ done:
  *
  * Results:
  *      
- *      A ProcMgr_ProcList.
+ *      A ProcMgrProcInfoArray.
  *
  * Side effects:
  *
  *----------------------------------------------------------------------
  */
 
-#if !defined(sun)
-ProcMgr_ProcList *
+ProcMgrProcInfoArray *
 ProcMgr_ListProcesses(void)
 {
-   ProcMgr_ProcList *procList = NULL;
-#if !defined(__FreeBSD__) && !defined(__APPLE__)
-   Bool failed = FALSE;
-   DynBuf dbProcId;
-   DynBuf dbProcCmd;
-   DynBuf dbProcStartTime;
-   DynBuf dbProcOwner;
+   ProcMgrProcInfoArray *procList = NULL;
+   ProcMgrProcInfo procInfo;
+   Bool failed = TRUE;
    DIR *dir;
    struct dirent *ent;
    static time_t hostStartTime = 0;
    static unsigned long long hertz = 100;
    int numberFound;
 
-   DynBuf_Init(&dbProcId);
-   DynBuf_Init(&dbProcCmd);
-   DynBuf_Init(&dbProcStartTime);
-   DynBuf_Init(&dbProcOwner);
+   procList = Util_SafeCalloc(1, sizeof *procList);
+   ProcMgrProcInfoArray_Init(procList, 0);
+   procInfo.procCmdName = NULL;
+   procInfo.procCmdLine = NULL;
+   procInfo.procOwner = NULL;
 
    /*
     * Figure out when the system started.  We need this number to
@@ -323,7 +339,6 @@ ProcMgr_ListProcesses(void)
    dir = opendir("/proc");
    if (NULL == dir) {
       Warning("ProcMgr_ListProcesses unable to open /proc\n");
-      failed = TRUE;
       goto abort;
    }
 
@@ -333,18 +348,16 @@ ProcMgr_ListProcesses(void)
       int statResult;
       int numRead = 0;   /* number of bytes that read() actually read */
       int cmdFd;
-      pid_t pid;
       int replaceLoop;
       struct passwd *pwd;
       char *cmdLineTemp = NULL;
       char *cmdStatTemp = NULL;
-      char *cmdLine;
-      char *userName = NULL;
       size_t strLen = 0;
       unsigned long long dummy;
       unsigned long long relativeStartTime;
       char *stringBegin;
-      time_t processStartTime;
+      char *cmdNameBegin;
+      Bool cmdNameLookup = TRUE;
 
       /*
        * We only care about dirs that look like processes.
@@ -396,6 +409,24 @@ ProcMgr_ListProcesses(void)
           */
          for (replaceLoop = 0 ; replaceLoop < (numRead - 1) ; replaceLoop++) {
             if ('\0' == cmdLineTemp[replaceLoop]) {
+               if (cmdNameLookup) {
+                  /*
+                   * Store the command name.
+                   * Find the last path separator, to get the cmd name.
+                   * If no separator is found, then use the whole name.
+                   */
+                  cmdNameBegin = strrchr(cmdLineTemp, '/');
+                  if (NULL == cmdNameBegin) {
+                     cmdNameBegin = cmdLineTemp;
+                  } else {
+                     /*
+                      * Skip over the last separator.
+                      */
+                     cmdNameBegin++;
+                  }
+                  procInfo.procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+                  cmdNameLookup = FALSE;
+               }
                cmdLineTemp[replaceLoop] = ' ';
             }
          }
@@ -446,6 +477,10 @@ ProcMgr_ListProcesses(void)
                *(copyItr++) = *(nameStart++);
             }
             *copyItr = '\0';
+            /*
+             * Store the command name.
+             */
+            procInfo.procCmdName = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
          }
       }
 
@@ -514,100 +549,639 @@ ProcMgr_ListProcesses(void)
       if (20 != numberFound) {
          goto next_entry;
       }
-      processStartTime = hostStartTime + (relativeStartTime / hertz);
 
       /*
        * Store the command line string pointer in dynbuf.
        */
       if (cmdLineTemp) {
-         cmdLine = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
+         procInfo.procCmdLine = Unicode_Alloc(cmdLineTemp, STRING_ENCODING_DEFAULT);
       } else {
-         cmdLine = Unicode_Alloc("", STRING_ENCODING_UTF8);
+         procInfo.procCmdLine = Unicode_Alloc("", STRING_ENCODING_UTF8);
       }
-      DynBuf_Append(&dbProcCmd, &cmdLine, sizeof cmdLine);
 
       /*
        * Store the pid in dynbuf.
        */
-      pid = (pid_t) atoi(ent->d_name);
-      DynBuf_Append(&dbProcId, &pid, sizeof pid);
+      procInfo.procId = (pid_t) atoi(ent->d_name);
 
       /*
        * Store the owner of the process.
        */
       pwd = getpwuid(fileStat.st_uid);
-      userName = (NULL == pwd)
-                 ? Str_Asprintf(&strLen, "%d", (int) fileStat.st_uid)
-                 : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
-      DynBuf_Append(&dbProcOwner, &userName, sizeof userName);
+      procInfo.procOwner = (NULL == pwd)
+                           ? Str_SafeAsprintf(&strLen, "%d", (int) fileStat.st_uid)
+                           : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
 
       /*
        * Store the time that the process started.
        */
-      DynBuf_Append(&dbProcStartTime,
-                    &processStartTime,
-                    sizeof processStartTime);
+      procInfo.procStartTime = hostStartTime + (relativeStartTime / hertz);
+
+      /*
+       * Store the process info pointer into a list buffer.
+       */
+      if (!ProcMgrProcInfoArray_Push(procList, procInfo)) {
+         Warning("%s: failed to expand DynArray - out of memory\n",
+                 __FUNCTION__);
+         goto abort;
+      }
+      procInfo.procCmdName = NULL;
+      procInfo.procCmdLine = NULL;
+      procInfo.procOwner = NULL;
 
 next_entry:
       free(cmdLineTemp);
       free(cmdStatTemp);
    } // while readdir
 
-   closedir(dir);
-
-   if (0 == DynBuf_GetSize(&dbProcId)) {
-      failed = TRUE;
-      goto abort;
+   if (0 < ProcMgrProcInfoArray_Count(procList)) {
+      failed = FALSE;
    }
 
-   /*
-    * We're done adding to DynBuf.  Trim off any unused allocated space.
-    * DynBuf_Trim() followed by DynBuf_Detach() avoids a memcpy().
-    */
-   DynBuf_Trim(&dbProcId);
-   DynBuf_Trim(&dbProcCmd);
-   DynBuf_Trim(&dbProcStartTime);
-   DynBuf_Trim(&dbProcOwner);
-   /*
-    * Create a ProcMgr_ProcList and populate its fields.
-    */
-   procList = (ProcMgr_ProcList *) calloc(1, sizeof(ProcMgr_ProcList));
-   ASSERT_NOT_IMPLEMENTED(procList);
-
-   procList->procCount = DynBuf_GetSize(&dbProcId) / sizeof(pid_t);
-
-   procList->procIdList = (pid_t *) DynBuf_Detach(&dbProcId);
-   ASSERT_NOT_IMPLEMENTED(procList->procIdList);
-   procList->procCmdList = (char **) DynBuf_Detach(&dbProcCmd);
-   ASSERT_NOT_IMPLEMENTED(procList->procCmdList);
-   procList->startTime = (time_t *) DynBuf_Detach(&dbProcStartTime);
-   ASSERT_NOT_IMPLEMENTED(procList->startTime);
-   procList->procOwnerList = (char **) DynBuf_Detach(&dbProcOwner);
-   ASSERT_NOT_IMPLEMENTED(procList->procOwnerList);
-
 abort:
-   DynBuf_Destroy(&dbProcId);
-   DynBuf_Destroy(&dbProcCmd);
-   DynBuf_Destroy(&dbProcStartTime);
-   DynBuf_Destroy(&dbProcOwner);
+   closedir(dir);
+
+   free(procInfo.procCmdName);
+   free(procInfo.procCmdLine);
+   free(procInfo.procOwner);
 
    if (failed) {
       ProcMgr_FreeProcList(procList);
       procList = NULL;
    }
-#endif // !defined(__FreeBSD__) && !defined(__APPLE__)
 
    return procList;
 }
-#endif // !defined(sun)
+#endif // defined(linux)
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * ProcMgr_ListProcesses --
+ *
+ *      List all the processes that the calling client has privilege to
+ *      enumerate. The strings in the returned structure should be all
+ *      UTF-8 encoded, although we do not enforce it right now.
+ *
+ * Results:
+ *
+ *      A ProcMgrProcInfoArray.
+ *
+ * Side effects:
+ *
+ *----------------------------------------------------------------------
+ */
+
+#if defined(__FreeBSD__)
+ProcMgrProcInfoArray *
+ProcMgr_ListProcesses(void)
+{
+   ProcMgrProcInfoArray *procList = NULL;
+   ProcMgrProcInfo procInfo;
+   Bool failed = TRUE;
+   static kvm_t *kd;
+   struct kinfo_proc *kp;
+   char errbuf[_POSIX2_LINE_MAX];
+   int i;
+   int nentries=-1;
+   int flag=0;
+
+   procList = Util_SafeCalloc(1, sizeof *procList);
+   procInfo.procCmdName = NULL;
+   procInfo.procCmdLine = NULL;
+   procInfo.procOwner = NULL;
+
+   /*
+    * Get the handle to the Kernel Virtual Memory
+    */
+   kd = kvm_openfiles(NULL, _PATH_DEVNULL, NULL, O_RDONLY, errbuf);
+   if (kd == NULL) {
+      Warning("%s: failed to open kvm with error: %s\n", __FUNCTION__, errbuf);
+      goto abort;
+   }
+
+   /*
+    * Get the list of process info structs
+    */
+   kp = kvm_getprocs(kd, KERN_PROC_PROC, flag, &nentries);
+   if (kp == NULL || nentries <= 0) {
+      Warning("%s: failed to get proc infos with error: %s\n",
+              __FUNCTION__, kvm_geterr(kd));
+      goto abort;
+   }
+
+   /*
+    * Pre-allocate the dynamic array of required size.
+    */
+   if (!ProcMgrProcInfoArray_Init(procList, nentries)) {
+      Warning("%s: failed to create DynArray - out of memory\n",
+              __FUNCTION__);
+      goto abort;
+   }
+
+   /*
+    * Iterate through the list of process entries
+    */
+   for (i = 0; i < nentries; ++i, ++kp) {
+      struct passwd *pwd;
+      char **cmdLineTemp = NULL;
+      char *cmdNameBegin = NULL;
+      Bool cmdNameLookup = TRUE;
+
+      /*
+       * Store the pid of the process.
+       */
+      procInfo.procId = kp->ki_pid;
+
+      /*
+       * Store the owner of the process.
+       */
+      pwd = getpwuid(kp->ki_uid);
+      procInfo.procOwner = (NULL == pwd)
+                           ? Str_SafeAsprintf(NULL, "%d", (int) kp->ki_uid)
+                           : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
+
+      /*
+       * If the command name in the kinfo_proc struct is strictly less than the
+       * maximum allowed size, then we can save it right now. Else we shall
+       * need to try and parse it from the entire command line.
+       */
+      if (strlen(kp->ki_comm) + 1 < sizeof kp->ki_comm) {
+         procInfo.procCmdName = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+         cmdNameLookup = FALSE;
+      }
+
+      /*
+       * Store the command line string of the process.
+       */
+      cmdLineTemp = kvm_getargv(kd, kp, 0);
+      if (cmdLineTemp != NULL) {
+         /*
+          * Flatten the argument list to get cmd & all params.
+          */
+         DynBuf dbuf;
+
+         DynBuf_Init(&dbuf);
+         while (*cmdLineTemp != NULL) {
+            if (!DynBuf_Append(&dbuf, *cmdLineTemp, strlen(*cmdLineTemp))) {
+               Warning("%s: failed to append cmd/args in DynBuf - no memory\n",
+                       __FUNCTION__);
+               goto abort;
+            }
+            if (cmdNameLookup) {
+               /*
+                * Store the command name of the process.
+                * Find the last path separator, to get the cmd name.
+                * If no separator is found, then use the whole name.
+                */
+               cmdNameBegin = strrchr(*cmdLineTemp, '/');
+               if (NULL == cmdNameBegin) {
+                  cmdNameBegin = *cmdLineTemp;
+               } else {
+                  /*
+                   * Skip over the last separator.
+                   */
+                  cmdNameBegin++;
+               }
+               procInfo.procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+               cmdNameLookup = FALSE;
+            }
+            cmdLineTemp++;
+            if (*cmdLineTemp != NULL) {
+               /*
+                * Add the whitespace between arguments.
+                */
+               if (!DynBuf_Append(&dbuf, " ", 1)) {
+                  Warning("%s: failed to append ' ' in DynBuf - no memory\n",
+                          __FUNCTION__);
+                  goto abort;
+               }
+            }
+         }
+         /*
+          * Add the NUL term.
+          */
+         if (!DynBuf_Append(&dbuf, "", 1)) {
+            Warning("%s: failed to append NUL in DynBuf - out of memory\n",
+                    __FUNCTION__);
+            goto abort;
+         }
+         DynBuf_Trim(&dbuf);
+         procInfo.procCmdLine = DynBuf_Detach(&dbuf);
+         DynBuf_Destroy(&dbuf);
+      } else {
+         procInfo.procCmdLine = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+         if (cmdNameLookup) {
+            procInfo.procCmdName = Unicode_Alloc(kp->ki_comm, STRING_ENCODING_DEFAULT);
+            cmdNameLookup = FALSE;
+         }
+      }
+
+      /*
+       * Store the start time of the process
+       */
+      procInfo.procStartTime = kp->ki_start.tv_sec;
+
+      /*
+       * Store the process info pointer into a list buffer.
+       */
+      *ProcMgrProcInfoArray_AddressOf(procList, i) = procInfo;
+      procInfo.procCmdLine = NULL;
+      procInfo.procCmdName = NULL;
+      procInfo.procOwner = NULL;
+
+   } // for nentries
+
+   failed = FALSE;
+
+abort:
+   if (kd != NULL) {
+      kvm_close(kd);
+   }
+
+   free(procInfo.procCmdLine);
+   free(procInfo.procCmdName);
+   free(procInfo.procOwner);
+
+   if (failed) {
+      ProcMgr_FreeProcList(procList);
+      procList = NULL;
+   }
+
+   return procList;
+}
+#endif // defined(__FreeBSD__)
+
+
+#if defined(__APPLE__)
+/*
+ *----------------------------------------------------------------------
+ *
+ * ProcMgrGetCommandLineArgs --
+ *
+ *      Fetch all the command line arguments for a given process id.
+ *      The argument names shall all be UTF-8 encoded.
+ *
+ * Results:
+ *      Number of arguments retrieved.
+ *      Buffer is returned with argument names, if any.
+ *      Command name is returned.
+ *
+ * Side effects:
+ *
+ *----------------------------------------------------------------------
+ */
+
+static int
+ProcMgrGetCommandLineArgs(long pid,               // IN:  process id
+                          DynBuf *argsBuf,        // OUT: Buffer with arguments
+                          char **procCmdName)     // OUT: Command name string
+{
+   int argCount = 0;
+   int argNum;
+   char *argUnicode = NULL;
+   char *cmdNameBegin = NULL;
+   char *cmdLineRaw = NULL;
+   char *cmdLineTemp;
+   char *cmdLineEnd;
+   size_t maxargs = 0;
+   size_t maxargsSize;
+   int maxargsName[] = {CTL_KERN, KERN_ARGMAX};
+   int argName[] = {CTL_KERN, KERN_PROCARGS2, pid};
+   Bool cmdNameLookup = TRUE;
+   Bool failed = TRUE;
+
+   if (NULL != procCmdName) {
+      *procCmdName = NULL;
+   }
+
+   /*
+    * Get the sysctl kern argmax.
+    */
+   maxargsSize = sizeof maxargs;
+   if (sysctl(maxargsName, ARRAYSIZE(maxargsName),
+              &maxargs, &maxargsSize, NULL, 0) < 0) {
+      Warning("%s: failed to get the kernel max args with errno = %d\n",
+               __FUNCTION__, errno);
+      goto abort;
+   }
+
+   /*
+    * Fetch the raw command line
+    */
+   cmdLineRaw = Util_SafeCalloc(maxargs, sizeof *cmdLineRaw);
+   if (sysctl(argName, ARRAYSIZE(argName), cmdLineRaw, &maxargs, NULL, 0) < 0) {
+      Debug("%s: No command line args for pid = %ld\n", __FUNCTION__, pid);
+      goto abort;
+   }
+   cmdLineEnd = &cmdLineRaw[maxargs];
+
+   /*
+    * Format of the raw command line (without line breaks):
+    * <argc value><full command path>
+    * <one or more '\0' for alignment of first arg>
+    * <arg-0 = command as typed><'\0'>
+    * <arg-1><'\0'>... <arg-(argc-1))><'\0'>
+    * <env-0><'\0'>... <env-n><'\0'>
+    * where:
+    * arg = command line args we want.
+    * env = environment vars we ignore.
+    */
+
+   /*
+    * Save the number of arguments.
+    */
+   memcpy(&argNum, cmdLineRaw, sizeof argNum);
+   if (0 >= argNum) {
+      Debug("%s: Invalid number of command line args (=%d) for pid = %ld\n",
+             __FUNCTION__, argNum, pid);
+      goto abort;
+   }
+
+   /*
+    * Skip over the number of args (argc value) and
+    * full path to command name in the command line.
+    * (Please refer to format in comment above.)
+    */
+   cmdLineTemp = cmdLineRaw + sizeof argNum;
+   cmdLineTemp += strlen(cmdLineTemp) + 1;
+
+   /*
+    * Save the arguments one by one
+    */
+   while (cmdLineTemp < cmdLineEnd && argCount < argNum) {
+      /*
+       * Skip over leading '\0' chars to reach new arg.
+       */
+      while (cmdLineTemp < cmdLineEnd && '\0' == *cmdLineTemp) {
+         ++cmdLineTemp;
+      }
+      /*
+       * If we are pointing to a valid arg, save it.
+       */
+      if (cmdLineTemp < cmdLineEnd && '\0' != *cmdLineTemp) {
+         /*
+          * KERN_PROCARGS2 is not guaranteed to provide argument names in UTF-8.
+          * As long as we find UTF-8 argument names, we keep adding to our list.
+          * As soon as we see any non UTF-8 argument, we ignore that argument
+          * and return the list we have built so far.
+          * NOTE: On MacOS, STRING_ENCODING_DEFAULT will default to UTF-8.
+          */
+         if (Unicode_IsBufferValid(cmdLineTemp, -1, STRING_ENCODING_DEFAULT) &&
+             (argUnicode = Unicode_Alloc(cmdLineTemp,
+                                          STRING_ENCODING_DEFAULT)) != NULL) {
+            /*
+             * Add the whitespace between arguments.
+             */
+            if (0 < argCount) {
+               if (!DynBuf_Append(argsBuf, " ", 1)) {
+                  Warning("%s: failed to append ' ' in DynBuf\
+                           - no memory\n", __FUNCTION__);
+                  goto abort;
+               }
+            }
+            /*
+             * Add the argument.
+             */
+            if (!DynBuf_Append(argsBuf, argUnicode, strlen(argUnicode))) {
+               Warning("%s: failed to append cmd/args in DynBuf\
+                        - no memory\n", __FUNCTION__);
+               goto abort;
+            }
+            free(argUnicode);
+            argUnicode = NULL;
+         } else {
+            break;
+         }
+         ++argCount;
+         /*
+          * If this is the command name, retrieve it.
+          */
+         if (NULL != procCmdName && cmdNameLookup) {
+            /*
+             * Store the command name of the process.
+             * Find the last path separator, to get the cmd name.
+             * If no separator is found, then use the whole name.
+             */
+            cmdNameBegin = strrchr(cmdLineTemp, '/');
+            if (NULL == cmdNameBegin) {
+               cmdNameBegin = cmdLineTemp;
+            } else {
+               /*
+                * Skip over the last separator.
+                */
+               cmdNameBegin++;
+            }
+            *procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+            cmdNameLookup = FALSE;
+         }
+      }
+      /*
+       * Skip over the current arg that we just saved.
+       */
+      while (cmdLineTemp < cmdLineEnd && '\0' != *cmdLineTemp) {
+         ++cmdLineTemp;
+      }
+   }
+
+   /*
+    * Add the NULL term.
+    */
+   if (!DynBuf_Append(argsBuf, "", 1)) {
+      Warning("%s: failed to append NUL in DynBuf - out of memory\n",
+              __FUNCTION__);
+      goto abort;
+   }
+   DynBuf_Trim(argsBuf);
+
+   failed = FALSE;
+abort:
+   free(cmdLineRaw);
+   free(argUnicode);
+   if (failed) {
+      if (NULL != procCmdName) {
+         free(*procCmdName);
+         *procCmdName = NULL;
+      }
+      argCount = 0;
+   }
+
+   return argCount;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ProcMgr_ListProcesses --
+ *
+ *      List all the processes that the calling client has privilege to
+ *      enumerate. The strings in the returned structure should be all
+ *      UTF-8 encoded, although we do not enforce it right now.
+ *
+ * Results:
+ *      A ProcMgrProcInfoArray.
+ *
+ * Side effects:
+ *
+ *----------------------------------------------------------------------
+ */
+
+ProcMgrProcInfoArray *
+ProcMgr_ListProcesses(void)
+{
+   ProcMgrProcInfoArray *procList = NULL;
+   ProcMgrProcInfo procInfo;
+   Bool failed = TRUE;
+   struct kinfo_proc *kptmp;
+   struct kinfo_proc *kp = NULL;
+   size_t procsize;
+   int i;
+   int nentries;
+   int procName[] = {CTL_KERN, KERN_PROC, KERN_PROC_ALL};
+
+   procList = Util_SafeCalloc(1, sizeof *procList);
+   procInfo.procCmdLine = NULL;
+   procInfo.procCmdName = NULL;
+   procInfo.procOwner = NULL;
+
+   /*
+    * Get the number of process info structs in the entire list.
+    */
+   if (sysctl(procName, ARRAYSIZE(procName), NULL, &procsize, NULL, 0) < 0) {
+      Warning("%s: failed to get the size of the process struct\
+               list with errno = %d\n", __FUNCTION__, errno);
+      goto abort;
+   }
+   nentries = (int)(procsize / sizeof *kp);
+
+   /*
+    * Get the list of process info structs.
+    */
+   kp = Util_SafeCalloc(nentries, sizeof *kp);
+   if (sysctl(procName, ARRAYSIZE(procName), kp, &procsize, NULL, 0) < 0) {
+      Warning("%s: failed to get the process struct list (errno = %d)\n",
+               __FUNCTION__, errno);
+      goto abort;
+   }
+
+   /*
+    * Recalculate the number of entries as they may have changed.
+    */
+   if (0 >= (nentries = (int)(procsize / sizeof *kp))) {
+      goto abort;
+   }
+
+   /*
+    * Pre-allocate the dynamic array of required size.
+    */
+   if (!ProcMgrProcInfoArray_Init(procList, nentries)) {
+      Warning("%s: failed to create DynArray - out of memory\n",
+              __FUNCTION__);
+      goto abort;
+   }
+
+   /*
+    * Iterate through the list of process entries
+    */
+   for (i = 0, kptmp = kp; i < nentries; ++i, ++kptmp) {
+      DynBuf argsBuf;
+      char buffer[BUFSIZ];
+      struct passwd pw;
+      struct passwd *ppw = &pw;
+      int error;
+      int argCount = 0;
+      Bool cmdNameLookup = TRUE;
+
+      /*
+       * Store the pid of the process
+       */
+      procInfo.procId = kptmp->kp_proc.p_pid;
+
+      /*
+       * Store the owner of the process.
+       */
+      error = Posix_Getpwuid_r(kptmp->kp_eproc.e_pcred.p_ruid,
+                               &pw, buffer, sizeof buffer, &ppw);
+      procInfo.procOwner = (0 != error || NULL == ppw)
+                           ? Str_SafeAsprintf(NULL, "%d", (int) kptmp->kp_eproc.e_pcred.p_ruid)
+                           : Unicode_Alloc(ppw->pw_name, STRING_ENCODING_DEFAULT);
+
+      /*
+       * If the command name in the kinfo_proc struct is strictly less than the
+       * maximum allowed size, then we can save it right now. Else we shall
+       * need to try and parse it from the entire command line.
+       */
+      if (strlen(kptmp->kp_proc.p_comm) + 1 < sizeof kptmp->kp_proc.p_comm) {
+         procInfo.procCmdName = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+         cmdNameLookup = FALSE;
+      }
+
+      /*
+       * Store the command line arguments of the process.
+       * If no arguments are found, use the full command name.
+       */
+      DynBuf_Init(&argsBuf);
+      if (cmdNameLookup) {
+         argCount = ProcMgrGetCommandLineArgs(kptmp->kp_proc.p_pid, &argsBuf, &procInfo.procCmdName);
+      } else {
+         argCount = ProcMgrGetCommandLineArgs(kptmp->kp_proc.p_pid, &argsBuf, NULL);
+      }
+      if (0 < argCount) {
+         procInfo.procCmdLine = DynBuf_Detach(&argsBuf);
+         /*
+          * cmdName would have been filled up by the function ProcMgrGetCommandLineArgs().
+          */
+         cmdNameLookup = FALSE;
+      } else {
+         procInfo.procCmdLine = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+         if (cmdNameLookup) {
+            procInfo.procCmdName = Unicode_Alloc(kptmp->kp_proc.p_comm, STRING_ENCODING_DEFAULT);
+            cmdNameLookup = FALSE;
+         }
+      }
+      DynBuf_Destroy(&argsBuf);
+
+      /*
+       * Store the start time of the process
+       */
+      procInfo.procStartTime = kptmp->kp_proc.p_starttime.tv_sec;
+
+      /*
+       * Store the process info pointer into a list buffer.
+       */
+      *ProcMgrProcInfoArray_AddressOf(procList, i) = procInfo;
+
+      procInfo.procCmdLine = NULL;
+      procInfo.procCmdName = NULL;
+      procInfo.procOwner = NULL;
+
+   } // nentries
+
+   failed = FALSE;
+
+abort:
+   free(kp);
+   free(procInfo.procCmdLine);
+   free(procInfo.procCmdName);
+   free(procInfo.procOwner);
+
+   if (failed) {
+      ProcMgr_FreeProcList(procList);
+      procList = NULL;
+   }
+
+   return procList;
+}
+#endif // defined(__APPLE__)
+
 /*
  *----------------------------------------------------------------------
  *
  * ProcMgr_FreeProcList --
  *
- *      Free the memory occupied by ProcMgr_ProcList.
+ *      Free the memory occupied by ProcMgrProcInfoArray.
  *
  * Results:
  *      
@@ -619,23 +1193,24 @@ abort:
  */
 
 void
-ProcMgr_FreeProcList(ProcMgr_ProcList *procList)
+ProcMgr_FreeProcList(ProcMgrProcInfoArray *procList)
 {
    int i;
+   size_t procCount;
 
    if (NULL == procList) {
       return;
    }
 
-   for (i = 0; i < procList->procCount; i++) {
-      free(procList->procCmdList[i]);
-      free(procList->procOwnerList[i]);
+   procCount = ProcMgrProcInfoArray_Count(procList);
+   for (i = 0; i < procCount; i++) {
+      ProcMgrProcInfo *procInfo = ProcMgrProcInfoArray_AddressOf(procList, i);
+      free(procInfo->procCmdName);
+      free(procInfo->procCmdLine);
+      free(procInfo->procOwner);
    }
 
-   free(procList->procIdList);
-   free(procList->procCmdList);
-   free(procList->startTime);
-   free(procList->procOwnerList);
+   ProcMgrProcInfoArray_Destroy(procList);
    free(procList);
 }
 
@@ -733,40 +1308,36 @@ ProcMgrStartProcess(char const *cmd,            // IN: UTF-8 encoded cmd
    if (pid == -1) {
       Warning("Unable to fork: %s.\n\n", strerror(errno));
    } else if (pid == 0) {
-#ifdef sun
-      /*
-       * On Solaris, /bin/sh is the Bourne shell, and it
-       * doesn't appear to have the optimization that bash does -- when
-       * called with -c, bash appears to just use exec() to replace itself.
-       * Bourne shell does a fork & exec, so 2 processes are started.
-       * This is bad for us because we then see the PID of the shell, not the
-       * app that it starts.  When this PID is returned to a user to
-       * watch, they'll watch the wrong process.
-       *
-       * So for Solaris, use bash instead if possible.  We support
-       * Solaris 10 and better; it contains bash, but not in its
-       * minimal 'core' package, so it may not exist.
-       */
-      static const char bashShellPath[] = SOLARIS_BASH_PATH;
+      static const char bashShellPath[] = BASH_PATH;
       char *bashArgs[] = { "bash", "-c", cmdCurrent, NULL };
       static const char bourneShellPath[] = "/bin/sh";
       char *bourneArgs[] = { "sh", "-c", cmdCurrent, NULL };
       const char *shellPath;
       char **args;
-#else
-      static const char shellPath[] = "/bin/sh";
-      char *args[] = { "sh", "-c", cmdCurrent, NULL };
-#endif
 
-#ifdef sun
-      if (File_Exists(SOLARIS_BASH_PATH)) {
+      /*
+       * Check bug 772203. To start the program, we start the shell
+       * and specify the program using the option '-c'. We should return the
+       * PID of the app that gets started.
+       *
+       * When the option '-c' is specified,
+       * - bash shell just uses exec() to replace itself. So, 'bash' returns
+       * the PID of the new application that is started.
+       *
+       * - bourne shell does a fork & exec. So two processes are started. We
+       * see the PID of the shell and not the app that it starts. When the PID
+       * is returned to a user to watch, they'll watch the wrong process.
+       *
+       * In order to return the proper PID, use bash if possible. If bash
+       * is not available, then use the bourne shell.
+       */
+      if (File_Exists(bashShellPath)) {
          shellPath = bashShellPath;
          args = bashArgs;
       } else {
          shellPath = bourneShellPath;
          args = bourneArgs;
       }
-#endif
 
       /*
        * Child
@@ -1435,7 +2006,7 @@ ProcMgr_Free(ProcMgr_AsyncProc *asyncProc) // IN
    free(asyncProc);
 }
 
-#ifdef linux
+#if defined(linux) || defined(__FreeBSD__) || defined(__APPLE__)
 
 /*
  *----------------------------------------------------------------------
@@ -1470,6 +2041,7 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
    gid_t root_gid;
    int error;
    int ret;
+   char *userLocal;
 
    if ((error = getpwuid_r(0, &pw, buffer, sizeof buffer, &ppw)) != 0 ||
        !ppw) {
@@ -1486,8 +2058,18 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
 
    root_gid = ppw->pw_gid;
 
-   if ((error = getpwnam_r(user, &pw, buffer, sizeof buffer, &ppw)) != 0 ||
-       !ppw) {
+   /* convert user name to local character set */
+   userLocal = (char *)Unicode_GetAllocBytes(user, Unicode_GetCurrentEncoding());
+   if (!userLocal) {
+       Warning("Failed to convert user name %s to local character set.\n", user);
+       return FALSE;
+   }
+
+   error = getpwnam_r(userLocal, &pw, buffer, sizeof buffer, &ppw);
+
+   free(userLocal);
+
+   if (error != 0 || !ppw) {
       if (error == 0) {
          error = ENOENT;
       }
@@ -1495,20 +2077,34 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
    }
 
    // first change group
+#if defined(USERWORLD)
+   ret = Id_SetREGid(ppw->pw_gid, ppw->pw_gid);
+#elif defined(__APPLE__)
+   ret = setregid(ppw->pw_gid, ppw->pw_gid);
+#else
    ret = setresgid(ppw->pw_gid, ppw->pw_gid, root_gid);
+#endif
    if (ret < 0) {
-      Warning("Failed to setresgid() for user %s\n", user);
+      Warning("Failed to set gid for user %s\n", user);
       return FALSE;
    }
+#ifndef USERWORLD
    ret = initgroups(ppw->pw_name, ppw->pw_gid);
    if (ret < 0) {
       Warning("Failed to initgroups() for user %s\n", user);
       goto failure;
    }
+#endif
    // now user
+#if defined(USERWORLD)
+   ret = Id_SetREUid(ppw->pw_uid, ppw->pw_uid);
+#elif defined(__APPLE__)
+   ret = setreuid(ppw->pw_uid, ppw->pw_uid);
+#else
    ret = setresuid(ppw->pw_uid, ppw->pw_uid, 0);
+#endif
    if (ret < 0) {
-      Warning("Failed to setresuid() for user %s\n", user);
+      Warning("Failed to set uid for user %s\n", user);
       goto failure;
    }
 
@@ -1562,23 +2158,37 @@ ProcMgr_ImpersonateUserStop(void)
    }
 
    // first change back user
+#if defined(USERWORLD)
+   ret = Id_SetREUid(ppw->pw_uid, ppw->pw_uid);
+#elif defined(__APPLE__)
+   ret = setreuid(ppw->pw_uid, ppw->pw_uid);
+#else
    ret = setresuid(ppw->pw_uid, ppw->pw_uid, 0);
+#endif
    if (ret < 0) {
-      Warning("Failed to setresuid() for root\n");
+      Warning("Failed to set uid for root\n");
       return FALSE;
    }
 
    // now group
+#if defined(USERWORLD)
+   ret = Id_SetREGid(ppw->pw_gid, ppw->pw_gid);
+#elif defined(__APPLE__)
+   ret = setregid(ppw->pw_gid, ppw->pw_gid);
+#else
    ret = setresgid(ppw->pw_gid, ppw->pw_gid, ppw->pw_gid);
+#endif
    if (ret < 0) {
-      Warning("Failed to setresgid() for root\n");
+      Warning("Failed to set gid for root\n");
       return FALSE;
    }
+#ifndef USERWORLD
    ret = initgroups(ppw->pw_name, ppw->pw_gid);
    if (ret < 0) {
       Warning("Failed to initgroups() for root\n");
       return FALSE;
    }
+#endif
 
    // set env
    setenv("USER", ppw->pw_name, 1);
@@ -1634,4 +2244,4 @@ ProcMgr_GetImpersonatedUserInfo(char **userName,            // OUT
    return TRUE;
 }
 
-#endif // linux
+#endif // linux || __FreeBSD__ || __APPLE__
