@@ -227,7 +227,7 @@ Unicode_Duplicate(ConstUnicode str) // IN
  */
 
 void
-Unicode_Free(Unicode str) // IN
+Unicode_Free(void *str) // IN
 {
    free(str);
 }
@@ -319,13 +319,15 @@ Unicode_FreeList(Unicode *list,    // IN: the list to free
  *
  * Unicode_GetAllocList --
  *
- *      Allocates a list (actually a vector) of NUL terminated buffers from a 
- *      list (vector) of strings of specified encoding.
+ *      Given a list of Unicode strings, converts them to a list of
+ *      buffers in the specified encoding.
+ *
  *      The input list has a specified length or can be an argv-style
  *      NULL-terminated list (if length is negative).
  *
  * Results:
- *      An allocated list (vector) of NUL terminated buffers,
+ *      An allocated list (vector) of NUL terminated buffers in the specified
+ *      encoding
  *      or NULL on conversion failure.
  *      The caller is responsible to free the memory allocated by
  *      this routine.
@@ -336,12 +338,12 @@ Unicode_FreeList(Unicode *list,    // IN: the list to free
  *-----------------------------------------------------------------------------
  */
 
-Unicode *
+char **
 Unicode_GetAllocList(Unicode const srcList[], // IN: list of strings
-		     ssize_t length,          // IN: list 
-		     StringEncoding encoding) // IN:
+		     ssize_t length,          // IN: length (-1 for NULL term.)
+		     StringEncoding encoding) // IN: Encoding of returned list
 {
-   Unicode *dstList = NULL;
+   char **dstList = NULL;
    ssize_t i;
 
    ASSERT(srcList != NULL);
@@ -353,7 +355,7 @@ Unicode_GetAllocList(Unicode const srcList[], // IN: list of strings
       while (srcList[length] != NULL) {
          length++;
       }
-      
+
       /* Include the sentinel element. */
       length++;
    }
