@@ -27,12 +27,16 @@
 #include "vmGuestLib.h"
 #include "vmGuestLibInt.h"
 #include "str.h"
-#include "rpcout.h"
+#include "vmware/tools/guestrpc.h"
 #include "vmcheck.h"
 #include "util.h"
 #include "debug.h"
 #include "strutil.h"
-#include "guestrpc/guestlibV3.h"
+#include "guestlibV3.h"
+#include "guestlibIoctl.h"
+#include "dynxdr.h"
+#include "xdrutil.h"
+#include "ctype.h"
 
 #define GUESTLIB_NAME "VMware Guest API"
 
@@ -103,6 +107,7 @@ typedef struct {
       ASSERT(_stat.d == (STATID));                                              \
       if (sizeof *(OUTPTR) < sizeof _stat.GuestLibV3Stat_u.FIELDNAME.value) {   \
          (ERROR) = VMGUESTLIB_ERROR_BUFFER_TOO_SMALL;                           \
+         break;                                                                 \
       }                                                                         \
       *(OUTPTR) = _stat.GuestLibV3Stat_u.FIELDNAME.value;                       \
       (ERROR) = VMGUESTLIB_ERROR_SUCCESS;                                       \
@@ -254,6 +259,8 @@ VMGuestLib_OpenHandle(VMGuestLibHandle *handle) // OUT
 {
    VMGuestLibHandleType *data;
 
+   VMTools_SetGuestSDKMode();
+
    if (!VmCheck_IsVirtualWorld()) {
       Debug("VMGuestLib_OpenHandle: Not in a VM.\n");
       return VMGUESTLIB_ERROR_NOT_RUNNING_IN_VM;
@@ -374,7 +381,7 @@ VMGuestLibUpdateInfo(VMGuestLibHandle handle) // IN
                   hostVersion);
 
       /* Send the request. */
-      if (RpcOut_sendOne(&reply, &replyLen, commandBuf)) {
+      if (RpcChannel_SendOne(&reply, &replyLen, commandBuf)) {
          VMGuestLibDataV2 *v2reply = (VMGuestLibDataV2 *)reply;
          VMSessionId sessionId = HANDLE_SESSIONID(handle);
 
@@ -599,6 +606,7 @@ VMGuestLib_UpdateInfo(VMGuestLibHandle handle) // IN
    error = VMGuestLibUpdateInfo(handle);
    if (error != VMGUESTLIB_ERROR_SUCCESS) {
       Debug("VMGuestLibUpdateInfo failed: %d\n", error);
+      HANDLE_SESSIONID(handle) = 0;
       return error;
    }
 
@@ -1586,3 +1594,248 @@ VMGuestLib_GetHostMemUnmappedMB(VMGuestLibHandle handle,    // IN
    return error;
 }
 
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_GetMemZippedMB --
+ *
+ *      Total zipped VM memory
+ *
+ * Results:
+ *      VMGuestLibError
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+VMGuestLibError
+VMGuestLib_GetMemZippedMB(VMGuestLibHandle handle,    // IN
+                          uint32 *memZippedMB)        // OUT
+{
+   VMGuestLibError error = VMGUESTLIB_ERROR_OTHER;
+   VMGUESTLIB_GETSTAT_V3(handle, error,
+                         memZippedMB, memZippedMB,
+                         GUESTLIB_MEM_ZIPPED_MB);
+   return error;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_GetMemZipSavedMB --
+ *
+ *      Total memopry saved by zipping VM memory
+ *
+ * Results:
+ *      VMGuestLibError
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+VMGuestLibError
+VMGuestLib_GetMemZipSavedMB(VMGuestLibHandle handle,    // IN
+                            uint32 *memZipSavedMB)      // OUT
+{
+   VMGuestLibError error = VMGUESTLIB_ERROR_OTHER;
+   VMGUESTLIB_GETSTAT_V3(handle, error,
+                         memZipSavedMB, memZipSavedMB,
+                         GUESTLIB_MEM_ZIPSAVED_MB);
+   return error;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_GetMemLLSwappedMB --
+ *
+ *      VM memory swapped to SSD
+ *
+ * Results:
+ *      VMGuestLibError
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+VMGuestLibError
+VMGuestLib_GetMemLLSwappedMB(VMGuestLibHandle handle,    // IN
+                             uint32 *memLLSwappedMB)     // OUT
+{
+   VMGuestLibError error = VMGUESTLIB_ERROR_OTHER;
+   VMGUESTLIB_GETSTAT_V3(handle, error,
+                         memLLSwappedMB, memLLSwappedMB,
+                         GUESTLIB_MEM_LLSWAPPED_MB);
+   return error;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_GetMemSwapTargetMB --
+ *
+ *      VM memory swap target
+ *
+ * Results:
+ *      VMGuestLibError
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+VMGuestLibError
+VMGuestLib_GetMemSwapTargetMB(VMGuestLibHandle handle,    // IN
+                              uint32 *memSwapTargetMB)    // OUT
+{
+   VMGuestLibError error = VMGUESTLIB_ERROR_OTHER;
+   VMGUESTLIB_GETSTAT_V3(handle, error,
+                         memSwapTargetMB, memSwapTargetMB,
+                         GUESTLIB_MEM_SWAP_TARGET_MB);
+   return error;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_GetMemBalloonTargetMB --
+ *
+ *      VM memory balloon target size
+ *
+ * Results:
+ *      VMGuestLibError
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+VMGuestLibError
+VMGuestLib_GetMemBalloonTargetMB(VMGuestLibHandle handle,    // IN
+                                 uint32 *memBalloonTargetMB) // OUT
+{
+   VMGuestLibError error = VMGUESTLIB_ERROR_OTHER;
+   VMGUESTLIB_GETSTAT_V3(handle, error,
+                         memBalloonTargetMB, memBalloonTargetMB,
+                         GUESTLIB_MEM_BALLOON_TARGET_MB);
+   return error;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_GetMemBalloonMaxMB --
+ *
+ *      VM memory balloon limit
+ *
+ * Results:
+ *      VMGuestLibError
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+VMGuestLibError
+VMGuestLib_GetMemBalloonMaxMB(VMGuestLibHandle handle,    // IN
+                              uint32 *memBalloonMaxMB)    // OUT
+{
+   VMGuestLibError error = VMGUESTLIB_ERROR_OTHER;
+   VMGUESTLIB_GETSTAT_V3(handle, error,
+                         memBalloonMaxMB, memBalloonMaxMB,
+                         GUESTLIB_MEM_BALLOON_MAX_MB);
+   return error;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLibIoctl --
+ *
+ *      Marshal and invoke the guestlib ioctl.
+ *
+ * Results:
+ *      TRUE on success, FALSE otherwise (reply contains error detail, if
+ *      provided)
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static Bool
+VMGuestLibIoctl(const GuestLibIoctlParam *param,
+                char **reply,
+                size_t *replySize)
+{
+   XDR xdrs;
+   Bool ret;
+   static const char *request = VMGUESTLIB_IOCTL_COMMAND_STRING " ";
+
+   if (param == NULL || param->d >= GUESTLIB_IOCTL_MAX) {
+      return FALSE;
+   }
+   if (NULL == DynXdr_Create(&xdrs)) {
+      return FALSE;
+   }
+   if (!DynXdr_AppendRaw(&xdrs, request, strlen(request)) ||
+       !xdr_GuestLibIoctlParam(&xdrs, (GuestLibIoctlParam *)param)) {
+      DynXdr_Destroy(&xdrs, TRUE);
+      return FALSE;
+   }
+   ret = RpcChannel_SendOneRaw(DynXdr_Get(&xdrs), xdr_getpos(&xdrs),
+                               reply, replySize);
+   DynXdr_Destroy(&xdrs, TRUE);
+   return ret;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMGuestLib_AtomicUpdateCookie --
+ *
+ *      Atomically update a cookie on the host.
+ *
+ * Results:
+ *      TRUE on success, FALSE otherwise.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+VMGuestLib_AtomicUpdateCookie(const char *src,    // IN
+                              const char *dst,    // IN
+                              char **reply,       // OUT
+                              size_t *replySize)  // OUT
+{
+   GuestLibIoctlParam param;
+
+   ASSERT(src != NULL);
+   ASSERT(dst != NULL);
+
+   param.d = GUESTLIB_IOCTL_ATOMIC_UPDATE_COOKIE;
+   param.GuestLibIoctlParam_u.atomicUpdateCookie.src = (char *)src;
+   param.GuestLibIoctlParam_u.atomicUpdateCookie.dst = (char *)dst;
+   return VMGuestLibIoctl(&param, reply, replySize);
+}
