@@ -108,10 +108,13 @@ typedef struct LogHandler {
 static gchar *gLogDomain = NULL;
 static gboolean gEnableCoreDump = TRUE;
 static gboolean gLogEnabled = FALSE;
+static gboolean gGuestSDKMode = FALSE;
 static guint gPanicCount = 0;
 static LogHandler *gDefaultData;
 static LogHandler *gErrorData;
 static GPtrArray *gDomains = NULL;
+static gboolean gLogInitialized = FALSE;
+static gboolean gLoggingStopped = FALSE;
 
 /* Internal functions. */
 
@@ -866,6 +869,7 @@ VMTools_ConfigLogging(const gchar *defaultDomain,
    }
 
    gLogEnabled |= force;
+   gLogInitialized = TRUE;
 
    if (allocDict) {
       g_key_file_free(cfg);
@@ -891,6 +895,20 @@ VMToolsLogWrapper(GLogLevelFlags level,
                   const char *fmt,
                   va_list args)
 {
+   if (!gLogInitialized && !IS_FATAL(level)) {
+      /*
+       * Avoid logging without initialization because
+       * it leads to spamming of the console output.
+       * Fatal messages are exception.
+       */
+      return;
+   }
+
+   if (gLoggingStopped) {
+      /* This is to avoid nested logging in vmxLogger */
+      return;
+   }
+
    if (gPanicCount == 0) {
       char *msg = Str_Vasprintf(NULL, fmt, args);
       if (msg != NULL) {
@@ -906,6 +924,39 @@ VMToolsLogWrapper(GLogLevelFlags level,
 }
 
 
+/**
+ * This is called to avoid nested logging in vmxLogger.
+ */
+
+void
+VMTools_StopLogging(void)
+{
+   gLoggingStopped = TRUE;
+}
+
+
+/**
+ * This is called to reset logging in vmxLogger.
+ */
+
+void
+VMTools_RestartLogging(void)
+{
+   gLoggingStopped = FALSE;
+}
+
+
+/**
+ * Called if vmtools lib is used along with Guestlib SDK.
+ */
+
+void
+VMTools_SetGuestSDKMode(void)
+{
+   gGuestSDKMode = TRUE;
+}
+
+
 /**
  * Logs a message using the G_LOG_LEVEL_DEBUG level.
  *
@@ -917,7 +968,11 @@ Debug(const char *fmt, ...)
 {
    va_list args;
    va_start(args, fmt);
-   VMToolsLogWrapper(G_LOG_LEVEL_DEBUG, fmt, args);
+   if (gGuestSDKMode) {
+      GuestSDK_Debug(fmt, args);
+   } else {
+      VMToolsLogWrapper(G_LOG_LEVEL_DEBUG, fmt, args);
+   }
    va_end(args);
 }
 
@@ -933,7 +988,11 @@ Log(const char *fmt, ...)
 {
    va_list args;
    va_start(args, fmt);
-   VMToolsLogWrapper(G_LOG_LEVEL_INFO, fmt, args);
+   if (gGuestSDKMode) {
+      GuestSDK_Log(fmt, args);
+   } else {
+      VMToolsLogWrapper(G_LOG_LEVEL_INFO, fmt, args);
+   }
    va_end(args);
 }
 
@@ -999,31 +1058,37 @@ Panic(const char *fmt, ...)
    va_list args;
 
    va_start(args, fmt);
-   if (gPanicCount == 0) {
-      char *msg = Str_Vasprintf(NULL, fmt, args);
-      if (msg != NULL) {
-         g_log(gLogDomain, G_LOG_LEVEL_ERROR, "%s", msg);
-         free(msg);
-      }
-      /*
-       * In case an user-installed custom handler doesn't panic on error, force a
-       * core dump. Also force a dump in the recursive case.
-       */
-      VMToolsLogPanic();
-   } else if (gPanicCount == 1) {
-      /*
-       * Use a stack allocated string since we're in a recursive panic, so
-       * probably already in a weird state.
-       */
-      gchar msg[1024];
-      Str_Vsnprintf(msg, sizeof msg, fmt, args);
-      fprintf(stderr, "Recursive panic: %s\n", msg);
-      VMToolsLogPanic();
+
+   if (gGuestSDKMode) {
+      GuestSDK_Panic(fmt, args);
    } else {
-      fprintf(stderr, "Recursive panic, giving up.\n");
-      exit(-1);
+      if (gPanicCount == 0) {
+         char *msg = Str_Vasprintf(NULL, fmt, args);
+         if (msg != NULL) {
+            g_log(gLogDomain, G_LOG_LEVEL_ERROR, "%s", msg);
+            free(msg);
+         }
+         /*
+          * In case an user-installed custom handler doesn't panic on error,
+          * force a core dump. Also force a dump in the recursive case.
+          */
+         VMToolsLogPanic();
+      } else if (gPanicCount == 1) {
+         /*
+          * Use a stack allocated string since we're in a recursive panic, so
+          * probably already in a weird state.
+          */
+         gchar msg[1024];
+         Str_Vsnprintf(msg, sizeof msg, fmt, args);
+         fprintf(stderr, "Recursive panic: %s\n", msg);
+         VMToolsLogPanic();
+      } else {
+         fprintf(stderr, "Recursive panic, giving up.\n");
+         exit(-1);
+      }
    }
    va_end(args);
+   while (1) ; // avoid compiler warning
 }
 
 
@@ -1038,7 +1103,11 @@ Warning(const char *fmt, ...)
 {
    va_list args;
    va_start(args, fmt);
-   VMToolsLogWrapper(G_LOG_LEVEL_WARNING, fmt, args);
+   if (gGuestSDKMode) {
+      GuestSDK_Warning(fmt, args);
+   } else {
+      VMToolsLogWrapper(G_LOG_LEVEL_WARNING, fmt, args);
+   }
    va_end(args);
 }
 
