@@ -186,8 +186,6 @@ typedef int (*VMCIEventReleaseCB)(void *clientData);
 
 #define VMCI_SEMA_RANK_QUEUEPAIRLIST    (VMCI_SEMA_RANK_QPHEADER - 1)
 #define VMCI_SEMA_RANK_GUESTMEM         (VMCI_SEMA_RANK_QUEUEPAIRLIST - 1)
-#define VMCI_SEMA_RANK_PACKET_QP        (VMCI_SEMA_RANK_QPHEADER - 1)
-//#define VMCI_SEMA_RANK_PACKET_QP        0xffd /* For vVol */
 
 /*
  * Host specific struct used for signalling.
@@ -254,6 +252,7 @@ Bool VMCIHost_WaitForCallLocked(VMCIHost *hostContext,
                                 Bool useBH);
 #ifdef VMKERNEL
 int VMCIHost_ContextToHostVmID(VMCIHost *hostContext, VMCIHostVmID *hostVmID);
+int VMCIHost_ContextHasUuid(VMCIHost *hostContext, const char *uuid);
 void VMCIHost_SetActiveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
 Bool VMCIHost_RemoveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
 Bool VMCIHost_IsActiveHnd(VMCIHost *hostContext, uintptr_t eventHnd);
@@ -371,11 +370,11 @@ typedef uint32 VMCIGuestMemID;
   void VMCI_ReleaseQueueMutex(struct VMCIQueue *queue);
 #else // Below are the guest OS'es without host side support.
 #  define VMCI_InitQueueMutex(_pq, _cq)
-#  define VMCI_CleanupQueueMutex(_pq, _cq)
+#  define VMCI_CleanupQueueMutex(_pq, _cq) do { } while (0)
 #  define VMCI_AcquireQueueMutex(_q, _cb) VMCI_SUCCESS
-#  define VMCI_ReleaseQueueMutex(_q)
+#  define VMCI_ReleaseQueueMutex(_q) do { } while (0)
 #  define VMCIHost_RegisterUserMemory(_ps, _pq, _cq) VMCI_ERROR_UNAVAILABLE
-#  define VMCIHost_UnregisterUserMemory(_pq, _cq)
+#  define VMCIHost_UnregisterUserMemory(_pq, _cq) do { } while (0)
 #  define VMCIHost_MapQueues(_pq, _cq, _f) VMCI_SUCCESS
 #  define VMCIHost_UnmapQueues(_gid, _pq, _cq) VMCI_SUCCESS
 #endif
@@ -386,8 +385,8 @@ typedef uint32 VMCIGuestMemID;
   void VMCIHost_MarkQueuesUnavailable(struct VMCIQueue *produceQ,
                                       struct VMCIQueue *consumeQ);
 #else
-#  define VMCIHost_MarkQueuesAvailable(_q, _p) while(0) { }
-#  define VMCIHost_MarkQueuesUnavailable(_q, _p) while(0) { }
+#  define VMCIHost_MarkQueuesAvailable(_q, _p) do { } while (0)
+#  define VMCIHost_MarkQueuesUnavailable(_q, _p) do { } while(0)
 #endif
 
 #if defined(VMKERNEL) || defined(__linux__)
