@@ -1256,7 +1256,7 @@ VMCIContext_GetId(VMCIContext *context) // IN:
 /*
  *----------------------------------------------------------------------
  *
- * VMCIContext_GetPrivFlags --
+ * vmci_context_get_priv_flags --
  *
  *      Retrieves the privilege flags of the given VMCI context ID.
  *
@@ -1269,9 +1269,9 @@ VMCIContext_GetId(VMCIContext *context) // IN:
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIContext_GetPrivFlags)
+VMCI_EXPORT_SYMBOL(vmci_context_get_priv_flags)
 VMCIPrivilegeFlags
-VMCIContext_GetPrivFlags(VMCIId contextID)  // IN
+vmci_context_get_priv_flags(VMCIId contextID)  // IN
 {
    if (VMCI_HostPersonalityActive()) {
       VMCIPrivilegeFlags flags;
@@ -1977,8 +1977,7 @@ VMCIContext_NotifyDoorbell(VMCIId srcCID,                   // IN
    if (srcCID != handle.context) {
       VMCIPrivilegeFlags dstPrivFlags;
 
-      if (!vmkernel && VMCI_CONTEXT_IS_VM(srcCID) &&
-          VMCI_CONTEXT_IS_VM(handle.context)) {
+      if (VMCI_CONTEXT_IS_VM(srcCID) && VMCI_CONTEXT_IS_VM(handle.context)) {
          VMCI_DEBUG_LOG(4, (LGPFX"Doorbell notification from VM to VM not "
                             "supported (src=0x%x, dst=0x%x).\n",
                             srcCID, handle.context));
@@ -1996,7 +1995,7 @@ VMCIContext_NotifyDoorbell(VMCIId srcCID,                   // IN
 
       if (srcCID != VMCI_HOST_CONTEXT_ID ||
           srcPrivFlags == VMCI_NO_PRIVILEGE_FLAGS) {
-         srcPrivFlags = VMCIContext_GetPrivFlags(srcCID);
+         srcPrivFlags = vmci_context_get_priv_flags(srcCID);
       }
 
       if (VMCIDenyInteraction(srcPrivFlags, dstPrivFlags)) {
@@ -2138,7 +2137,7 @@ VMCIContext_SignalPendingDatagrams(VMCIId contextID)
 /*
  *----------------------------------------------------------------------
  *
- * VMCI_ContextID2HostVmID --
+ * vmci_cid_2_host_vm_id --
  *
  *      Maps a context ID to the host specific (process/world) ID
  *      of the VM/VMX.
@@ -2152,11 +2151,11 @@ VMCIContext_SignalPendingDatagrams(VMCIId contextID)
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCI_ContextID2HostVmID)
+VMCI_EXPORT_SYMBOL(vmci_cid_2_host_vm_id)
 int
-VMCI_ContextID2HostVmID(VMCIId contextID,    // IN
-                        void *hostVmID,      // OUT
-                        size_t hostVmIDLen)  // IN
+vmci_cid_2_host_vm_id(VMCIId contextID,    // IN
+                      void *hostVmID,      // OUT
+                      size_t hostVmIDLen)  // IN
 {
 #if defined(VMKERNEL)
    VMCIContext *context;
@@ -2189,13 +2188,14 @@ VMCI_ContextID2HostVmID(VMCIId contextID,    // IN
 /*
  *----------------------------------------------------------------------
  *
- * VMCI_IsContextOwner --
+ * vmci_is_context_owner --
  *
  *      Determines whether a given host OS specific representation of
  *      user is the owner of the VM/VMX.
  *
  * Results:
- *      VMCI_SUCCESS if the hostUser is owner, error code otherwise.
+ *      Linux: 1 (true) if hostUser is owner, 0 (false) otherwise.
+ *      Other: VMCI_SUCCESS if the hostUser is owner, error code otherwise.
  *
  * Side effects:
  *      None.
@@ -2203,10 +2203,33 @@ VMCI_ContextID2HostVmID(VMCIId contextID,    // IN
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCI_IsContextOwner)
+VMCI_EXPORT_SYMBOL(vmci_is_context_owner)
+#if defined(linux) && !defined(VMKERNEL)
 int
-VMCI_IsContextOwner(VMCIId contextID,   // IN
-                    void *hostUser)     // IN
+vmci_is_context_owner(VMCIId contextID,   // IN
+                      uid_t uid)          // IN
+{
+   int isOwner = 0;
+
+   if (VMCI_HostPersonalityActive()) {
+      VMCIContext *context = VMCIContext_Get(contextID);
+      if (context) {
+         if (context->validUser) {
+            if (VMCIHost_CompareUser((VMCIHostUser *)&uid,
+                                     &context->user) == VMCI_SUCCESS) {
+               isOwner = 1;
+            }
+         }
+         VMCIContext_Release(context);
+      }
+   }
+
+   return isOwner;
+}
+#else // !linux || VMKERNEL
+int
+vmci_is_context_owner(VMCIId contextID,   // IN
+                      void *hostUser)     // IN
 {
    if (VMCI_HostPersonalityActive()) {
       VMCIContext *context;
@@ -2237,6 +2260,7 @@ VMCI_IsContextOwner(VMCIId contextID,   // IN
    }
    return VMCI_ERROR_UNAVAILABLE;
 }
+#endif // !linux || VMKERNEL
 
 
 /*
