@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2011 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006-2013 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 /*
  * vmciContext.h --
  *
- *	VMCI state to enable sending calls between VMs.
+ *    VMCI state to enable sending calls between VMs.
  */
 
 #ifndef _VMCI_CONTEXT_H_
@@ -36,6 +36,7 @@
 #include "vmci_handle_array.h"
 #include "vmci_infrastructure.h"
 #include "vmci_kernel_if.h"
+#include "vmciCommonInt.h"
 
 #define MAX_QUEUED_GUESTCALLS_PER_VM  100
 
@@ -69,7 +70,7 @@ Bool VMCIContext_SupportsHostQP(VMCIContext *context);
 void VMCIContext_ReleaseContext(VMCIContext *context);
 int VMCIContext_EnqueueDatagram(VMCIId cid, VMCIDatagram *dg);
 int VMCIContext_DequeueDatagram(VMCIContext *context, size_t *maxSize,
-				VMCIDatagram **dg);
+                                VMCIDatagram **dg);
 int VMCIContext_PendingDatagrams(VMCIId cid, uint32 *pending);
 VMCIContext *VMCIContext_Get(VMCIId cid);
 void VMCIContext_Release(VMCIContext *context);
@@ -113,8 +114,7 @@ void VMCIContext_ReceiveNotificationsRelease(VMCIId contextID,
 #if defined(VMKERNEL)
 void VMCIContext_SignalPendingDoorbells(VMCIId contextID);
 void VMCIContext_SignalPendingDatagrams(VMCIId contextID);
-
-int VMCIContextID2HostVmID(VMCIId contextID, void *hostVmID, size_t hostVmIDLen);
-#endif
-
+int VMCIContext_FilterSet(VMCIId cid, VMCIFilterState *filterState);
+int VMCI_Uuid2ContextId(const char *uuidString, VMCIId *contextID);
+#endif // VMKERNEL
 #endif // _VMCI_CONTEXT_H_
