@@ -33,6 +33,11 @@
 #include "compat_kernel.h"
 #include "compat_slab.h"
 
+/* Must be after compat_fs.h */
+#if defined VMW_USE_AIO
+#include <linux/aio.h>
+#endif
+
 #include "cpName.h"
 #include "hgfsProto.h"
 #include "module.h"
@@ -45,13 +50,12 @@
 /* Private functions. */
 static int HgfsPackOpenRequest(struct inode *inode,
                                struct file *file,
-			       HgfsOp opUsed,
+                               HgfsOp opUsed,
                                HgfsReq *req);
 static int HgfsUnpackOpenReply(HgfsReq *req,
                                HgfsOp opUsed,
                                HgfsHandle *file,
                                HgfsLockType *lock);
-static int HgfsGetOpenFlags(uint32 flags);
 
 /* HGFS file operations for files. */
 static int HgfsOpen(struct inode *inode,
@@ -173,7 +177,7 @@ struct file_operations HgfsFileFileOperations = {
 static int
 HgfsPackOpenRequest(struct inode *inode, // IN: Inode of the file to open
                     struct file *file,   // IN: File pointer for this open
-		    HgfsOp opUsed,       // IN: Op to use
+                    HgfsOp opUsed,       // IN: Op to use
                     HgfsReq *req)        // IN/OUT: Packet to write into
 {
    char *name;
@@ -226,6 +230,8 @@ HgfsPackOpenRequest(struct inode *inode, // IN: Inode of the file to open
       }
       requestV3->flags = result;
 
+      LOG(4, (KERN_DEBUG "VMware hgfs: %s: mode file %o inode %o -> user %o\n",
+              __func__, file->f_mode, inode->i_mode, (inode->i_mode & S_IRWXU) >> 6));
       /* Set permissions. */
       requestV3->specialPerms = (inode->i_mode & (S_ISUID | S_ISGID | S_ISVTX))
                                 >> 9;
@@ -426,94 +432,6 @@ HgfsUnpackOpenReply(HgfsReq *req,          // IN: Packet with reply inside
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * HgfsGetOpenFlags --
- *
- *    Based on the flags requested by the process making the open()
- *    syscall, determine which flags to send to the server to open the
- *    file.
- *
- * Results:
- *    Returns the correct HgfsOpenFlags enumeration to send to the
- *    server, or -1 on failure.
- *
- * Side effects:
- *    None
- *
- *----------------------------------------------------------------------
- */
-
-static int
-HgfsGetOpenFlags(uint32 flags) // IN: Open flags
-{
-   uint32 mask = O_CREAT | O_TRUNC | O_EXCL;
-   int result = -1;
-
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsGetOpenFlags: entered\n"));
-
-   /*
-    * Mask the flags to only look at O_CREAT, O_EXCL, and O_TRUNC.
-    */
-
-   flags &= mask;
-
-   /* O_EXCL has no meaning if O_CREAT is not set. */
-   if (!(flags & O_CREAT)) {
-      flags &= ~O_EXCL;
-   }
-
-   /* Pick the right HgfsOpenFlags. */
-   switch (flags) {
-
-   case 0:
-      /* Regular open; fails if file nonexistant. */
-      result = HGFS_OPEN;
-      break;
-
-   case O_CREAT:
-      /* Create file; if it exists already just open it. */
-      result = HGFS_OPEN_CREATE;
-      break;
-
-   case O_TRUNC:
-      /* Truncate existing file; fails if nonexistant. */
-      result = HGFS_OPEN_EMPTY;
-      break;
-
-   case (O_CREAT | O_EXCL):
-      /* Create file; fail if it exists already. */
-      result = HGFS_OPEN_CREATE_SAFE;
-      break;
-
-   case (O_CREAT | O_TRUNC):
-      /* Create file; if it exists already, truncate it. */
-      result = HGFS_OPEN_CREATE_EMPTY;
-      break;
-
-   default:
-      /*
-       * This can only happen if all three flags are set, which
-       * conceptually makes no sense because O_EXCL and O_TRUNC are
-       * mutually exclusive if O_CREAT is set.
-       *
-       * However, the open(2) man page doesn't say you can't set all
-       * three flags, and certain apps (*cough* Nautilus *cough*) do
-       * so. To be friendly to those apps, we just silenty drop the
-       * O_TRUNC flag on the assumption that it's safer to honor
-       * O_EXCL.
-       */
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsGetOpenFlags: invalid open "
-              "flags %o. Ignoring the O_TRUNC flag.\n", flags));
-      result = HGFS_OPEN_CREATE_SAFE;
-      break;
-   }
-
-   return result;
-}
-
-
 /*
  * HGFS file operations for files.
  */
@@ -559,6 +477,10 @@ HgfsOpen(struct inode *inode,  // IN: Inode of the file to open
 
    iinfo = INODE_GET_II_P(inode);
 
+   LOG(4, (KERN_DEBUG "VMware hgfs: %s(%s/%s)\n",
+           __func__, file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name));
+
    req = HgfsGetNewRequest();
    if (!req) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsOpen: out of memory while "
@@ -743,15 +665,21 @@ HgfsAioRead(struct kiocb *iocb,      // IN:  I/O control block
             loff_t offset)           // IN:  Offset at which to read
 {
    int result;
+   struct dentry *readDentry;
 
    ASSERT(iocb);
    ASSERT(iocb->ki_filp);
    ASSERT(iocb->ki_filp->f_dentry);
    ASSERT(iov);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsAioRead: was called\n"));
+   readDentry = iocb->ki_filp->f_dentry;
 
-   result = HgfsRevalidate(iocb->ki_filp->f_dentry);
+   LOG(4, (KERN_DEBUG "VMware hgfs: %s(%s/%s, %lu@%lu)\n",
+           __func__, readDentry->d_parent->d_name.name,
+           readDentry->d_name.name,
+           (unsigned long) iov_length(iov, numSegs), (unsigned long) offset));
+
+   result = HgfsRevalidate(readDentry);
    if (result) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsAioRead: invalid dentry\n"));
       goto out;
@@ -792,15 +720,21 @@ HgfsAioWrite(struct kiocb *iocb,      // IN:  I/O control block
              loff_t offset)           // IN:  Offset at which to read
 {
    int result;
+   struct dentry *writeDentry;
 
    ASSERT(iocb);
    ASSERT(iocb->ki_filp);
    ASSERT(iocb->ki_filp->f_dentry);
    ASSERT(iov);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsAioWrite: was called\n"));
+   writeDentry = iocb->ki_filp->f_dentry;
 
-   result = HgfsRevalidate(iocb->ki_filp->f_dentry);
+   LOG(4, (KERN_DEBUG "VMware hgfs: %s(%s/%s, %lu@%Ld)\n",
+          __func__, writeDentry->d_parent->d_name.name,
+          writeDentry->d_name.name,
+          (unsigned long) iov_length(iov, numSegs), (long long) offset));
+
+   result = HgfsRevalidate(writeDentry);
    if (result) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsAioWrite: invalid dentry\n"));
       goto out;
@@ -845,8 +779,9 @@ HgfsRead(struct file *file,  // IN:  File to read from
    ASSERT(buf);
    ASSERT(offset);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsRead: read %Zu bytes from fh %u "
-           "at offset %Lu\n", count, FILE_GET_FI_P(file)->handle, *offset));
+   LOG(4, (KERN_DEBUG "VMware hgfs: %s(%s/%s,%Zu@%lld)\n",
+           __func__, file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name, count, (long long) *offset));
 
    result = HgfsRevalidate(file->f_dentry);
    if (result) {
@@ -896,8 +831,9 @@ HgfsWrite(struct file *file,      // IN: File to write to
    ASSERT(buf);
    ASSERT(offset);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsWrite: write %Zu bytes to fh %u "
-           "at offset %Lu\n", count, FILE_GET_FI_P(file)->handle, *offset));
+   LOG(4, (KERN_DEBUG "VMware hgfs: %s(%s/%s,%Zu@%lld)\n",
+           __func__, file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name, count, (long long) *offset));
 
    result = HgfsRevalidate(file->f_dentry);
    if (result) {
@@ -941,12 +877,15 @@ HgfsSeek(struct file *file,  // IN:  File to seek
    ASSERT(file);
    ASSERT(file->f_dentry);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsSeek: seek to %Lu bytes from fh %u "
-           "from position %d\n", offset, FILE_GET_FI_P(file)->handle, origin));
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s(%s/%s, %u, %lld, %d)\n",
+           __func__,
+            file->f_dentry->d_parent->d_name.name,
+            file->f_dentry->d_name.name,
+            FILE_GET_FI_P(file)->handle, offset, origin));
 
    result = (loff_t) HgfsRevalidate(file->f_dentry);
    if (result) {
-      LOG(6, (KERN_DEBUG "VMware hgfs: HgfsSeek: invalid dentry\n"));
+      LOG(6, (KERN_DEBUG "VMware hgfs: %s: invalid dentry\n", __func__));
       goto out;
    }
 
@@ -989,16 +928,25 @@ HgfsSeek(struct file *file,  // IN:  File to seek
  */
 
 static int
-HgfsFsync(struct file *file,		// IN: File we operate on
+HgfsFsync(struct file *file,            // IN: File we operate on
 #if defined VMW_FSYNC_OLD
           struct dentry *dentry,        // IN: Dentry for this file
 #elif defined VMW_FSYNC_31
           loff_t start,                 // IN: start of range to sync
           loff_t end,                   // IN: end of range to sync
 #endif
-          int datasync)	                // IN: fdatasync or fsync
+          int datasync)                 // IN: fdatasync or fsync
 {
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsFsync: was called\n"));
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s(%s/%s, %lld, %lld, %d)\n",
+           __func__,
+           file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name,
+#if defined VMW_FSYNC_31
+           start, end,
+#else
+           (loff_t)0, (loff_t)0,
+#endif
+           datasync));
 
    return 0;
 }
@@ -1024,8 +972,8 @@ HgfsFsync(struct file *file,		// IN: File we operate on
  */
 
 static int
-HgfsMmap(struct file *file,		// IN: File we operate on
-         struct vm_area_struct *vma)	// IN/OUT: VM area information
+HgfsMmap(struct file *file,            // IN: File we operate on
+         struct vm_area_struct *vma)   // IN/OUT: VM area information
 {
    int result;
 
@@ -1033,11 +981,14 @@ HgfsMmap(struct file *file,		// IN: File we operate on
    ASSERT(vma);
    ASSERT(file->f_dentry);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsMmap: was called\n"));
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s(%s/%s)\n",
+           __func__,
+           file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name));
 
    result = HgfsRevalidate(file->f_dentry);
    if (result) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsMmap: invalid dentry\n"));
+      LOG(4, (KERN_DEBUG "VMware hgfs: %s: invalid dentry\n", __func__));
       goto out;
    }
 
@@ -1080,7 +1031,11 @@ HgfsRelease(struct inode *inode,  // IN: Inode that this file points to
    ASSERT(file->f_dentry->d_sb);
 
    handle = FILE_GET_FI_P(file)->handle;
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsRelease: close fh %u\n", handle));
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s(%s/%s, %u)\n",
+           __func__,
+           file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name,
+           handle));
 
    /*
     * This may be our last open handle to an inode, so we should flush our
@@ -1262,16 +1217,20 @@ HgfsSpliceRead(struct file *file,            // IN: File to read from
    ASSERT(file);
    ASSERT(file->f_dentry);
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsSpliceRead: was called\n"));
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s(%s/%s, %lu@%Lu)\n",
+           __func__,
+           file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name,
+           (unsigned long) len, (unsigned long long) *offset));
 
    result = HgfsRevalidate(file->f_dentry);
    if (result) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsSpliceRead: invalid dentry\n"));
+      LOG(4, (KERN_DEBUG "VMware hgfs: %s: invalid dentry\n", __func__));
       goto out;
    }
 
    result = generic_file_splice_read(file, offset, pipe, len, flags);
-  out:
+out:
    return result;
 
 }
