@@ -45,11 +45,20 @@ static int HgfsFollowlink(struct dentry *dentry,
 static int HgfsReadlink(struct dentry *dentry,
                         char __user *buffer,
                         int buflen);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 13)
+static void HgfsPutlink(struct dentry *dentry,
+                        struct nameidata *nd,
+                        void *cookie);
+#else
+static void HgfsPutlink(struct dentry *dentry,
+                        struct nameidata *nd);
+#endif
 
 /* HGFS inode operations structure for symlinks. */
 struct inode_operations HgfsLinkInodeOperations = {
    .follow_link   = HgfsFollowlink,
    .readlink      = HgfsReadlink,
+   .put_link      = HgfsPutlink,
 };
 
 /*
@@ -87,6 +96,7 @@ HgfsFollowlink(struct dentry *dentry, // IN: Dentry containing link
 #endif
 {
    HgfsAttrInfo attr;
+   char *fileName = NULL;
    int error;
 
    ASSERT(dentry);
@@ -98,24 +108,24 @@ HgfsFollowlink(struct dentry *dentry, // IN: Dentry containing link
       goto out;
    }
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsFollowlink: calling "
-           "HgfsPrivateGetattr\n"));
-   attr.fileName = NULL;
-   error = HgfsPrivateGetattr(dentry, &attr);
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: calling HgfsPrivateGetattr %s\n",
+           __func__, dentry->d_name.name));
+   error = HgfsPrivateGetattr(dentry, &attr, &fileName);
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: HgfsPrivateGetattr %s ret %d\n",
+           __func__, dentry->d_name.name, error));
    if (!error) {
 
       /* Let's make sure we got called on a symlink. */
-      if (attr.type != HGFS_FILE_TYPE_SYMLINK ||
-          attr.fileName == NULL) {
+      if (attr.type != HGFS_FILE_TYPE_SYMLINK || fileName == NULL) {
          LOG(6, (KERN_DEBUG "VMware hgfs: HgfsFollowlink: got called "
                  "on something that wasn't a symlink\n"));
          error = -EINVAL;
+         kfree(fileName);
       } else {
-         LOG(6, (KERN_DEBUG "VMware hgfs: HgfsFollowlink: calling "
-                 "vfs_follow_link\n"));
-         error = vfs_follow_link(nd, attr.fileName);
+         LOG(6, (KERN_DEBUG "VMware hgfs: %s: calling nd_set_link %s\n",
+                 __func__, fileName));
+         nd_set_link(nd, fileName);
       }
-      kfree(attr.fileName);
    }
   out:
 
@@ -151,6 +161,7 @@ HgfsReadlink(struct dentry *dentry,  // IN:  Dentry containing link
 
 {
    HgfsAttrInfo attr;
+   char *fileName = NULL;
    int error;
 
    ASSERT(dentry);
@@ -161,24 +172,67 @@ HgfsReadlink(struct dentry *dentry,  // IN:  Dentry containing link
       return -EINVAL;
    }
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsReadlink: calling "
-           "HgfsPrivateGetattr\n"));
-   attr.fileName = NULL;
-   error = HgfsPrivateGetattr(dentry, &attr);
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: calling HgfsPrivateGetattr %s\n",
+           __func__, dentry->d_name.name));
+   error = HgfsPrivateGetattr(dentry, &attr, &fileName);
    if (!error) {
 
       /* Let's make sure we got called on a symlink. */
-      if (attr.type != HGFS_FILE_TYPE_SYMLINK ||
-          attr.fileName == NULL) {
+      if (attr.type != HGFS_FILE_TYPE_SYMLINK || fileName == NULL) {
          LOG(6, (KERN_DEBUG "VMware hgfs: HgfsReadlink: got called "
                  "on something that wasn't a symlink\n"));
          error = -EINVAL;
       } else {
-         LOG(6, (KERN_DEBUG "VMware hgfs: HgfsReadlink: calling "
-                 "vfs_readlink\n"));
-         error = vfs_readlink(dentry, buffer, buflen, attr.fileName);
+         LOG(6, (KERN_DEBUG "VMware hgfs: %s: calling vfs_readlink %s\n",
+                 __func__, fileName));
+         error = vfs_readlink(dentry, buffer, buflen, fileName);
+         LOG(6, (KERN_DEBUG "VMware hgfs: %s: vfs_readlink %s ret %dn",
+                 __func__, fileName, error));
       }
-      kfree(attr.fileName);
+      kfree(fileName);
    }
    return error;
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * HgfsPutlink --
+ *
+ *    Modeled after page_put_link from a 2.6.9 kernel so it'll work
+ *    across all kernel revisions we care about.
+ *
+ * Results:
+ *    None
+ *
+ * Side effects:
+ *    None
+ *
+ *----------------------------------------------------------------------
+ */
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 13)
+static void
+HgfsPutlink(struct dentry *dentry, // dentry
+            struct nameidata *nd,  // lookup name information
+            void *cookie)          // cookie
+#else
+static void
+HgfsPutlink(struct dentry *dentry, // dentry
+            struct nameidata *nd)  // lookup name information
+#endif
+{
+   char *fileName = NULL;
+
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: put for %s\n",
+           __func__, dentry->d_name.name));
+
+   fileName = nd_get_link(nd);
+   if (!IS_ERR(fileName)) {
+      LOG(6, (KERN_DEBUG "VMware hgfs: %s: putting %s\n",
+              __func__, fileName));
+      kfree(fileName);
+      nd_set_link(nd, NULL);
+   }
+}
