@@ -32,6 +32,7 @@
 #include "compat_fs.h"
 #include "compat_kernel.h"
 #include "compat_pagemap.h"
+#include "compat_highmem.h"
 #include <linux/writeback.h>
 
 #include "cpName.h"
@@ -63,9 +64,10 @@ static int HgfsDoWritepage(HgfsHandle handle,
                            struct page *page,
                            unsigned pageFrom,
                            unsigned pageTo);
-static void HgfsDoWriteBegin(struct page *page,
-                             unsigned pageFrom,
-                             unsigned pageTo);
+static int HgfsDoWriteBegin(struct file *file,
+                            struct page *page,
+                            unsigned pageFrom,
+                            unsigned pageTo);
 static int HgfsDoWriteEnd(struct file *file,
                           struct page *page,
                           unsigned pageFrom,
@@ -176,7 +178,7 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
 
    req = HgfsGetNewRequest();
    if (!req) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: out of memory while "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: out of memory while "
               "getting new request\n"));
       result = -ENOMEM;
       goto out;
@@ -200,14 +202,14 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
       req->dataPacket = kmalloc(numEntries * sizeof req->dataPacket[0],
                                 GFP_KERNEL);
       if (!req->dataPacket) {
-         LOG(4, (KERN_DEBUG "%s: Failed to allocate mem\n", __func__));
+         LOG(4, (KERN_WARNING "%s: Failed to allocate mem\n", __func__));
          result = -ENOMEM;
          goto out;
       }
       memcpy(req->dataPacket, dataPacket, numEntries * sizeof req->dataPacket[0]);
       req->numEntries = numEntries;
 
-      LOG(4, (KERN_DEBUG "VMware hgfs: Fast Read V4\n"));
+      LOG(4, (KERN_WARNING "VMware hgfs: Fast Read V4\n"));
    } else if (opUsed == HGFS_OP_READ_V3) {
       HgfsRequest *header;
       HgfsRequestReadV3 *request;
@@ -219,9 +221,11 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
       request = (HgfsRequestReadV3 *)(HGFS_REQ_PAYLOAD_V3(req));
       request->file = handle;
       request->offset = offset;
-      request->requiredSize = count;
+      request->requiredSize = MIN(req->bufferSize - sizeof *request -
+                                  sizeof *header, count);
       request->reserved = 0;
       req->dataPacket = NULL;
+      req->numEntries = 0;
       req->payloadSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
    } else {
       HgfsRequestRead *request;
@@ -231,8 +235,9 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
       request->header.op = opUsed;
       request->file = handle;
       request->offset = offset;
-      request->requiredSize = count;
+      request->requiredSize = MIN(req->bufferSize - sizeof *request, count);
       req->dataPacket = NULL;
+      req->numEntries = 0;
       req->payloadSize = sizeof *request;
    }
 
@@ -257,14 +262,14 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
 
          /* Sanity check on read size. */
          if (actualSize > count) {
-            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: read too big!\n"));
+            LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: read too big!\n"));
             result = -EPROTO;
             goto out;
          }
 
          if (!actualSize) {
             /* We got no bytes. */
-            LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoRead: server returned "
+            LOG(6, (KERN_WARNING "VMware hgfs: HgfsDoRead: server returned "
                    "zero\n"));
             result = actualSize;
             goto out;
@@ -275,24 +280,28 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
             buf = kmap(dataPacket[0].page) + dataPacket[0].offset;
             ASSERT(buf);
             memcpy(buf, payload, actualSize);
-            LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoRead: copied %u\n",
+            LOG(6, (KERN_WARNING "VMware hgfs: HgfsDoRead: copied %u\n",
                     actualSize));
             kunmap(dataPacket[0].page);
          }
          result = actualSize;
-	 break;
+	      break;
 
       case -EPROTO:
          /* Retry with older version(s). Set globally. */
          switch (opUsed) {
          case HGFS_OP_READ_FAST_V4:
-            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: Fast Read V4 not "
+            LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: Fast Read V4 not "
                     "supported. Falling back to V3 Read.\n"));
+            if (req->dataPacket) {
+               kfree(req->dataPacket);
+               req->dataPacket = NULL;
+            }
             hgfsVersionRead = HGFS_OP_READ_V3;
             goto retry;
 
          case HGFS_OP_READ_V3:
-            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: Version 3 not "
+            LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: Version 3 not "
                     "supported. Falling back to version 1.\n"));
             hgfsVersionRead = HGFS_OP_READ;
             goto retry;
@@ -300,18 +309,18 @@ HgfsDoRead(HgfsHandle handle,             // IN:  Handle for this file
          default:
             break;
          }
-	 break;
+	      break;
 
       default:
          break;
       }
    } else if (result == -EIO) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: timed out\n"));
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: timed out\n"));
    } else if (result == -EPROTO) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: server "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: server "
               "returned error: %d\n", result));
    } else {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoRead: unknown error: "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoRead: unknown error: "
               "%d\n", result));
    }
 
@@ -372,7 +381,7 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
 
    req = HgfsGetNewRequest();
    if (!req) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: out of memory while "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: out of memory while "
               "getting new request\n"));
       result = -ENOMEM;
       goto out;
@@ -400,7 +409,7 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
       req->dataPacket = kmalloc(numEntries * sizeof req->dataPacket[0],
                                 GFP_KERNEL);
       if (!req->dataPacket) {
-         LOG(4, (KERN_DEBUG "%s: Failed to allocate mem\n", __func__));
+         LOG(4, (KERN_WARNING "%s: Failed to allocate mem\n", __func__));
          result = -ENOMEM;
          goto out;
       }
@@ -408,7 +417,7 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
       req->numEntries = numEntries;
       reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
       req->payloadSize = reqSize;
-      LOG(4, (KERN_DEBUG "VMware hgfs: Fast Write V4\n"));
+      LOG(4, (KERN_WARNING "VMware hgfs: Fast Write V4\n"));
    } else if (opUsed == HGFS_OP_WRITE_V3) {
       HgfsRequest *header;
       HgfsRequestWriteV3 *request;
@@ -421,12 +430,15 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
       request->file = handle;
       request->flags = 0;
       request->offset = offset;
-      request->requiredSize = count;
+      request->requiredSize = MIN(req->bufferSize - sizeof *header -
+                                  sizeof *request, count);
+      LOG(4, (KERN_WARNING "VMware hgfs: Using write V3\n"));
       request->reserved = 0;
       payload = request->payload;
       requiredSize = request->requiredSize;
       reqSize = HGFS_REQ_PAYLOAD_SIZE_V3(request);
       req->dataPacket = NULL;
+      req->numEntries = 0;
       buf = kmap(dataPacket[0].page) + dataPacket[0].offset;
       memcpy(payload, buf, requiredSize);
       kunmap(dataPacket[0].page);
@@ -441,11 +453,12 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
       request->file = handle;
       request->flags = 0;
       request->offset = offset;
-      request->requiredSize = count;
+      request->requiredSize = MIN(req->bufferSize - sizeof *request, count);
       payload = request->payload;
       requiredSize = request->requiredSize;
       reqSize = sizeof *request;
       req->dataPacket = NULL;
+      req->numEntries = 0;
       buf = kmap(dataPacket[0].page) + dataPacket[0].offset;
       memcpy(payload, buf, requiredSize);
       kunmap(dataPacket[0].page);
@@ -460,6 +473,7 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
       replyStatus = HgfsReplyStatus(req);
       result = HgfsStatusConvertToLinux(replyStatus);
 
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: res %u\n", result));
       switch (result) {
       case 0:
          if (opUsed == HGFS_OP_WRITE_V3 || opUsed == HGFS_OP_WRITE_FAST_V4) {
@@ -469,7 +483,7 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
          }
 
          /* Return result. */
-         LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: wrote %u bytes\n",
+         LOG(6, (KERN_WARNING "VMware hgfs: HgfsDoWrite: wrote %u bytes\n",
                  actualSize));
          result = actualSize;
          break;
@@ -478,13 +492,17 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
          /* Retry with older version(s). Set globally. */
          switch (opUsed) {
          case HGFS_OP_WRITE_FAST_V4:
-            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: Fast Write V4 not "
+            LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: Fast Write V4 not "
                     "supported. Falling back to V3 write.\n"));
-            hgfsVersionRead = HGFS_OP_WRITE_V3;
+            if (req->dataPacket) {
+               kfree(req->dataPacket);
+               req->dataPacket = NULL;
+            }
+            hgfsVersionWrite = HGFS_OP_WRITE_V3;
             goto retry;
 
          case HGFS_OP_WRITE_V3:
-            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: Version 3 not "
+            LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: Version 3 not "
                     "supported. Falling back to version 1.\n"));
             hgfsVersionWrite = HGFS_OP_WRITE;
             goto retry;
@@ -495,17 +513,17 @@ HgfsDoWrite(HgfsHandle handle,             // IN: Handle for this file
          break;
 
       default:
-         LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: server "
+         LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: server "
                  "returned error: %d\n", result));
          break;
       }
    } else if (result == -EIO) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: timed out\n"));
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: timed out\n"));
    } else if (result == -EPROTO) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: server "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: server "
               "returned error: %d\n", result));
    } else {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWrite: unknown error: "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoWrite: unknown error: "
               "%d\n", result));
    }
 
@@ -549,7 +567,7 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
    size_t nextCount, remainingCount = pageTo - pageFrom;
    HgfsDataPacket dataPacket[1];
 
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsDoReadpage: read %Zu bytes from fh %u "
+   LOG(6, (KERN_WARNING "VMware hgfs: HgfsDoReadpage: read %Zu bytes from fh %u "
            "at offset %Lu\n", remainingCount, handle, curOffset));
 
    /*
@@ -566,7 +584,7 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
       dataPacket[0].len = nextCount;
       result = HgfsDoRead(handle, dataPacket, 1, curOffset);
       if (result < 0) {
-         LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoReadpage: read error %d\n",
+         LOG(4, (KERN_WARNING "VMware hgfs: HgfsDoReadpage: read error %d\n",
                  result));
          goto out;
       }
@@ -582,6 +600,8 @@ HgfsDoReadpage(HgfsHandle handle,  // IN:     Handle to use for reading
     */
    if (remainingCount) {
       char *buffer = kmap(page) + pageTo;
+      LOG(6, (KERN_DEBUG "VMware hgfs: %s: zeroing last %Zu bytes\n",
+              __func__, remainingCount));
       memset(buffer - remainingCount, 0, remainingCount);
       kunmap(page);
    }
@@ -651,6 +671,8 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
    ASSERT(page->mapping->host);
    inode = page->mapping->host;
 
+   LOG(4, (KERN_WARNING "VMware hgfs: %s: start writes at %Lu\n",
+           __func__, curOffset));
    /*
     * Call HgfsDoWrite repeatedly until either
     * - HgfsDoWrite returns an error, or
@@ -665,8 +687,8 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
       dataPacket[0].len = nextCount;
       result = HgfsDoWrite(handle, dataPacket, 1, curOffset);
       if (result < 0) {
-         LOG(4, (KERN_DEBUG "VMware hgfs: HgfsDoWritepage: write error %d\n",
-                 result));
+         LOG(4, (KERN_WARNING "VMware hgfs: %s: write error %d\n",
+                 __func__, result));
          goto out;
       }
       remainingCount -= result;
@@ -680,6 +702,8 @@ HgfsDoWritepage(HgfsHandle handle,  // IN: Handle to use for writing
    } while ((result > 0) && (remainingCount > 0));
 
    result = 0;
+   LOG(4, (KERN_WARNING "VMware hgfs: %s: end writes at %Lu rem %zu\n",
+           __func__, curOffset, remainingCount));
 
   out:
    return result;
@@ -724,7 +748,7 @@ HgfsReadpage(struct file *file, // IN:     File to read from
    ASSERT(page);
 
    handle = FILE_GET_FI_P(file)->handle;
-   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsReadPage: reading from handle %u\n",
+   LOG(6, (KERN_WARNING "VMware hgfs: HgfsReadPage: reading from handle %u\n",
            handle));
 
    page_cache_get(page);
@@ -776,7 +800,7 @@ HgfsWritepage(struct page *page,             // IN: Page to write from
                           HGFS_OPEN_MODE_WRITE_ONLY + 1,
                           &handle);
    if (result) {
-      LOG(4, (KERN_DEBUG "VMware hgfs: HgfsWritepage: could not get writable "
+      LOG(4, (KERN_WARNING "VMware hgfs: HgfsWritepage: could not get writable "
               "file handle\n"));
       goto exit;
    }
@@ -795,6 +819,8 @@ HgfsWritepage(struct page *page,             // IN: Page to write from
     */
    currentFileSize = compat_i_size_read(inode);
    lastPageIndex = currentFileSize >> PAGE_CACHE_SHIFT;
+   LOG(4, (KERN_WARNING "VMware hgfs: %s: file size lpi %lu pi %lu\n",
+           __func__, lastPageIndex, page->index));
    if (page->index > lastPageIndex) {
       goto exit;
    } else if (page->index == lastPageIndex) {
@@ -850,7 +876,7 @@ HgfsWritepage(struct page *page,             // IN: Page to write from
  *      Initialize the page if the file is to be appended.
  *
  * Results:
- *      None.
+ *    Zero on success, always.
  *
  * Side effects:
  *      None.
@@ -858,37 +884,38 @@ HgfsWritepage(struct page *page,             // IN: Page to write from
  *-----------------------------------------------------------------------------
  */
 
-static void
-HgfsDoWriteBegin(struct page *page,         // IN: Page to be written
+static int
+HgfsDoWriteBegin(struct file *file,         // IN: File to be written
+                 struct page *page,         // IN: Page to be written
                  unsigned pageFrom,         // IN: Starting page offset
                  unsigned pageTo)           // IN: Ending page offset
 {
-   loff_t offset;
-   loff_t currentFileSize;
-
    ASSERT(page);
 
-   offset = (loff_t)page->index << PAGE_CACHE_SHIFT;
-   currentFileSize = compat_i_size_read(page->mapping->host);
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: off %Lu: %u to %u\n", __func__,
+           (loff_t)page->index << PAGE_CACHE_SHIFT, pageFrom, pageTo));
 
-   /*
-    * If we are doing a partial write into a new page (beyond end of
-    * file), then intialize it. This allows other writes to this page
-    * to accumulate before we need to write it to the server.
-    */
-   if ((offset >= currentFileSize) ||
-       ((pageFrom == 0) && (offset + pageTo) >= currentFileSize)) {
-      void *kaddr = kmap_atomic(page, KM_USER0);
-
-      if (pageFrom) {
+   if (!PageUptodate(page)) {
+      /*
+       * If we are doing a partial write into a new page (beyond end of
+       * file), then intialize it. This allows other writes to this page
+       * to accumulate before we need to write it to the server.
+       */
+      if (pageTo - pageFrom != PAGE_CACHE_SIZE) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+         zero_user_segments(page, 0, pageFrom, pageTo, PAGE_CACHE_SIZE);
+#else
+         void *kaddr = compat_kmap_atomic(page);
          memset(kaddr, 0, pageFrom);
-      }
-      if (pageTo < PAGE_CACHE_SIZE) {
          memset(kaddr + pageTo, 0, PAGE_CACHE_SIZE - pageTo);
+         flush_dcache_page(page);
+         compat_kunmap_atomic(kaddr);
+#endif
       }
-      kunmap_atomic(kaddr, KM_USER0);
-      flush_dcache_page(page);
    }
+
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: returns 0\n", __func__));
+   return 0;
 }
 
 
@@ -903,7 +930,7 @@ HgfsDoWriteBegin(struct page *page,         // IN: Page to be written
  *      receiving the write.
  *
  * Results:
- *      Always zero.
+ *      On success zero, always.
  *
  * Side effects:
  *      None.
@@ -912,14 +939,12 @@ HgfsDoWriteBegin(struct page *page,         // IN: Page to be written
  */
 
 static int
-HgfsPrepareWrite(struct file *file,  // IN: Ignored
+HgfsPrepareWrite(struct file *file,  // IN: File to be written
                  struct page *page,  // IN: Page to prepare
                  unsigned pageFrom,  // IN: Beginning page offset
                  unsigned pageTo)    // IN: Ending page offset
 {
-   HgfsDoWriteBegin(page, pageFrom, pageTo);
-
-   return 0;
+   return HgfsDoWriteBegin(file, page, pageFrom, pageTo);
 }
 
 #else
@@ -955,18 +980,34 @@ HgfsWriteBegin(struct file *file,             // IN: File to be written
                void **clientData)             // OUT: Opaque to pass to write_end, unused
 {
    pgoff_t index = pos >> PAGE_CACHE_SHIFT;
-   unsigned pageFrom = pos & (PAGE_CACHE_SHIFT - 1);
-   unsigned pageTo = pos + len;
+   unsigned pageFrom = pos & (PAGE_CACHE_SIZE - 1);
+   unsigned pageTo = pageFrom + len;
    struct page *page;
+   int result;
+
+   LOG(6, (KERN_WARNING "VMware hgfs: %s: (%s/%s(%ld), %u@%lld)\n",
+           __func__, file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name,
+           mapping->host->i_ino, len, (long long) pos));
 
    page = compat_grab_cache_page_write_begin(mapping, index, flags);
    if (page == NULL) {
-      return -ENOMEM;
+      result = -ENOMEM;
+      goto exit;
    }
    *pagePtr = page;
 
-   HgfsDoWriteBegin(page, pageFrom, pageTo);
-   return 0;
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: file size %Lu @ %Lu page %u to %u\n", __func__,
+         (loff_t)compat_i_size_read(page->mapping->host),
+         (loff_t)page->index << PAGE_CACHE_SHIFT,
+         pageFrom, pageTo));
+
+   result = HgfsDoWriteBegin(file, page, pageFrom, pageTo);
+   ASSERT(result == 0);
+
+exit:
+   LOG(6, (KERN_DEBUG "VMware hgfs: %s: return %d\n", __func__, result));
+   return result;
 }
 #endif
 
@@ -998,54 +1039,35 @@ HgfsDoWriteEnd(struct file *file, // IN: File we're writing to
                loff_t writeTo,    // IN: File position to write to
                unsigned copied)   // IN: Number of bytes copied to the page
 {
-   HgfsHandle handle;
-   struct inode *inode;
    loff_t currentFileSize;
-   loff_t offset;
+   struct inode *inode;
 
    ASSERT(file);
    ASSERT(page);
    inode = page->mapping->host;
    currentFileSize = compat_i_size_read(inode);
-   offset = (loff_t)page->index << PAGE_CACHE_SHIFT;
+
+   LOG(6, (KERN_WARNING "VMware hgfs: %s: (%s/%s(%ld), from %u to %u@%lld => %u)\n",
+           __func__, file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name,
+           page->mapping->host->i_ino, pageFrom, pageTo, (long long) writeTo, copied));
+
+   /*
+    * Zero any uninitialised parts of the page, and then mark the page
+    * as up to date if it turns out that we're extending the file.
+    */
+   if (!PageUptodate(page)) {
+      SetPageUptodate(page);
+   }
 
    if (writeTo > currentFileSize) {
       compat_i_size_write(inode, writeTo);
    }
 
-   /* We wrote a complete page, so it is up to date. */
-   if (copied == PAGE_CACHE_SIZE) {
-      SetPageUptodate(page);
-   }
+   set_page_dirty(page);
 
-   /*
-    * Check if this is a partial write to a new page, which was
-    * initialized in HgfsDoWriteBegin.
-    */
-   if ((offset >= currentFileSize) ||
-       ((pageFrom == 0) && (writeTo >= currentFileSize))) {
-      SetPageUptodate(page);
-   }
-
-   /*
-    * If the page is uptodate, then just mark it dirty and let
-    * the page cache write it when it wants to.
-    */
-   if (PageUptodate(page)) {
-      set_page_dirty(page);
-      return 0;
-   }
-
-   /*
-    * We've recieved a partial write to page that is not uptodate, so
-    * do the write now while the page is still locked.  Another
-    * alternative would be to read the page in HgfsDoWriteBegin, which
-    * would make it uptodate (ie a complete cached page).
-    */
-   handle = FILE_GET_FI_P(file)->handle;
-   LOG(6, (KERN_DEBUG "VMware hgfs: %s: writing to handle %u\n", __func__,
-           handle));
-   return HgfsDoWritepage(handle, page, pageFrom, pageTo);
+   LOG(6, (KERN_WARNING "VMware hgfs: %s: return 0\n", __func__));
+   return 0;
 }
 
 
@@ -1127,7 +1149,7 @@ HgfsWriteEnd(struct file *file,              // IN: File to write
              void *clientData)               // IN: From write_begin, unused.
 {
    unsigned pageFrom = pos & (PAGE_CACHE_SIZE - 1);
-   unsigned pageTo = pageFrom + copied;
+   unsigned pageTo = pageFrom + len;
    loff_t writeTo = pos + copied;
    int ret;
 
@@ -1135,6 +1157,16 @@ HgfsWriteEnd(struct file *file,              // IN: File to write
    ASSERT(mapping);
    ASSERT(page);
 
+
+   LOG(6, (KERN_WARNING "VMware hgfs: %s: (%s/%s(%ld), %u@%lld,=>%u)\n",
+           __func__, file->f_dentry->d_parent->d_name.name,
+           file->f_dentry->d_name.name,
+           mapping->host->i_ino, len, (long long) pos, copied));
+
+   if (copied < len) {
+      zero_user_segment(page, pageFrom + copied, pageFrom + len);
+   }
+
    ret = HgfsDoWriteEnd(file, page, pageFrom, pageTo, writeTo, copied);
    if (ret == 0) {
       ret = copied;
