@@ -59,7 +59,6 @@
 
 static void
 HgfsRequestInit(HgfsReq *req,       // IN: request to initialize
-                size_t bufferSize,  // Size of the buffer allocated with request
                 int requestId)      // IN: ID assigned to the request
 {
    ASSERT(req);
@@ -69,8 +68,8 @@ HgfsRequestInit(HgfsReq *req,       // IN: request to initialize
    init_waitqueue_head(&req->queue);
    req->id = requestId;
    req->payloadSize = 0;
-   req->bufferSize = bufferSize;
    req->state = HGFS_REQ_STATE_ALLOCATED;
+   req->numEntries = 0;
 }
 
 /*
@@ -102,12 +101,12 @@ HgfsGetNewRequest(void)
       return NULL;
    }
 
-   HgfsRequestInit(req, HGFS_PACKET_MAX,
-                   atomic_inc_return(&hgfsIdCounter) - 1);
+   HgfsRequestInit(req, atomic_inc_return(&hgfsIdCounter) - 1);
 
    return req;
 }
 
+
 /*
  *----------------------------------------------------------------------
  *
@@ -139,9 +138,12 @@ HgfsCopyRequest(HgfsReq *req)   // IN: request to be copied
       return NULL;
    }
 
-   HgfsRequestInit(newReq, req->bufferSize, req->id);
+   HgfsRequestInit(newReq, req->id);
 
-   /* Copy payload from the original request. */
+   memcpy(newReq->dataPacket, req->dataPacket,
+          req->numEntries * sizeof (req->dataPacket[0]));
+
+   newReq->numEntries = req->numEntries;
    newReq->payloadSize = req->payloadSize;
    memcpy(newReq->payload, req->payload, req->payloadSize);
 
@@ -170,14 +172,13 @@ HgfsSendRequest(HgfsReq *req)       // IN/OUT: Outgoing request
    int ret;
 
    ASSERT(req);
-   ASSERT(req->payloadSize <= HGFS_PACKET_MAX);
-
+   ASSERT(req->payloadSize <= req->bufferSize);
    req->state = HGFS_REQ_STATE_UNSENT;
 
-   LOG(8, (KERN_DEBUG "VMware hgfs: HgfsSendRequest: Sending request id %d\n",
+   LOG(10, (KERN_DEBUG "VMware hgfs: HgfsSendRequest: Sending request id %d\n",
            req->id));
    ret = HgfsTransportSendRequest(req);
-   LOG(8, (KERN_DEBUG "VMware hgfs: HgfsSendRequest: request finished, "
+   LOG(10, (KERN_DEBUG "VMware hgfs: HgfsSendRequest: request finished, "
            "return %d\n", ret));
 
    return ret;
@@ -205,7 +206,7 @@ static void HgfsRequestFreeMemory(struct kref *kref)
 
    LOG(10, (KERN_DEBUG "VMware hgfs: %s: freeing request %d\n",
             __func__, req->id));
-   kfree(req);
+   HgfsTransportFreeRequest(req);
 }
 
 /*
