@@ -42,7 +42,6 @@
 /* Must be included after semaphore.h. */
 #include <linux/timer.h>
 /* Must be included after sched.h. */
-#include <linux/smp_lock.h>
 #include <linux/interrupt.h> /* for spin_lock_bh */
 
 
@@ -168,24 +167,6 @@ HgfsTransportSetupNewChannel(void)
 
    USE_VMCI = 0;
 
-   newChannel = HgfsGetVSocketChannel();
-   if (newChannel != NULL) {
-      if (HgfsTransportOpenChannel(newChannel)) {
-         hgfsChannel = newChannel;
-         LOG(10, ("CHANNEL: Vsocket channel\n"));
-         return TRUE;
-      }
-   }
-
-   newChannel = HgfsGetTcpChannel();
-   if (newChannel != NULL) {
-      if (HgfsTransportOpenChannel(newChannel)) {
-         hgfsChannel = newChannel;
-         LOG(10, ("CHANNEL: Tcp channel\n"));
-         return TRUE;
-      }
-   }
-
    newChannel = HgfsGetBdChannel();
    LOG(10, ("CHANNEL: Bd channel\n"));
    ASSERT(newChannel);
