@@ -223,7 +223,7 @@ HgfsRequestAsyncShmemDispatch(HgfsAsyncIov *iov, // IN: request vectors
  *      Passes down free pages to the hgfs Server. HgfsServer will use this pages
  *      for sending change notification, oplock breaks etc.
  *
- *      XXX It seems safe to call VMCIDatagram_Send in atomic context.
+ *      XXX It seems safe to call vmci_datagram_send in atomic context.
  *
  * Results:
  *      None
@@ -243,6 +243,7 @@ HgfsVmciChannelPassGuestPages(HgfsTransportChannel *channel) // IN:
    int j = 0;
    size_t transportHeaderSize;
    HgfsVmciTransportHeader *transportHeader = NULL;
+   HgfsVmciHeaderNode *headerNode;
    VMCIDatagram *dg;
 
    if (!gHgfsShmemPages.freePageCount) {
@@ -260,6 +261,7 @@ HgfsVmciChannelPassGuestPages(HgfsTransportChannel *channel) // IN:
    }
 
    transportHeader = VMCI_DG_PAYLOAD(dg);
+   headerNode = &transportHeader->node;
 
    for (i = 0; i < gHgfsShmemPages.totalPageCount; i++) {
       if (gHgfsShmemPages.list[i].free) {
@@ -275,13 +277,14 @@ HgfsVmciChannelPassGuestPages(HgfsTransportChannel *channel) // IN:
    dg->dst = VMCI_MAKE_HANDLE(VMCI_HYPERVISOR_CONTEXT_ID, VMCI_HGFS_TRANSPORT);
    dg->payloadSize = transportHeaderSize;
 
-   transportHeader->version = HGFS_VMCI_VERSION_1;
+   headerNode->version = HGFS_VMCI_VERSION_1;
+   headerNode->pktType = HGFS_TH_REP_GET_PAGES;
+
    ASSERT(gHgfsShmemPages.freePageCount == j);
    transportHeader->iovCount = j;
-   transportHeader->pktType = HGFS_TH_REP_GET_PAGES;
 
    LOG(10, (KERN_WARNING "Sending %d Guest pages \n", i));
-   if ((ret = VMCIDatagram_Send(dg)) < VMCI_SUCCESS) {
+   if ((ret = vmci_datagram_send(dg)) < VMCI_SUCCESS) {
       if (ret == HGFS_VMCI_TRANSPORT_ERROR) {
          LOG(0, (KERN_WARNING "HGFS Transport error occured. Don't blame VMCI\n"));
       }
@@ -376,15 +379,16 @@ static int HgfsVmciChannelCallback(void *data,       // IN: unused
                                    VMCIDatagram *dg) // IN: datagram
 {
    HgfsVmciAsyncReply *reply  = (HgfsVmciAsyncReply *)VMCI_DG_PAYLOAD(dg);
+   HgfsVmciHeaderNode *replyNode = &reply->node;
    HgfsTransportChannel *channel;
 
    LOG(10, (KERN_WARNING "Received VMCI channel Callback \n"));
 
-   if (reply->version != HGFS_VMCI_VERSION_1) {
+   if (replyNode->version != HGFS_VMCI_VERSION_1) {
       return HGFS_VMCI_VERSION_MISMATCH;
    }
 
-   switch (reply->pktType) {
+   switch (replyNode->pktType) {
 
    case HGFS_ASYNC_IOREP:
       LOG(10, (KERN_WARNING "Received ID%"FMT64"x \n", reply->response.id));
@@ -447,11 +451,12 @@ HgfsVmciChannelOpen(HgfsTransportChannel *channel) // IN: Channel
       goto error;
    }
 
-   ret = VMCIDatagram_CreateHnd(VMCI_INVALID_ID,        /* Resource ID */
-                                VMCI_FLAG_DG_NONE,      /* Flags */
-                                HgfsVmciChannelCallback,/* Datagram Recv Callback */
-                                NULL,                   /* Callback data */
-                                channel->priv);         /* VMCI outhandle */
+   ret = vmci_datagram_create_handle(
+                          VMCI_INVALID_ID,        /* Resource ID */
+                          VMCI_FLAG_DG_NONE,      /* Flags */
+                          HgfsVmciChannelCallback,/* Datagram Recv Callback */
+                          NULL,                   /* Callback data */
+                          channel->priv);         /* VMCI outhandle */
    if (ret != VMCI_SUCCESS) {
       LOG(1, (KERN_WARNING "Failed to create VMCI handle %d\n", ret));
       goto error;
@@ -524,6 +529,7 @@ HgfsVmciChannelTerminateSession(HgfsTransportChannel *channel) {
    int ret = 0;
    VMCIDatagram *dg;
    HgfsVmciTransportHeader *transportHeader;
+   HgfsVmciHeaderNode *headerNode;
 
    dg = kmalloc(sizeof *dg + sizeof *transportHeader, GFP_KERNEL);
    if (NULL == dg) {
@@ -537,12 +543,15 @@ HgfsVmciChannelTerminateSession(HgfsTransportChannel *channel) {
    dg->payloadSize = sizeof *transportHeader;
 
    transportHeader = VMCI_DG_PAYLOAD(dg);
-   transportHeader->version = HGFS_VMCI_VERSION_1;
+   headerNode = &transportHeader->node;
+
+   headerNode->pktType = HGFS_TH_TERMINATE_SESSION;
+   headerNode->version = HGFS_VMCI_VERSION_1;
+
    transportHeader->iovCount = 0;
-   transportHeader->pktType = HGFS_TH_TERMINATE_SESSION;
 
    LOG(1, (KERN_WARNING "Terminating session with host \n"));
-   if ((ret = VMCIDatagram_Send(dg)) < VMCI_SUCCESS) {
+   if ((ret = vmci_datagram_send(dg)) < VMCI_SUCCESS) {
       if (ret == HGFS_VMCI_TRANSPORT_ERROR) {
          LOG(0, (KERN_WARNING "HGFS Transport error occured. Don't blame VMCI\n"));
       }
@@ -580,7 +589,7 @@ HgfsVmciChannelClose(HgfsTransportChannel *channel) // IN: Channel
 {
    ASSERT(channel->priv != NULL);
    HgfsVmciChannelTerminateSession(channel);
-   VMCIDatagram_DestroyHnd(*(VMCIHandle *)channel->priv);
+   vmci_datagram_destroy_handle(*(VMCIHandle *)channel->priv);
    kfree(channel->priv);
    kfree(gHgfsShmemPages.list);
    channel->priv = NULL;
@@ -672,6 +681,7 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    int iovCount = 0;
    VMCIDatagram *dg;
    HgfsVmciTransportHeader *transportHeader;
+   HgfsVmciHeaderNode *headerNode;
    HgfsVmciTransportStatus *transportStatus;
    size_t transportHeaderSize;
    size_t bufferSize;
@@ -707,7 +717,10 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    dg->payloadSize = transportHeaderSize;
 
    transportHeader = VMCI_DG_PAYLOAD(dg);
-   transportHeader->version = HGFS_VMCI_VERSION_1;
+   headerNode = &transportHeader->node;
+
+   headerNode->version = HGFS_VMCI_VERSION_1;
+   headerNode->pktType = HGFS_TH_REQUEST;
 
    total = req->bufferSize + sizeof (HgfsVmciTransportStatus);
    bufferSize = 0;
@@ -746,7 +759,6 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    }
 
    transportHeader->iovCount = iovCount;
-   transportHeader->pktType = HGFS_TH_REQUEST;
 
    /* Initialize transport Status */
    transportStatus = (HgfsVmciTransportStatus *)req->buffer;
@@ -754,14 +766,14 @@ HgfsVmciChannelSend(HgfsTransportChannel *channel, // IN: Channel
    transportStatus->size = req->bufferSize + sizeof (HgfsVmciTransportStatus);
 
    /*
-    * Don't try to set req->state after VMCIDatagram_Send().
+    * Don't try to set req->state after vmci_datagram_send().
     * It may be too late then. We could have received a datagram by then and
     * datagram handler expects request's state to be submitted.
     */
    req->state = HGFS_REQ_STATE_SUBMITTED;
    id = req->id;
 
-   if ((ret = VMCIDatagram_Send(dg)) < VMCI_SUCCESS) {
+   if ((ret = vmci_datagram_send(dg)) < VMCI_SUCCESS) {
       if (ret == HGFS_VMCI_TRANSPORT_ERROR) {
          LOG(0, (KERN_WARNING "HGFS Transport error occured. Don't blame VMCI\n"));
       } else if (ret == HGFS_VMCI_VERSION_MISMATCH) {
