@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2012,2014 VMware, Inc. All rights reserved.
+ * Copyright (C) 2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,6 +26,9 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of VMware Inc. nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission of VMware Inc.
  *
  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -284,9 +287,9 @@ Backdoor_MonitorLockPage(Balloon *b,     // IN/OUT
                         &b->resetFlag);
 
    /* update stats */
-   STATS_INC(b->stats.lock[FALSE]);
+   STATS_INC(b->stats.lock);
    if (status != BALLOON_SUCCESS) {
-      STATS_INC(b->stats.lockFail[FALSE]);
+      STATS_INC(b->stats.lockFail);
    }
 
    return status;
@@ -328,9 +331,9 @@ Backdoor_MonitorUnlockPage(Balloon *b,     // IN/OUT
                         &b->resetFlag);
 
    /* update stats */
-   STATS_INC(b->stats.unlock[FALSE]);
+   STATS_INC(b->stats.unlock);
    if (status != BALLOON_SUCCESS) {
-      STATS_INC(b->stats.unlockFail[FALSE]);
+      STATS_INC(b->stats.unlockFail);
    }
 
    return status;
@@ -353,27 +356,18 @@ Backdoor_MonitorUnlockPage(Balloon *b,     // IN/OUT
  */
 
 int
-Backdoor_MonitorLockPagesBatched(Balloon *b,      // IN/OUT
-                                 PPN64 ppn,       // IN
-                                 uint32 nPages,   // IN
-                                 int isLargePage, // IN
-                                 uint32 *target)  // OUT
+Backdoor_MonitorLockPagesBatched(Balloon *b,     // IN/OUT
+                                 PPN64 ppn,      // IN
+                                 uint32 nPages,  // IN
+                                 uint32 *target) // OUT
 {
-   int status;
-   uint16 cmd;
-
-   if (isLargePage) {
-      cmd = BALLOON_BDOOR_CMD_BATCHED_2M_LOCK;
-   } else {
-      cmd = BALLOON_BDOOR_CMD_BATCHED_LOCK;
-   }
-
-   status = BackdoorCmd(cmd, (size_t)ppn, nPages, target, &b->resetFlag);
+   int status = BackdoorCmd(BALLOON_BDOOR_CMD_BATCHED_LOCK,
+                            (size_t)ppn, nPages, target, &b->resetFlag);
 
    /* update stats */
-   STATS_INC(b->stats.lock[isLargePage]);
+   STATS_INC(b->stats.lock);
    if (status != BALLOON_SUCCESS) {
-      STATS_INC(b->stats.lockFail[isLargePage]);
+      STATS_INC(b->stats.lockFail);
    }
 
    return status;
@@ -396,27 +390,18 @@ Backdoor_MonitorLockPagesBatched(Balloon *b,      // IN/OUT
  */
 
 int
-Backdoor_MonitorUnlockPagesBatched(Balloon *b,      // IN/OUT
-                                   PPN64 ppn,       // IN
-                                   uint32 nPages,   // IN
-                                   int isLargePage, // IN
-                                   uint32 *target)  // OUT
+Backdoor_MonitorUnlockPagesBatched(Balloon *b,          // IN/OUT
+                                   PPN64 ppn,           // IN
+                                   uint32 nPages,       // IN
+                                   uint32 *target)      // OUT
 {
-   int status;
-   uint16 cmd;
-
-   if (isLargePage) {
-      cmd = BALLOON_BDOOR_CMD_BATCHED_2M_UNLOCK;
-   } else {
-      cmd = BALLOON_BDOOR_CMD_BATCHED_UNLOCK;
-   }
-
-   status = BackdoorCmd(cmd, (size_t)ppn, nPages, target, &b->resetFlag);
+   int status = BackdoorCmd(BALLOON_BDOOR_CMD_BATCHED_UNLOCK,
+                            (size_t)ppn, nPages, target, &b->resetFlag);
 
    /* update stats */
-   STATS_INC(b->stats.unlock[isLargePage]);
+   STATS_INC(b->stats.unlock);
    if (status != BALLOON_SUCCESS) {
-      STATS_INC(b->stats.unlockFail[isLargePage]);
+      STATS_INC(b->stats.unlockFail);
    }
 
    return status;
