@@ -26,14 +26,53 @@
  *    http://www.jwz.org/doc/x-cut-and-paste.html
  */
 
+
+/*
+ * A Word on Selection Timestamps
+ *
+ * ICCCM §2.6.2 Target Atoms
+ *    The TIMESTAMP property is an INTEGER.
+ *
+ * ICCCM §2.7 Use of Selection Properties
+ *    The format of INTEGER is 32.
+ *
+ * XGetWindowProperty(3)
+ *    “If the returned format is 32, the property will be stored as an
+ *    array of longs (which in a 64-bit application will be 64-bit values
+ *    that are padded in the upper 4 bytes).”
+ *
+ * For all intents and purposes, on x86 and x86_64 X selection timestamps
+ * are a 32-bit quantity. (X11/Xproto.h's xSetSelectionOwnerReq defines the
+ * “time” member as the lower 32 bits of type Time.) X clients, on the
+ * other hand, operate on Time as either a CARD32 (uint32) or an unsigned
+ * long (i.e., on a 64-bit machine Time may occupy 8 bytes).
+ *
+ * Breaking it down:
+ *   · When Gtk+ provides an X11 selection via Gtk::SelectionData, on a
+ *     32-bit machine we'll have 4 bytes of raw data.  Everything's copacetic.
+ *   · On a 64-bit machine, even if the source client provides 32 bits of
+ *     timestamp data, Gtk+ will decode as an unsigned long and provide 8
+ *     bytes of raw data.
+ *   · On a 64-bit machine with a wacky application which actually tries
+ *     to record a full 64 bits of timestamp data, Gtk+ will provide 16 bytes:
+ *     <low 32 bits> <32 bits of 0s> <high 32 bits> <32 bits of 0s>.  (See
+ *     PR 882322, mrxvt.)
+ *
+ *   In all instances, we're interested in _only_ the lowest 32 bits, so we'll
+ *   ignore everything else.
+ */
+
+
 #define G_LOG_DOMAIN "dndcp"
 
 #include <sys/time.h>
 #include <time.h>
+#include <cxxabi.h>
 #include "copyPasteDnDWrapper.h"
 #include "copyPasteUIX11.h"
 #include "dndFileList.hh"
 #include "guestDnDCPMgr.hh"
+#include "tracer.hh"
 
 extern "C" {
    #include "vmblock.h"
@@ -44,9 +83,7 @@ extern "C" {
    #include "cpName.h"
    #include "cpNameUtil.h"
    #include "rpcout.h"
-   #include "eventManager.h"
    #include "vmware/guestrpc/tclodefs.h"
-   #include "guestApp.h"
 }
 
 /*
@@ -88,11 +125,15 @@ CopyPasteUIX11::CopyPasteUIX11()
  : mClipboardEmpty(true),
    mHGStagingDir(""),
    mIsClipboardOwner(false),
-   mHGGetFilesInitiated(false),
-   mFileTransferDone(false),
+   mClipTime(0),
+   mPrimTime(0),
+   mLastTimestamp(0),
+   mHGGetFileStatus(DND_FILE_TRANSFER_NOT_STARTED),
    mBlockAdded(false),
    mBlockCtrl(0),
-   mInited(false)
+   mInited(false),
+   mTotalFileSize(0),
+   mGetTimestampOnly(false)
 {
    GuestDnDCPMgr *p = GuestDnDCPMgr::GetInstance();
    ASSERT(p);
@@ -121,7 +162,7 @@ CopyPasteUIX11::CopyPasteUIX11()
 bool
 CopyPasteUIX11::Init()
 {
-   g_debug("%s: enter\n", __FUNCTION__);
+   TRACE_CALL();
    if (mInited) {
       return true;
    }
@@ -166,8 +207,30 @@ CopyPasteUIX11::Init()
 
 CopyPasteUIX11::~CopyPasteUIX11()
 {
-   g_debug("%s: enter\n", __FUNCTION__);
+   TRACE_CALL();
    CPClipboard_Destroy(&mClipboard);
+   /* Any files from last unfinished file transfer should be deleted. */
+   if (DND_FILE_TRANSFER_IN_PROGRESS == mHGGetFileStatus &&
+       !mHGStagingDir.empty()) {
+      uint64 totalSize = File_GetSizeEx(mHGStagingDir.c_str());
+      if (mTotalFileSize != totalSize) {
+         g_debug("%s: deleting %s, expecting %"FMT64"d, finished %"FMT64"d\n",
+                 __FUNCTION__, mHGStagingDir.c_str(),
+                 mTotalFileSize, totalSize);
+         DnD_DeleteStagingFiles(mHGStagingDir.c_str(), FALSE);
+      } else {
+         g_debug("%s: file size match %s\n",
+                 __FUNCTION__, mHGStagingDir.c_str());
+      }
+   }
+   if (mBlockAdded) {
+      g_debug("%s: removing block for %s\n", __FUNCTION__, mHGStagingDir.c_str());
+      /* We need to make sure block subsystem has not been shut off. */
+      if (DnD_BlockIsReady(mBlockCtrl)) {
+         mBlockCtrl->RemoveBlock(mBlockCtrl->fd, mHGStagingDir.c_str());
+      }
+      mBlockAdded = false;
+   }
 }
 
 
@@ -223,11 +286,7 @@ CopyPasteUIX11::GetLocalClipboard(void)
    if (mIsClipboardOwner) {
       /* If we are clipboard owner, send a not-changed clip to host. */
       g_debug("%s: we are owner, send unchanged clip back.\n", __FUNCTION__);
-      CPClipboard clip;
-      CPClipboard_Init(&clip);
-      CPClipboard_SetChanged(&clip, FALSE);
-      mCP->DestUISendClip(&clip);
-      CPClipboard_Destroy(&clip);
+      SendClipNotChanged();
       return;
    }
 
@@ -242,6 +301,7 @@ CopyPasteUIX11::GetLocalClipboard(void)
    mClipTime = 0;
    mPrimTime = 0;
    mGHSelection = GDK_SELECTION_CLIPBOARD;
+   mGetTimestampOnly = false;
    g_debug("%s: retrieving timestamps\n", __FUNCTION__);
    refClipboard->request_contents(TARGET_NAME_TIMESTAMP,
       sigc::mem_fun(this, &CopyPasteUIX11::LocalClipboardTimestampCB));
@@ -301,7 +361,6 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
                                       guint info)                    // IN:
 {
    g_debug("%s: enter.\n", __FUNCTION__);
-   mHGCopiedUriList = "";
    VmTimeType curTime;
 
    curTime = GetCurrentTime();
@@ -327,27 +386,22 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
    g_debug("%s: Got paste request, target is %s\n", __FUNCTION__,
          sd.get_target().c_str());
 
-   if (mHGGetFilesInitiated) {
+   if (mHGGetFileStatus != DND_FILE_TRANSFER_NOT_STARTED) {
       /*
        * On KDE (at least), we can see this multiple times, so ignore if
        * we are already getting files.
        */
-      g_debug("%s: already processing a GetFile request, returning.\n",
-              __FUNCTION__);
+      g_debug("%s: GetFiles already started, returning uriList [%s]\n",
+              __FUNCTION__, mHGCopiedUriList.c_str());
       sd.set(sd.get_target().c_str(), mHGCopiedUriList.c_str());
       return;
-   }
-
-
-   /* Copy the files. */
-   if (!mHGGetFilesInitiated) {
+   } else {
       utf::string str;
       utf::string hgStagingDir;
       utf::string stagingDirName;
       utf::string pre;
       utf::string post;
       size_t index = 0;
-      mFileTransferDone = false;
 
       hgStagingDir = static_cast<utf::string>(mCP->SrcUIRequestFiles());
       g_debug("%s: Getting files. Staging dir: %s", __FUNCTION__,
@@ -358,7 +412,7 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
          sd.set(sd.get_target().c_str(), "");
          return;
       }
-      mHGGetFilesInitiated = true;
+      mHGGetFileStatus = DND_FILE_TRANSFER_IN_PROGRESS;
 
       mBlockAdded = false;
       if (DnD_BlockIsReady(mBlockCtrl) && mBlockCtrl->AddBlock(mBlockCtrl->fd, hgStagingDir.c_str())) {
@@ -434,12 +488,9 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
        * apparently it only has so much patience regarding how quickly we
        * return.
        */
-      g_debug("%s no blocking driver, waiting for "
-            "HG file copy done ... mFileTransferDone is %d\n", __FUNCTION__,
-            (int) mFileTransferDone);
       CopyPasteDnDWrapper *wrapper = CopyPasteDnDWrapper::GetInstance();
       ToolsAppCtx *ctx = wrapper->GetToolsAppCtx();
-      while (mFileTransferDone == false) {
+      while (mHGGetFileStatus == DND_FILE_TRANSFER_IN_PROGRESS) {
          struct timeval tv;
 
          tv.tv_sec = 0;
@@ -567,16 +618,20 @@ void
 CopyPasteUIX11::LocalClipboardTimestampCB(const Gtk::SelectionData& sd)  // IN
 {
    int length = sd.get_length();
-   g_debug("%s: enter sd.get_length() %d.\n", __FUNCTION__,
-         length);
-   if (length == 4) {
-      mClipTime = ((uint32*) sd.get_data())[0];
-      g_debug("%s: mClipTime: %"FMT64"u.", __FUNCTION__, mClipTime);
-   } else if (length == 8) {
-      mClipTime = ((uint64*) sd.get_data())[0];
-      g_debug("%s: mClipTime: %"FMT64"u.", __FUNCTION__, mClipTime);
+
+   /*
+    * See “A Word on Selection Timestamps” above.
+    */
+   if (   (   sd.get_data_type().compare("INTEGER") == 0
+           || sd.get_data_type().compare("TIMESTAMP") == 0)
+       && sd.get_format() == 32
+       && length >= 4 /* sizeof uint32 */) {
+      mClipTime = reinterpret_cast<const uint32*>(sd.get_data())[0];
    } else {
-      g_debug("%s: Unable to get mClipTime.", __FUNCTION__);
+      g_debug("%s: Unable to get mClipTime (sd: len %d, type %s, fmt %d).",
+              __FUNCTION__, length,
+              length >= 0 ? sd.get_data_type().c_str() : "(n/a)",
+              sd.get_format());
    }
 
    Glib::RefPtr<Gtk::Clipboard> refClipboard
@@ -607,21 +662,44 @@ void
 CopyPasteUIX11::LocalPrimTimestampCB(const Gtk::SelectionData& sd)  // IN
 {
    int length = sd.get_length();
-   g_debug("%s: enter sd.get_length() is %d.\n", __FUNCTION__, length);
-   if (length == 4) {
-      mPrimTime = ((uint32*) sd.get_data())[0];
-      g_debug("%s: mPrimTime: %"FMT64"u.", __FUNCTION__, mPrimTime);
-   } else if (length == 8) {
-      mPrimTime = ((uint64*) sd.get_data())[0];
-      g_debug("%s: mPrimTime: %"FMT64"u.", __FUNCTION__, mPrimTime);
+
+   /*
+    * See “A Word on Selection Timestamps” above.
+    */
+   if (   (   sd.get_data_type().compare("INTEGER") == 0
+           || sd.get_data_type().compare("TIMESTAMP") == 0)
+       && sd.get_format() == 32
+       && length >= 4 /* sizeof uint32 */) {
+      mPrimTime = reinterpret_cast<const uint32*>(sd.get_data())[0];
    } else {
-      g_debug("%s: Unable to get mPrimTime.", __FUNCTION__);
+      g_debug("%s: Unable to get mPrimTime (sd: len %d, type %s, fmt %d).",
+              __FUNCTION__, length,
+              length >= 0 ? sd.get_data_type().c_str() : "(n/a)",
+              sd.get_format());
+   }
+
+   if (mGetTimestampOnly) {
+      mLastTimestamp = mClipTime > mPrimTime ? mClipTime : mPrimTime;
+      return;
    }
 
    /* After got both timestamp, choose latest one as active selection. */
-   mGHSelection = GDK_SELECTION_PRIMARY;
    if (mClipTime > mPrimTime) {
       mGHSelection = GDK_SELECTION_CLIPBOARD;
+      if (mClipTime > 0 && mClipTime == mLastTimestamp) {
+         g_debug("%s: clip is not changed\n", __FUNCTION__);
+         SendClipNotChanged();
+         return;
+      }
+      mLastTimestamp = mClipTime;
+   } else {
+      mGHSelection = GDK_SELECTION_PRIMARY;
+      if (mPrimTime > 0 && mPrimTime == mLastTimestamp) {
+         g_debug("%s: clip is not changed\n", __FUNCTION__);
+         SendClipNotChanged();
+         return;
+      }
+      mLastTimestamp = mPrimTime;
    }
 
    Glib::RefPtr<Gtk::Clipboard> refClipboard;
@@ -1093,7 +1171,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    void *buf;
    size_t sz;
 
-   g_debug("%s: enter\n", __FUNCTION__);
+   TRACE_CALL();
    if (!clip) {
       g_debug("%s: No clipboard contents.", __FUNCTION__);
       return;
@@ -1158,21 +1236,41 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    if (CPClipboard_GetItem(clip, CPFORMAT_IMG_PNG, &buf, &sz)) {
       g_debug("%s: PNG data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
       /* Try to load buf into pixbuf, and write to local clipboard. */
-      Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();
-
-      if (loader) {
+      try {
+         Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();
          loader->write((const guint8 *)buf, sz);
          loader->close();
 
          refClipboard->set_image(loader->get_pixbuf());
          refPrimary->set_image(loader->get_pixbuf());
+
+         /*
+          * Record current clipboard timestamp to prevent unexpected clipboard
+          * exchange.
+          *
+          * XXX We should do this for all formats.
+          */
+         mClipTime = 0;
+         mPrimTime = 0;
+         mGetTimestampOnly = true;
+         refClipboard->request_contents(TARGET_NAME_TIMESTAMP,
+            sigc::mem_fun(this, &CopyPasteUIX11::LocalClipboardTimestampCB));
+      } catch (const Gdk::PixbufError& e) {
+         g_message("%s: caught Gdk::PixbufError %s\n", __FUNCTION__, e.what().c_str());
+      } catch (std::exception& e) {
+         g_message("%s: caught std::exception %s\n", __FUNCTION__, e.what());
+      } catch (...) {
+         g_message("%s: caught unknown exception (typename %s)\n", __FUNCTION__,
+                   __cxxabiv1::__cxa_current_exception_type()->name());
       }
       return;
    }
+
    if (CPClipboard_GetItem(clip, CPFORMAT_FILELIST, &buf, &sz)) {
       g_debug("%s: File data.\n", __FUNCTION__);
       DnDFileList flist;
       flist.FromCPClipboard(buf, sz);
+      mTotalFileSize = flist.GetFileSize();
       mHGFCPData = flist.GetRelPathsStr();
 
       refClipboard->set(mListTargets,
@@ -1184,7 +1282,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
 
       mIsClipboardOwner = true;
       mHGGetListTime = GetCurrentTime();
-      mHGGetFilesInitiated = false;
+      mHGGetFileStatus = DND_FILE_TRANSFER_NOT_STARTED;
       mHGCopiedUriList = "";
    }
 
@@ -1431,7 +1529,7 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
       mBlockAdded = false;
    }
 
-   mFileTransferDone = true;
+   mHGGetFileStatus = DND_FILE_TRANSFER_FINISHED;
    if (success) {
       /*
        * Mark current staging dir to be deleted on next reboot for FCP. The
@@ -1443,7 +1541,35 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
       /* Copied files are already removed in common layer. */
       mHGStagingDir.clear();
    }
-   mHGGetFilesInitiated = false;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * CopyPasteUIX11::SendClipNotChanged --
+ *
+ *    Send a not-changed clip to host.
+ *
+ * Results:
+ *    None.
+ *
+ * Side effects:
+ *    None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+CopyPasteUIX11::SendClipNotChanged(void)
+{
+   CPClipboard clip;
+
+   g_debug("%s: enter.\n", __FUNCTION__);
+   CPClipboard_Init(&clip);
+   CPClipboard_SetChanged(&clip, FALSE);
+   mCP->DestUISendClip(&clip);
+   CPClipboard_Destroy(&clip);
 }
 
 
@@ -1465,6 +1591,6 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
 void
 CopyPasteUIX11::Reset(void)
 {
-   g_debug("%s: enter\n", __FUNCTION__);
+   TRACE_CALL();
    /* Cancel any pending file transfer. */
 }
