@@ -46,7 +46,17 @@ extern "C" {
 
 #include "guestCopyPaste.hh"
 
+/*
+ * Make sure exception types are public and therefore shared between libg*mm
+ * and this plugin. 
+ *
+ * See
+ * http://gcc.gnu.org/wiki/Visibility#Problems_with_C.2B-.2B-_exceptions_.28please_read.21.29
+ */
+#pragma GCC visibility push(default)
 #include <gtkmm.h>
+#pragma GCC visibility pop
+
 #include <list>
 #include <vector>
 
@@ -87,6 +97,7 @@ private:
    void LocalPrimTimestampCB(const Gtk::SelectionData& sd);
    void LocalReceivedFileListCB(const Gtk::SelectionData& selection_data);
    void GetLocalFilesDone(bool success);
+   void SendClipNotChanged(void);
 
    /* Conversion methods. */
    utf::utf8string GetNextPath(utf::utf8string &str, size_t& index);
@@ -103,21 +114,23 @@ private:
    bool mIsClipboardOwner;
    uint64 mClipTime;
    uint64 mPrimTime;
+   uint64 mLastTimestamp;
    GdkAtom mGHSelection;
    CPClipboard mClipboard;
 
    /* File vars. */
-   bool mHGGetFilesInitiated;
    VmTimeType mHGGetListTime;
    utf::string mHGCopiedUriList;
    utf::utf8string mHGFCPData;
    utf::string mHGTextData;
    std::string mHGRTFData;
    std::vector<utf::string> mHGFileContentsList;
-   bool mFileTransferDone;
+   DND_FILE_TRANSFER_STATUS mHGGetFileStatus;
    bool mBlockAdded;
    DnDBlockControl *mBlockCtrl;
    bool mInited;
+   uint64 mTotalFileSize;
+   bool mGetTimestampOnly;
 };
 
 #endif // __COPYPASTE_UI_X11_H__
