@@ -54,6 +54,14 @@
 #ifdef sun
 #define ACCESSPERMS           (S_IRWXU | S_IRWXG | S_IRWXO)
 #endif
+#ifdef __ANDROID__
+/*
+ * Android doesn't support setmntent(), endmntent() or MOUNTED.
+ */
+#define NO_SETMNTENT
+#define NO_ENDMNTENT
+#define ACCESSPERMS           (S_IRWXU | S_IRWXG | S_IRWXO)
+#endif
 
 
 /*
@@ -326,6 +334,8 @@ Bool
 DnD_AddBlockLegacy(int blockFd,                    // IN
                    const char *blockPath)          // IN
 {
+   LOG(1, ("%s: placing block on %s\n", __func__, blockPath));
+
    ASSERT(blockFd >= 0);
 
    if (VMBLOCK_CONTROL(blockFd, VMBLOCK_ADD_FILEBLOCK, blockPath) != 0) {
@@ -359,6 +369,8 @@ Bool
 DnD_RemoveBlockLegacy(int blockFd,                    // IN
                       const char *blockedPath)        // IN
 {
+   LOG(1, ("%s: removing block on %s\n", __func__, blockedPath));
+
    if (blockFd >= 0) {
       if (VMBLOCK_CONTROL(blockFd, VMBLOCK_DEL_FILEBLOCK, blockedPath) != 0) {
          Log("%s: Cannot delete block on %s (%s)\n",
@@ -421,6 +433,8 @@ Bool
 DnD_AddBlockFuse(int blockFd,                    // IN
                  const char *blockPath)          // IN
 {
+   LOG(1, ("%s: placing block on %s\n", __func__, blockPath));
+
    ASSERT(blockFd >= 0);
 
    if (VMBLOCK_CONTROL_FUSE(blockFd, VMBLOCK_FUSE_ADD_FILEBLOCK,
@@ -455,6 +469,8 @@ Bool
 DnD_RemoveBlockFuse(int blockFd,                    // IN
                     const char *blockedPath)        // IN
 {
+   LOG(1, ("%s: removing block on %s\n", __func__, blockedPath));
+
    if (blockFd >= 0) {
       if (VMBLOCK_CONTROL_FUSE(blockFd, VMBLOCK_FUSE_DEL_FILEBLOCK,
                                blockedPath) != 0) {
@@ -504,8 +520,11 @@ DnD_CheckBlockFuse(int blockFd)                    // IN
    }
 
    if (size != sizeof(VMBLOCK_FUSE_READ_RESPONSE)) {
-      LOG(4, ("%s: Response too short (%"FMTSZ"d vs. %"FMTSZ"u).\n",
-              __func__, size, sizeof(VMBLOCK_FUSE_READ_RESPONSE)));
+      /*
+       * Refer to bug 817761 of casting size to size_t.
+       */
+      LOG(4, ("%s: Response too short (%"FMTSZ"u vs. %"FMTSZ"u).\n",
+              __func__, (size_t)size, sizeof(VMBLOCK_FUSE_READ_RESPONSE)));
 
       return FALSE;
    }
@@ -546,6 +565,11 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
                    mode_t vmbDeviceMode,           // IN
                    Bool (*verifyBlock)(int fd))    // IN
 {
+#if defined NO_SETMNTENT || defined NO_ENDMNTENT
+   NOT_IMPLEMENTED();
+   errno = ENOSYS;
+   return -1;
+#else
    Bool found = FALSE;
    int blockFd = -1;
    char *realMntPoint;
@@ -609,6 +633,7 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
 out:
    free(realMntPoint);
    return blockFd;
+#endif
 }
 
 
