@@ -297,12 +297,6 @@ string::string(const char *s,           // IN
  *
  *      Constructor.
  *
- *      XXX: When initializing mUstr, we do a deep copy of the string data
- *      instead of just calling mUstr(s). This is because Glib::ustring is very
- *      smart about sharing storage, and zero_clear is very dumb. Once we get
- *      rid of zero_clear and have a separate sensitive-string class, this can
- *      go back to being simple.
- *
  * Results:
  *      None.
  *
@@ -312,8 +306,8 @@ string::string(const char *s,           // IN
  *-----------------------------------------------------------------------------
  */
 
-string::string(const Glib::ustring &s)    // IN
-   : mUstr(s.c_str()),
+string::string(const Glib::ustring& s) // IN
+   : mUstr(s),
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
@@ -328,12 +322,6 @@ string::string(const Glib::ustring &s)    // IN
  *
  *      Copy constructor.
  *
- *      XXX: When initializing mUstr, we do a deep copy of the string data
- *      instead of just calling mUstr(s). This is because Glib::ustring is very
- *      smart about sharing storage, and zero_clear is very dumb. Once we get
- *      rid of zero_clear and have a separate sensitive-string class, this can
- *      go back to being simple.
- *
  * Results:
  *      None.
  *
@@ -343,8 +331,8 @@ string::string(const Glib::ustring &s)    // IN
  *-----------------------------------------------------------------------------
  */
 
-string::string(const string &s) // IN
-   : mUstr(s.mUstr.c_str()),
+string::string(const string& s) // IN
+   : mUstr(s.mUstr),
      mUtf16Cache(NULL),
      mUtf16Length(npos)
 {
@@ -627,30 +615,6 @@ string::empty()
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::isASCII --
- *
- *      Test if every character in the string is in the ASCII range.
- *
- * Results:
- *      true if all ASCII, otherwise false.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-bool
-string::isASCII()
-   const
-{
-   return mUstr.is_ascii();
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1095,7 +1059,7 @@ string::assign(const string &s) // IN
 
 string&
 string::insert(size_type i,      // IN
-               const string &s)  // IN
+               const string& s)  // IN
 {
    InvalidateCache();
    mUstr.insert(i, s.mUstr);
@@ -1114,6 +1078,16 @@ string::insert(size_type i,      // IN
 }
 
 
+string&
+string::insert(iterator p,    // IN
+               value_type uc) // IN
+{
+   InvalidateCache();
+   mUstr.insert(p, uc);
+   return *this;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1138,46 +1112,6 @@ string::clear()
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::zero_clear --
- *
- *      Zeroes and clears this string.
- *
- *      XXX: This is temporary until we have a separate string class for
- *      passwords.
- *
- * Results:
- *      None
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-string::zero_clear()
-{
-   if (mUtf16Cache != NULL) {
-      Util_ZeroFree(mUtf16Cache,
-                    Unicode_UTF16Strlen(mUtf16Cache) * sizeof *mUtf16Cache);
-      mUtf16Cache = NULL;
-   }
-
-   /*
-    * This is a best effort.  We aren't guaranteed that Glib::ustring doesn't
-    * leave behind any internal copies of the string.
-    */
-   if (mUstr.c_str() != mUstr.data()) {
-      Util_Zero(const_cast<char *>(mUstr.c_str()), mUstr.bytes());
-   }
-   Util_Zero(const_cast<char *>(mUstr.data()), mUstr.bytes());
-   mUstr.clear();
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1256,7 +1190,7 @@ string::erase(iterator pbegin,    // IN
 string&
 string::replace(size_type i,     // IN
                 size_type n,     // IN
-                const string &s) // IN
+                const string& s) // IN
 {
    InvalidateCache();
    mUstr.replace(i, n, s.mUstr);
@@ -1282,8 +1216,8 @@ string::replace(size_type i,     // IN
  */
 
 string&
-string::replace(const string &from, // IN
-                const string &to)   // IN
+string::replace(const string& from, // IN
+                const string& to)   // IN
 {
    size_type end;
    size_type start = 0;
@@ -2115,9 +2049,11 @@ string::end()
 bool
 Validate(const Glib::ustring& s) // IN
 {
-   bool isValid = s.validate();
+   bool isValid = Unicode_IsBufferValid(s.c_str(), s.bytes(),
+                                        STRING_ENCODING_UTF8);
    if (!isValid) {
-      char *escaped = Unicode_EscapeBuffer(s.c_str(), -1, STRING_ENCODING_UTF8);
+      char *escaped = Unicode_EscapeBuffer(s.c_str(), s.bytes(),
+                                           STRING_ENCODING_UTF8);
       Warning("Invalid UTF-8 string: \"%s\"\n", escaped);
       free(escaped);
    }
