@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2008-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2008-2013 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -29,7 +29,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
-#include <cstring>
 
 extern "C" {
 #include "vm_assert.h"
@@ -991,10 +990,6 @@ GetMonitorWorkArea(Glib::RefPtr<Gdk::Screen> screen,    // IN:
       bool haveStrut = false;
       NETWMStrutPartial strut = NETWMStrutPartial();
 
-      if (monitor != screen->get_monitor_at_window(gdkWindow)) {
-         continue;
-      }
-
       /*
        * The EWMH spec says that the new _NET_WM_STRUT_PARTIAL takes precedence
        * over the older _NET_WM_STRUT API.
