@@ -97,6 +97,7 @@
 #include "vmware/tools/utils.h"
 
 #if !defined(__APPLE__)
+#include "vm_version.h"
 #include "embed_version.h"
 #include "vmtoolsd_version.h"
 VM_EMBED_VERSION(VMTOOLSD_VERSION_STRING);
@@ -783,8 +784,12 @@ TimeSyncSetOption(gpointer src,
             return FALSE;
          }
 
-      } else if (!start && data->state == TIMESYNC_RUNNING) {
-         TimeSyncStopLoop(ctx, data);
+      } else if (!start) {
+         if (data->state == TIMESYNC_RUNNING) {
+            TimeSyncStopLoop(ctx, data);
+         } else {
+            data->state = TIMESYNC_STOPPED;
+         }
       }
 
    } else if (strcmp(option, TOOLSOPTION_SYNCTIME_SLEWCORRECTION) == 0) {
