@@ -45,7 +45,7 @@
  */
 
 static void
-ToolsCoreCheckReset(struct RpcChannel *chan,
+ToolsCoreCheckReset(RpcChannel *chan,
                     gboolean success,
                     gpointer _state)
 {
@@ -241,11 +241,11 @@ ToolsCore_InitRpc(ToolsServiceState *state)
        * XXX: this should be relaxed when we try to bring up a VMCI or TCP channel.
        */
       if (!state->ctx.isVMware) {
-         g_warning("The %s service needs to run inside a virtual machine.\n",
-                   state->name);
+         g_debug("The %s service needs to run inside a virtual machine.\n",
+                 state->name);
          state->ctx.rpc = NULL;
       } else {
-         state->ctx.rpc = BackdoorChannel_New();
+         state->ctx.rpc = RpcChannel_New();
       }
       app = ToolsCore_GetTcloName(state);
       if (app == NULL) {
